#include "ags_editor.h"
#include "editor/ags_toolbar.h"
#include "editor/ags_notebook.h"

GType ags_editor_get_type(void);
void ags_editor_class_init(AgsEditorClass *editor);
void ags_editor_init(AgsEditor *editor);
void ags_editor_destroy(GtkObject *object);
void ags_editor_show(GtkWidget *widget);

extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget *widget);

GtkStyle *editor_style;

GType
ags_editor_get_type(void)
{
  static GType editor_type = 0;

  if (!editor_type){
    static const GtkTypeInfo editor_info = {
      "AgsEditor\0",
      sizeof(AgsEditor), /* base_init */
      sizeof(AgsEditorClass), /* base_finalize */
      (GtkClassInitFunc) ags_editor_class_init,
      (GtkObjectInitFunc) ags_editor_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };
    editor_type = gtk_type_unique (GTK_TYPE_VBOX, &editor_info);
  }

  return (editor_type);
}

void
ags_editor_class_init(AgsEditorClass *editor)
{
  GtkObjectClass *object = (GtkObjectClass *) editor;
  GtkWidgetClass *widget = (GtkWidgetClass *) editor;

  object->destroy = ags_editor_destroy;

  widget->show = ags_editor_show;
}

void
ags_editor_init(AgsEditor *editor)
{
  GtkHPaned *paned;
  GtkTable *table;

  gtk_box_pack_start((GtkBox *) editor,
		     (GtkWidget *) ags_toolbar_new(),
		     FALSE, FALSE, 0);

  paned = (GtkHPaned *) gtk_hpaned_new();
  gtk_box_pack_start((GtkBox *) editor,
		     (GtkWidget *) paned,
		     FALSE, FALSE, 0);

  editor->index = (GtkVBox *) gtk_vbox_new(FALSE, 0);
  gtk_paned_add1((GtkPaned *) paned, (GtkWidget *) editor->index);

  table = (GtkTable *) gtk_table_new(3, 3, FALSE);
  gtk_paned_add2((GtkPaned *) paned, (GtkWidget *) table);

  gtk_table_attach(table, (GtkWidget *) ags_notebook_new(),
		   0, 1, 0, 1,
		   GTK_FILL, GTK_FILL,
		   0, 0);

  editor->drawing_area = (GtkDrawingArea *) gtk_drawing_area_new();
  gtk_table_attach(table, (GtkWidget *) editor->drawing_area,
		   0, 1, 1, 2,
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND,
		   0, 0);

  editor->vscrollbar = (GtkVScrollbar *) gtk_vscrollbar_new(g_object_new(GTK_TYPE_ADJUSTMENT, NULL));
  gtk_table_attach(table, (GtkWidget *) editor->vscrollbar,
		   1, 2, 1, 2,
		   GTK_FILL, GTK_FILL,
		   0, 0);

  editor->hscrollbar = (GtkHScrollbar *) gtk_hscrollbar_new(g_object_new(GTK_TYPE_ADJUSTMENT, NULL));
  gtk_table_attach(table, (GtkWidget *) editor->hscrollbar,
		   0, 1, 2, 3,
		   GTK_FILL, GTK_FILL,
		   0, 0);
}

void
ags_editor_destroy(GtkObject *object)
{
  AgsEditor *editor = (AgsEditor *) object;
  GList *list0, *list1;
  GList *list2, *list3;

  gtk_widget_destroy((GtkWidget *) editor->index);

  list0 = gtk_container_get_children((GtkContainer *) editor);
  list1 = gtk_container_get_children((GtkContainer *) list0->next->data);
  list2 = gtk_container_get_children((GtkContainer *) list1->data);

  while(list2 != NULL){
    list3 = list2->next;
    gtk_widget_destroy((GtkWidget *) list1->data);

    list2 = list3;
  }

  list2 = gtk_container_get_children((GtkContainer *) list1->next);
  gtk_widget_destroy((GtkWidget *) list1->data);

  while(list2 != NULL){
    list3 = list2->next;
    gtk_widget_destroy((GtkWidget *) list2->data);

    list2 = list3;
  }

  while(list0 != NULL){
    list1 = list0->next;
    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  if(gtk_container_destroy)
    gtk_container_destroy(object);
}

void
ags_editor_show(GtkWidget *widget)
{
  if(gtk_widget_real_show)
    gtk_widget_real_show(widget);

  gtk_widget_show_all(widget);
}

AgsEditor*
ags_editor_new()
{
  AgsEditor *editor;

  editor = (AgsEditor *) g_object_new(AGS_TYPE_EDITOR, NULL);

  return(editor);
}
