#include "ags_link_editor.h"
#include "ags_link_editor_callbacks.h"

#include "ags_window.h"
#include "ags_machine_editor.h"

GType ags_link_editor_get_type(void);
void ags_link_editor_class_init(AgsLinkEditorClass *link_editor);
void ags_link_editor_init(AgsLinkEditor *link_editor);
void ags_link_editor_destroy(GtkObject *object);
void ags_link_editor_show(GtkWidget *widget);
void ags_link_editor_connect(AgsLinkEditor *link_editor);

extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget *widget);

GType
ags_link_editor_get_type(void)
{
  static GType link_editor_type = 0;

  if (!link_editor_type){
    static const GtkTypeInfo link_editor_info = {
      "AgsLinkEditor\0",
      sizeof(AgsLinkEditor), /* base_init */
      sizeof(AgsLinkEditorClass), /* base_finalize */
      (GtkClassInitFunc) ags_link_editor_class_init,
      (GtkObjectInitFunc) ags_link_editor_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    link_editor_type = gtk_type_unique (GTK_TYPE_HBOX, &link_editor_info);
  }

  return (link_editor_type);
}

void
ags_link_editor_class_init(AgsLinkEditorClass *link_editor)
{
  GtkObjectClass *object = (GtkObjectClass *) link_editor;
  GtkWidgetClass *widget = (GtkWidgetClass *) link_editor;

  object->destroy = ags_link_editor_destroy;
  widget->show = ags_link_editor_show;
}

void
ags_link_editor_init(AgsLinkEditor *link_editor)
{
  GtkHBox *hbox;
  GtkMenu *menu;
  GtkMenuItem *item;

  gtk_box_pack_start((GtkBox *) link_editor,
		     (GtkWidget *) gtk_label_new(NULL),
		     FALSE, FALSE, 0);

  link_editor->option = (GtkOptionMenu *) gtk_option_menu_new();
  menu = (GtkMenu *) gtk_menu_new();

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("NULL \0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  gtk_option_menu_set_menu(link_editor->option, (GtkWidget *) menu);
  gtk_box_pack_start((GtkBox *) link_editor,
		     (GtkWidget *) link_editor->option,
		     FALSE, FALSE, 0);

  link_editor->spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, 0.0, 1.0);
  gtk_box_pack_start((GtkBox *) link_editor,
		     (GtkWidget *) link_editor->spin_button,
		     FALSE, FALSE, 0);
}

void
ags_link_editor_destroy(GtkObject *object)
{
  AgsLinkEditor *link_editor = (AgsLinkEditor *) object;
  GList *list0, *list1;

  list0 = gtk_container_get_children((GtkContainer *) link_editor->option->menu);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  gtk_widget_destroy((GtkWidget *) link_editor->option->menu);

  gtk_widget_destroy((GtkWidget *) link_editor->spin_button);
  gtk_widget_destroy((GtkWidget *) link_editor->option);

  if(gtk_container_destroy)
    gtk_container_destroy(object);
}

void
ags_link_editor_show(GtkWidget *widget)
{
  AgsLinkEditor *link_editor = (AgsLinkEditor *) widget;
  AgsWindow *window;
  AgsMachineEditor *machine_editor;
  GtkMenuItem *item;
  GList *list;
  GSList *slist, *start;

  machine_editor = (AgsMachineEditor *) gtk_widget_get_ancestor(widget, AGS_TYPE_MACHINE_EDITOR);

  list = gtk_container_get_children((GtkContainer *) GTK_WIDGET(machine_editor->machine)->parent);

  while(list != NULL){
    item = (GtkMenuItem *) gtk_menu_item_new_with_label(AGS_MACHINE(list->data)->name);
    gtk_menu_shell_append((GtkMenuShell *) link_editor->option->menu, (GtkWidget *) item);

    list = list->next;
  }

  if(gtk_widget_real_show)
    gtk_widget_real_show(widget);

  gtk_widget_show_all((GtkWidget *) link_editor->option);
  gtk_widget_show((GtkWidget *) link_editor->spin_button);
}

void
ags_link_editor_connect(AgsLinkEditor *link_editor)
{
  g_signal_connect(G_OBJECT(link_editor->option), "changed\0",
		   G_CALLBACK(ags_link_editor_option_changed_callback), (gpointer) link_editor);
}

AgsLinkEditor*
ags_link_editor_new()
{
  AgsLinkEditor *link_editor;

  link_editor = (AgsLinkEditor *) g_object_new(AGS_TYPE_LINK_EDITOR, NULL);

  return(link_editor);
}
