#include "ags_machine.h"
#include "ags_machine_callbacks.h"

#include "ags_window.h"

GType ags_machine_get_type(void);
void ags_machine_class_init(AgsMachineClass *machine);
void ags_machine_init(AgsMachine *machine);
void ags_machine_destroy(GtkObject *gobject);
void ags_machine_show(GtkWidget *widget);

/* popup */
GType ags_machine_popup_get_type (void);
void ags_machine_popup_class_init (AgsMachinePopupClass *popup_class);
void ags_machine_popup_init (AgsMachinePopup *popup);
void ags_machine_popup_connect (AgsMachinePopup *popup);

extern void (*gtk_handle_box_destroy)(GtkObject *object);
extern void (*gtk_handle_box_show)(GtkWidget *widget);
extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget);

extern void ags_file_read_machine(AgsFile *file, AgsMachine *machine);
extern void ags_file_write_machine(AgsFile *file, AgsMachine *machine);

GType
ags_machine_get_type(void)
{
  static GType machine_type = 0;

  if (!machine_type){
    static const GtkTypeInfo machine_info = {
      "AgsMachine\0",
      sizeof(AgsMachine), /* base_init */
      sizeof(AgsMachineClass), /* base_finalize */
      (GtkClassInitFunc) ags_machine_class_init,
      (GtkObjectInitFunc) ags_machine_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    machine_type = gtk_type_unique (GTK_TYPE_HANDLE_BOX, &machine_info);
  }

  return (machine_type);
}

void
ags_machine_class_init(AgsMachineClass *machine)
{
  GtkObjectClass *object = (GtkObjectClass *) machine;
  GtkWidgetClass *widget = (GtkWidgetClass *) machine;

  object->destroy = ags_machine_destroy;
  widget->show = ags_machine_show;

  machine->write_file = ags_file_write_machine;
}

void
ags_machine_init(AgsMachine *machine)
{
  GtkFrame *frame;

  machine->name = NULL;

  frame = (GtkFrame *) gtk_frame_new(NULL);
  gtk_container_add((GtkContainer *) machine, (GtkWidget *) frame);

  machine->audio = ags_audio_new();
  machine->audio->machine = (GtkWidget *) machine;
}

void
ags_machine_destroy(GtkObject *object)
{
  AgsMachine *machine = (AgsMachine *) object;
  GList *list0;

  ags_audio_destroy(machine->audio);

  list0 = gtk_container_get_children((GtkContainer *) machine);
  gtk_widget_destroy((GtkWidget *) list0->data);

  if(machine->name != NULL)
    free(machine->name);

  if(gtk_handle_box_destroy)
    gtk_handle_box_destroy(object);
}

void
ags_machine_show(GtkWidget *widget)
{
  AgsMachine *machine = (AgsMachine *) widget;
  AgsWindow *window;
  GtkFrame *frame;

  window = (AgsWindow *) gtk_widget_get_toplevel(widget);
  window->counter->everything++;

  frame = (GtkFrame *) gtk_container_get_children((GtkContainer *) machine)->data;
  gtk_frame_set_label(frame,
		      g_strconcat(G_OBJECT_TYPE_NAME(widget),
				  ": \0",
				  machine->name, NULL));

  if(gtk_handle_box_show)
    gtk_handle_box_show(widget);

  gtk_widget_show((GtkWidget *) frame);
}

void
ags_machine_connect(AgsMachine *machine)
{
  g_signal_connect (G_OBJECT (machine), "button_press_event\0",
                    G_CALLBACK (ags_machine_button_press_callback), (gpointer) machine);
}

AgsMachine*
ags_machine_new()
{
  AgsMachine *machine;

  machine = (AgsMachine *) g_object_new(AGS_TYPE_MACHINE, NULL);

  return(machine);
}

GType
ags_machine_popup_get_type (void)
{
  static GType machine_popup_type = 0;

  if(!machine_popup_type){
    static const GtkTypeInfo machine_popup_info ={
      "AgsMachinePopup\0",
      sizeof(AgsMachinePopup), /* base_init */
      sizeof(AgsMachinePopupClass), /* base_finalize */
      (GtkClassInitFunc) ags_machine_popup_class_init,
      (GtkObjectInitFunc) ags_machine_popup_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    machine_popup_type = gtk_type_unique (GTK_TYPE_MENU, &machine_popup_info);
  }

  return (machine_popup_type);
}

void
ags_machine_popup_class_init (AgsMachinePopupClass *popup_class)
{
}

void
ags_machine_popup_init (AgsMachinePopup *popup)
{
  GtkMenuItem *item;

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("move up\0");
  gtk_menu_shell_append((GtkMenuShell *) popup, (GtkWidget*) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("move down\0");
  gtk_menu_shell_append((GtkMenuShell *) popup, (GtkWidget*) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("hide\0");
  gtk_menu_shell_append((GtkMenuShell *) popup, (GtkWidget*) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("destroy\0");
  gtk_menu_shell_append((GtkMenuShell *) popup, (GtkWidget*) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("rename\0");
  gtk_menu_shell_append((GtkMenuShell *) popup, (GtkWidget*) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("properties\0");
  gtk_menu_shell_append((GtkMenuShell *) popup, (GtkWidget*) item);
}

void
ags_machine_popup_connect (AgsMachinePopup *popup)
{
  GList *list;

  list = gtk_container_get_children((GtkContainer *) popup);

  g_signal_connect((GObject*) list->data, "activate\0",
		   G_CALLBACK(ags_machine_popup_move_up_activate_callback), (gpointer) popup);
  list = list->next;

  g_signal_connect((GObject*) list->data, "activate\0",
		   G_CALLBACK(ags_machine_popup_move_down_activate_callback), (gpointer) popup);
  list = list->next;

  g_signal_connect((GObject*) list->data, "activate\0",
		   G_CALLBACK(ags_machine_popup_hide_activate_callback), (gpointer) popup);
  list = list->next;

  g_signal_connect((GObject*) list->data, "activate\0",
		   G_CALLBACK(ags_machine_popup_destroy_activate_callback), (gpointer) popup);
  list = list->next;

  g_signal_connect((GObject*) list->data, "activate\0",
		   G_CALLBACK(ags_machine_popup_rename_activate_callback), (gpointer) popup);
  list = list->next;

  g_signal_connect((GObject*) list->data, "activate\0",
		   G_CALLBACK(ags_machine_popup_properties_activate_callback), (gpointer) popup);
}

AgsMachinePopup*
ags_machine_popup_new(AgsMachine *machine)
{
  static AgsMachinePopup *popup = NULL;

  if(popup == NULL){
    popup = (AgsMachinePopup *) g_object_new(AGS_TYPE_MACHINE_POPUP, NULL);
    ags_machine_popup_connect(popup);
    gtk_widget_show_all((GtkWidget*) popup);
  }

  popup->machine = machine;

  return(popup);
}
