#include "ags_machine_editor.h"
#include "ags_machine_editor_callbacks.h"

#include "ags_pad_editor.h"
#include "ags_line_editor.h"

#include <gtk/gtkvbox.h>

GType ags_machine_editor_get_type(void);
void ags_machine_editor_class_init(AgsMachineEditorClass *machine_editor);
void ags_machine_editor_init(AgsMachineEditor *machine_editor);
void ags_machine_editor_destroy(GtkObject *object);
void ags_machine_editor_show(GtkWidget *widget);
void ags_machine_editor_connect(AgsMachineEditor *machine_editor);

extern void (*gtk_window_destroy)(GtkObject *object);
extern void (*gtk_window_show)(GtkWidget *widget);

GType
ags_machine_editor_get_type(void)
{
  static GType machine_editor_type = 0;

  if (!machine_editor_type){
    static const GtkTypeInfo machine_editor_info = {
      "AgsMachineEditor\0",
      sizeof(AgsMachineEditor), /* base_init */
      sizeof(AgsMachineEditorClass), /* base_finalize */
      (GtkClassInitFunc) ags_machine_editor_class_init,
      (GtkObjectInitFunc) ags_machine_editor_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    machine_editor_type = gtk_type_unique (GTK_TYPE_DIALOG, &machine_editor_info);
  }

  return (machine_editor_type);
}

void
ags_machine_editor_class_init(AgsMachineEditorClass *machine_editor)
{
  GtkWidgetClass *widget = (GtkWidgetClass *) machine_editor;

  widget->show = ags_machine_editor_show;
}

void
ags_machine_editor_init(AgsMachineEditor *machine_editor)
{
  GtkNotebook *notebook;
  GtkScrolledWindow *scrolled_window;

  machine_editor->notebook =
    notebook = (GtkNotebook *) gtk_notebook_new();
  gtk_box_pack_start((GtkBox *) machine_editor->dialog.vbox, (GtkWidget*) notebook, TRUE, TRUE, 0);

  machine_editor->output_scrolled_window =
    scrolled_window = (GtkScrolledWindow *) gtk_scrolled_window_new(NULL, NULL);
  gtk_notebook_append_page(notebook, (GtkWidget *) scrolled_window, (GtkWidget *) gtk_label_new("output\0"));

  machine_editor->output = (GtkVBox*) gtk_vbox_new(FALSE, 0);
  gtk_scrolled_window_add_with_viewport(scrolled_window, (GtkWidget *) machine_editor->output);

  machine_editor->input_scrolled_window =
    scrolled_window = (GtkScrolledWindow *) gtk_scrolled_window_new(NULL, NULL);
  gtk_notebook_append_page(notebook, (GtkWidget *) scrolled_window, (GtkWidget *) gtk_label_new("input\0"));

  machine_editor->input = (GtkVBox*) gtk_vbox_new(FALSE, 0);
  gtk_scrolled_window_add_with_viewport(scrolled_window, (GtkWidget *) machine_editor->input);

  gtk_box_pack_start((GtkBox *) machine_editor->dialog.action_area, gtk_button_new_from_stock(GTK_STOCK_ADD), FALSE, FALSE, 0);
  gtk_box_pack_start((GtkBox *) machine_editor->dialog.action_area, gtk_button_new_from_stock(GTK_STOCK_REMOVE), FALSE, FALSE, 0);

  gtk_dialog_add_action_widget((GtkDialog *) machine_editor,
			       gtk_button_new_from_stock(GTK_STOCK_OK),
			       GTK_RESPONSE_OK);

  gtk_dialog_add_action_widget((GtkDialog *) machine_editor,
			       gtk_button_new_from_stock(GTK_STOCK_CANCEL),
			       GTK_RESPONSE_CANCEL);

  machine_editor->add = NULL;
  machine_editor->remove = NULL;
}

void
ags_machine_editor_destroy(GtkObject *object)
{
  AgsMachineEditor *machine_editor = (AgsMachineEditor *) object;
  GList *list0, *list1;

  if(machine_editor->add != NULL)
    gtk_widget_destroy((GtkWidget *) machine_editor->add);

  if(machine_editor->remove != NULL)
    gtk_widget_destroy((GtkWidget *) machine_editor->remove);

  list0 = gtk_container_get_children((GtkContainer *) machine_editor->output);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1->next;
  }

  gtk_widget_destroy((GtkWidget *) machine_editor->output);
  gtk_widget_destroy((GtkWidget *) machine_editor->output_scrolled_window);

  list0 = gtk_container_get_children((GtkContainer *) machine_editor->input);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1->next;
  }

  gtk_widget_destroy((GtkWidget *) machine_editor->input);
  gtk_widget_destroy((GtkWidget *) machine_editor->input_scrolled_window);
  gtk_widget_destroy((GtkWidget *) machine_editor->notebook);

  list0 = gtk_container_get_children((GtkContainer *) machine_editor->dialog.action_area);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  if(gtk_window_destroy)
    gtk_window_destroy(object);
}

void
ags_machine_editor_show(GtkWidget *widget)
{
  AgsMachineEditor *machine_editor;
  AgsMachine *machine;
  AgsPadEditor *pad_editor;
  GtkVBox *vbox;
  GList *list;
  guint i, j;

  machine_editor = AGS_MACHINE_EDITOR(widget);
  machine = machine_editor->machine;

  vbox = machine_editor->output;

  for(i = 0; i < machine->audio->output_pads; i++){
    pad_editor = ags_pad_editor_new(machine->audio->audio_channels);
    gtk_label_set_label(pad_editor->label,
			g_strdup_printf("pad %d\0", i));
    gtk_box_pack_start((GtkBox *) vbox, (GtkWidget *) pad_editor,
		       FALSE, FALSE, 0);
  }

  vbox = machine_editor->input;

  for(i = 0; i < machine->audio->input_pads; i++){
    pad_editor = ags_pad_editor_new(machine->audio->audio_channels);
    gtk_label_set_label(pad_editor->label,
			g_strdup_printf("pad %d\0", i));
    gtk_box_pack_start((GtkBox *) vbox, (GtkWidget *) pad_editor,
		       FALSE, FALSE, 0);
   }

  if(gtk_window_show)
    gtk_window_show(widget);

  gtk_widget_show((GtkWidget *) machine_editor->notebook);
  gtk_widget_show((GtkWidget *) machine_editor->output_scrolled_window);
  gtk_widget_show((GtkWidget *) machine_editor->output);
  gtk_widget_show((GtkWidget *) machine_editor->input_scrolled_window);
  gtk_widget_show((GtkWidget *) machine_editor->input);

  list = gtk_container_get_children((GtkContainer *) machine_editor->output);

  while(list != NULL){
    gtk_widget_show(GTK_WIDGET(list->data));
    list = list->next;
  }

  list = gtk_container_get_children((GtkContainer *) machine_editor->input);

  while(list != NULL){
    gtk_widget_show(GTK_WIDGET(list->data));
    list = list->next;
  }

  list = gtk_container_get_children((GtkContainer *) machine_editor->dialog.action_area);

  while(list != NULL){
    gtk_widget_show((GtkWidget *) list->data);

    list = list->next;
  }
}

void
ags_machine_editor_connect(AgsMachineEditor *machine_editor)
{
  GList *list;

  g_signal_connect(G_OBJECT(machine_editor), "response\0",
		   G_CALLBACK(ags_machine_editor_response_callback), (gpointer) machine_editor);

  list = gtk_container_get_children((GtkContainer *) machine_editor->output);

  while(list != NULL){
    ags_pad_editor_connect((AgsPadEditor *) list->data);

    list = list->next; 
  }

  list = gtk_container_get_children((GtkContainer *) machine_editor->input);

  while(list != NULL){
    ags_pad_editor_connect(AGS_PAD_EDITOR(list->data));

    list = list->next; 
  }

  list = gtk_container_get_children((GtkContainer *) machine_editor->dialog.action_area);
  g_signal_connect(G_OBJECT(list->data), "clicked\0",
		   G_CALLBACK(ags_machine_editor_add_callback), (gpointer) machine_editor);

  list = list->next;
  g_signal_connect(G_OBJECT(list->data), "clicked\0",
		   G_CALLBACK(ags_machine_editor_remove_callback), (gpointer) machine_editor);
}

AgsMachineEditor*
ags_machine_editor_new()
{
  AgsMachineEditor *machine_editor;

  machine_editor = (AgsMachineEditor *) g_object_new(AGS_TYPE_MACHINE_EDITOR, NULL);

  return(machine_editor);
}
