#include "ags_menu_bar.h"
#include "ags_menu_bar_callbacks.h"

#include <glib/glist.h>
#include <gtk/gtkmenushell.h>

GType ags_menu_bar_get_type(void);
void ags_menu_bar_class_init(AgsMenuBarClass *menu_bar);
void ags_menu_bar_init(AgsMenuBar *menu_bar);
void ags_menu_bar_connect(AgsMenuBar *menu_bar);
void ags_menu_bar_destroy(GtkObject *object);
void ags_menu_bar_show(GtkWidget *widget);

extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget *widget);

GType
ags_menu_bar_get_type(void)
{
  static GType menu_bar_type = 0;

  if(!menu_bar_type){
    static const GtkTypeInfo menu_bar_info = {
      "AgsMenuBar\0",
      sizeof(AgsMenuBar), /* base_init */
      sizeof(AgsMenuBarClass), /* base_finalize */
      (GtkClassInitFunc) ags_menu_bar_class_init,
      (GtkObjectInitFunc) ags_menu_bar_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    menu_bar_type = gtk_type_unique (GTK_TYPE_MENU_BAR, &menu_bar_info);
  }

  return(menu_bar_type);
}

void
ags_menu_bar_class_init(AgsMenuBarClass *menu_bar)
{
  GtkObjectClass *object = (GtkObjectClass*) menu_bar;
  GtkWidgetClass *widget = (GtkWidgetClass*) menu_bar;

  object->destroy = ags_menu_bar_destroy;

  widget->show = ags_menu_bar_show;
}

void
ags_menu_bar_init(AgsMenuBar *menu_bar)
{
  GtkImageMenuItem *item;

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_FILE, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar, (GtkWidget*) item);

  menu_bar->file = (GtkMenu *) gtk_menu_new();
  gtk_menu_item_set_submenu((GtkMenuItem*) item, (GtkWidget*) menu_bar->file);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_OPEN, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->file, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->file, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE_AS, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->file, (GtkWidget*) item);

  gtk_menu_shell_append((GtkMenuShell*) menu_bar->file,
			(GtkWidget*) gtk_separator_menu_item_new());

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->file, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_EDIT, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar, (GtkWidget*) item);

  menu_bar->edit = (GtkMenu *) gtk_menu_new();
  gtk_menu_item_set_submenu((GtkMenuItem*) item, (GtkWidget*) menu_bar->edit);

  //  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_UNDO, NULL);
  //  gtk_menu_shell_append((GtkMenuShell*) menu, (GtkWidget*) item);

  //  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_REDO, NULL);
  //  gtk_menu_shell_append((GtkMenuShell*) menu, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->edit, (GtkWidget*) item);

  menu_bar->add = (GtkMenu *) gtk_menu_new();
  gtk_menu_item_set_submenu((GtkMenuItem*) item, (GtkWidget*) menu_bar->add);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_with_label("Panel\0");
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->add, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_with_label("Mixer\0");
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->add, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_with_label("Drum\0");
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->add, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_with_label("Matrix\0");
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->add, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_with_label("Synth\0");
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->add, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_with_label("FPlayer\0");
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->add, (GtkWidget*) item);

  item = (GtkImageMenuItem *) gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, NULL);
  gtk_menu_shell_append((GtkMenuShell*) menu_bar->edit, (GtkWidget*) item);
}

void
ags_menu_bar_connect(AgsMenuBar *menu_bar)
{
  GList *list0, *list1, *list2;

  list0 = GTK_MENU_SHELL(menu_bar)->children;

  list1 = gtk_container_get_children ((GtkContainer *) gtk_menu_item_get_submenu((GtkMenuItem *) list0->data));

  g_signal_connect (G_OBJECT (list1->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_open_callback), (gpointer) menu_bar);
  list1 = list1->next;

  g_signal_connect (G_OBJECT (list1->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_save_callback), (gpointer) menu_bar);
  list1 = list1->next;

  g_signal_connect (G_OBJECT (list1->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_save_as_callback), (gpointer) menu_bar);
  list1 = list1->next->next;

  g_signal_connect (G_OBJECT (list1->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_quit_callback), (gpointer) menu_bar);

  list0 = list0->next;
  list1 = gtk_container_get_children((GtkContainer *) gtk_menu_item_get_submenu((GtkMenuItem *) list0->data));

  g_signal_connect (G_OBJECT (list1->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_add_callback), (gpointer) menu_bar);
  list2 = gtk_container_get_children((GtkContainer *) gtk_menu_item_get_submenu((GtkMenuItem *) list1->data));
  list1 = list1->next;

  g_signal_connect (G_OBJECT (list2->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_add_panel_callback), (gpointer) menu_bar);
  list2 = list2->next;

  g_signal_connect (G_OBJECT (list2->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_add_mixer_callback), (gpointer) menu_bar);
  list2 = list2->next;

  g_signal_connect (G_OBJECT (list2->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_add_drum_callback), (gpointer) menu_bar);
  list2 = list2->next;

  g_signal_connect (G_OBJECT (list2->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_add_matrix_callback), (gpointer) menu_bar);
  list2 = list2->next;

  g_signal_connect (G_OBJECT (list2->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_add_synth_callback), (gpointer) menu_bar);
  list2 = list2->next;

  g_signal_connect (G_OBJECT (list2->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_add_ffplayer_callback), (gpointer) menu_bar);

  g_signal_connect (G_OBJECT (list1->data), "activate\0",
                    G_CALLBACK (ags_menu_bar_remove_callback), (gpointer) menu_bar);
}

void
ags_menu_bar_destroy(GtkObject *object)
{
  AgsMenuBar *menu_bar = (AgsMenuBar *) object;
  GList *list0, *list1;

  list0 = gtk_container_get_children((GtkContainer *) menu_bar->add);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  list0 = gtk_container_get_children((GtkContainer *) menu_bar->edit);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  list0 = gtk_container_get_children((GtkContainer *) menu_bar->file);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  list0 = gtk_container_get_children((GtkContainer *) menu_bar);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  if(gtk_container_destroy)
    gtk_container_destroy(object);
}

void
ags_menu_bar_show(GtkWidget *widget)
{
  if(gtk_widget_real_show)
    gtk_widget_real_show(widget);

  gtk_widget_show_all(widget);
}

AgsMenuBar*
ags_menu_bar_new()
{
  AgsMenuBar *menu_bar;

  menu_bar = (AgsMenuBar *) g_object_new(AGS_TYPE_MENU_BAR, NULL);

  return(menu_bar);
}
