#include "ags_navigation.h"
#include "ags_navigation_callbacks.h"

GType ags_navigation_get_type(void);
void ags_navigation_class_init(AgsNavigationClass *navigation);
void ags_navigation_init(AgsNavigation *navigation);
void ags_navigation_destroy(GtkObject *object);
void ags_navigation_show(GtkWidget *widget);
void ags_navigation_connect(AgsNavigation *navigation);

extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget *widget);

GType
ags_navigation_get_type(void)
{
  static GType navigation_type = 0;

  if (!navigation_type){
    static const GtkTypeInfo navigation_info = {
      "AgsNavigation\0",
      sizeof(AgsNavigation), /* base_init */
      sizeof(AgsNavigationClass), /* base_finalize */
      (GtkClassInitFunc) ags_navigation_class_init,
      (GtkObjectInitFunc) ags_navigation_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    navigation_type = gtk_type_unique (GTK_TYPE_HBOX, &navigation_info);
  }

  return (navigation_type);
}

void
ags_navigation_class_init(AgsNavigationClass *navigation)
{
  GtkObjectClass *object = (GtkObjectClass *) navigation;
  GtkWidgetClass *widget = (GtkWidgetClass *) navigation;

  object->destroy = ags_navigation_destroy;

  widget->show = ags_navigation_show;
}

void
ags_navigation_init(AgsNavigation *navigation)
{
  GtkHBox *hbox;
  GtkToggleButton *toggle;
  GtkLabel *label;
  GtkSpinButton *spin_button;
  GtkButton *button;
  GtkCheckButton *check;

  hbox = (GtkHBox *) gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start((GtkBox *) navigation, (GtkWidget *) hbox, FALSE, FALSE, 2);

  toggle = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_box_pack_start((GtkBox*) hbox, (GtkWidget *) toggle, FALSE, FALSE, 2);
  gtk_container_add((GtkContainer *) toggle,
		    (GtkWidget *) gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_NONE));

  label = (GtkLabel *) gtk_label_new("bpm\0");
  gtk_box_pack_start((GtkBox*) hbox, (GtkWidget *) label, FALSE, FALSE, 2);

  spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(1.0, 1000.0, 1.0);
  gtk_box_pack_start((GtkBox*) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 2);


  toggle = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_container_add((GtkContainer *) toggle, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_MEDIA_REWIND, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) toggle, FALSE, FALSE, 0);

  button = (GtkButton *) gtk_button_new();
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_MEDIA_PREVIOUS, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) button, FALSE, FALSE, 0);

  toggle = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_container_add((GtkContainer *) toggle, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) toggle, FALSE, FALSE, 0);

  toggle = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_container_add((GtkContainer *) toggle, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_MEDIA_STOP, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) toggle, FALSE, FALSE, 0);

  button = (GtkButton *) gtk_toggle_button_new();
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_MEDIA_NEXT, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) button, FALSE, FALSE, 0);

  toggle = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_container_add((GtkContainer *) toggle, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_MEDIA_FORWARD, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) toggle, FALSE, FALSE, 0);


  check = (GtkCheckButton *) gtk_check_button_new_with_label("loop\0");
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) check, FALSE, FALSE, 2);

  label = (GtkLabel *) gtk_label_new("position\0");
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) label, FALSE, FALSE, 2);

  spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, 65000.0, 1.0);
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 2);

  spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, 60.0, 1.0);
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 2);


  label = (GtkLabel *) gtk_label_new("duration\0");
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) label, FALSE, FALSE, 2);

  spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, 65000.0, 1.0);
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 2);

  spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, 60.0, 1.0);
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 2);


  hbox = (GtkHBox *) gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start((GtkBox *) navigation, (GtkWidget *) hbox, FALSE, FALSE, 2);

  label = (GtkLabel *) gtk_label_new("loop\0");
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) label, FALSE, FALSE, 2);

  spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, 65000.0, 1.0);
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 2);

  spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, 60.0, 1.0);
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 2);

  check = (GtkCheckButton *) gtk_check_button_new_with_label("raster\0");
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) check, FALSE, FALSE, 2);
}

void
ags_navigation_destroy(GtkObject *object)
{
  GList *list0, *list1;

  list0 = gtk_container_get_children((GtkContainer *) object);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  if(gtk_container_destroy)
    gtk_container_destroy(object);
}

void
ags_navigation_show(GtkWidget *widget)
{
  GList *list0, *list1;

  if(gtk_widget_real_show)
    gtk_widget_real_show(widget);

  list0 = gtk_container_get_children((GtkContainer *) widget);
  list0 = gtk_container_get_children((GtkContainer *) list0->data);
  list1 = gtk_container_get_children((GtkContainer *) list0->data);

  if(GTK_ARROW(list1->data)->arrow_type == GTK_ARROW_DOWN)
    gtk_widget_show_all(widget);
  else
    gtk_widget_show_all((GtkWidget *) list0->data);
}

void
ags_navigation_connect(AgsNavigation *navigation)
{
  GList *list0, *list1;

  list0 = gtk_container_get_children((GtkContainer *) navigation);
  list1 = gtk_container_get_children((GtkContainer *) list0->data);

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_expander_callback), (gpointer) navigation);
  list1 = list1->next->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_bpm_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_rewind_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_prev_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_play_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_stop_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_next_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_forward_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_loop_callback), (gpointer) navigation);
  list1 = list1->next->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_position_sec_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_position_min_callback), (gpointer) navigation);
  list1 = list1->next->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_duration_sec_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_duration_min_callback), (gpointer) navigation);
  list1 = list1->next;


  list1 = gtk_container_get_children((GtkContainer *) list0->next->data)->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_loop_left_sec_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_loop_left_min_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_loop_right_sec_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "value-changed\0",
		   G_CALLBACK(ags_navigation_loop_right_min_callback), (gpointer) navigation);
  list1 = list1->next;

  g_signal_connect((GObject *) list1->data, "clicked\0",
		   G_CALLBACK(ags_navigation_raster_callback), (gpointer) navigation);
  list1 = list1->next;
}

AgsNavigation*
ags_navigation_new()
{
  AgsNavigation *navigation;

  navigation = (AgsNavigation *) g_object_new(AGS_TYPE_NAVIGATION, NULL);

  return(navigation);
}
