#include "ags_pad.h"
#include "ags_pad_callbacks.h"

GType ags_pad_get_type(void);
void ags_pad_class_init(AgsPadClass *pad);
void ags_pad_init(AgsPad *pad);
void ags_pad_destroy(GtkObject *object);
void ags_pad_show(GtkWidget *widget);
void ags_pad_connect(AgsPad *pad);

extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget *widget);

GType
ags_pad_get_type(void)
{
  static GType pad_type = 0;

  if (!pad_type){
    static const GtkTypeInfo pad_info = {
      "AgsPad\0",
      sizeof(AgsPad), /* base_init */
      sizeof(AgsPadClass), /* base_finalize */
      (GtkClassInitFunc) ags_pad_class_init,
      (GtkObjectInitFunc) ags_pad_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };
    pad_type = gtk_type_unique (GTK_TYPE_VBOX, &pad_info);
  }

  return (pad_type);
}

void
ags_pad_class_init(AgsPadClass *pad)
{
  GtkObjectClass *object = (GtkObjectClass *) pad;
  GtkWidgetClass *widget = (GtkWidgetClass *) pad;

  object->destroy = ags_pad_destroy;

  widget->show = ags_pad_show;
}

void
ags_pad_init(AgsPad *pad)
{
  GtkMenu *menu;

  pad->option = (GtkOptionMenu *) gtk_option_menu_new();
  menu = (GtkMenu *) gtk_menu_new();

  gtk_option_menu_set_menu(pad->option, (GtkWidget *) menu);
  gtk_box_pack_start((GtkBox *) pad, (GtkWidget *) pad->option, FALSE, FALSE, 0);

  pad->hbox = (GtkHBox *) gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start((GtkBox *) pad, (GtkWidget *) pad->hbox, FALSE, FALSE, 0);
}

void
ags_pad_destroy(GtkObject *object)
{
  AgsPad *pad = (AgsPad *) object;
  GList *list0, *list1;

  list0 = gtk_container_get_children((GtkContainer *) pad->option->menu);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  gtk_widget_destroy((GtkWidget *) pad->option->menu);
  gtk_widget_destroy((GtkWidget *) pad->option);
  gtk_widget_destroy((GtkWidget *) pad->hbox);

  if(gtk_container_destroy)
    gtk_container_destroy(object);
}

void
ags_pad_show(GtkWidget *widget)
{
  AgsPad *pad = (AgsPad *) widget;

  pad->selected_line = (AgsLine *) pad->option->menu_item;

  if(gtk_widget_real_show)
    gtk_widget_real_show(widget);

  gtk_widget_show_all((GtkWidget *) pad->option);
  gtk_widget_show_all((GtkWidget *) pad->hbox);
}

void
ags_pad_connect(AgsPad *pad)
{
  g_signal_connect((GObject *) pad->option, "changed\0",
		   G_CALLBACK(ags_pad_option_changed_callback), (gpointer) pad);
}

AgsPad*
ags_pad_new()
{
  AgsPad *pad;

  pad = (AgsPad *) g_object_new(AGS_TYPE_PAD, NULL);

  return(pad);
}
