#include "ags_window.h"
#include "ags_window_callbacks.h"
#include "ags_menu_bar.h"

GType ags_window_get_type(void);
void ags_window_class_init(AgsWindowClass *window);
void ags_window_init(AgsWindow *window);
void ags_window_connect(AgsWindow *window);
void ags_window_destroy(GtkObject *object);
void ags_window_show(GtkWidget *widget);

extern void (*gtk_window_destroy)(GtkObject *object);
extern void (*gtk_window_show)(GtkWidget *widget);

GType
ags_window_get_type(void)
{
  static GType window_type = 0;

  if(!window_type){
    static const GtkTypeInfo window_info = {
      "AgsWindow\0",
      sizeof(AgsWindow), /* base_init */
      sizeof(AgsWindowClass), /* base_finalize */
      (GtkClassInitFunc) ags_window_class_init,
      (GtkObjectInitFunc) ags_window_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };
    window_type = gtk_type_unique (GTK_TYPE_WINDOW, &window_info);
  }

  return(window_type);
}

void
ags_window_class_init(AgsWindowClass *window)
{
  GtkObjectClass *object = (GtkObjectClass*) window;
  GtkWidgetClass *widget = (GtkWidgetClass*) window;

  object->destroy = ags_window_destroy;

  widget->show = ags_window_show;
}

void
ags_window_init(AgsWindow *window)
{
  GtkVBox *vbox;
  GtkVPaned *paned;
  GtkWidget *scrolled_window;

  vbox = (GtkVBox*) gtk_vbox_new(FALSE, 0);
  gtk_container_add((GtkContainer*) window, (GtkWidget*) vbox);

  gtk_box_pack_start((GtkBox*) vbox,
		     (GtkWidget*) ags_menu_bar_new(),
		     FALSE, FALSE, 0);

  paned = (GtkVPaned *) gtk_vpaned_new();
  gtk_box_pack_start((GtkBox*) vbox, (GtkWidget*) paned, TRUE, TRUE, 0);

  gtk_paned_add1((GtkPaned *) paned,
		 (scrolled_window = (GtkWidget *) gtk_scrolled_window_new(NULL, NULL)));

  window->machines = (GtkVBox *) gtk_vbox_new(FALSE, 0);
  gtk_scrolled_window_add_with_viewport((GtkScrolledWindow *) scrolled_window,
					(GtkWidget *) window->machines);

  gtk_paned_add2((GtkPaned *) paned,
		 (GtkWidget *) ags_editor_new());

  gtk_box_pack_start((GtkBox*) vbox,
		     (GtkWidget*) ags_navigation_new(),
		     FALSE, FALSE, 0);

  window->selected = NULL;
  window->counter = (AgsMachineCounter *) malloc(sizeof(AgsMachineCounter));
  window->counter->everything = 0;
  window->counter->panel = 0;
  window->counter->mixer = 0;
  window->counter->drum = 0;
  window->counter->matrix = 0;
  window->counter->synth = 0;
  window->counter->ffplayer = 0;
}

void
ags_window_destroy(GtkObject *object)
{
  AgsWindow *window;
  GList *list0, *list1;

  list0 = gtk_container_get_children((GtkContainer *) window->machines);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1->data;
  }

  list0 = gtk_container_get_children((GtkContainer *) window);

  list1 = list0->next;
  gtk_widget_destroy((GtkWidget *) list0->data);

  gtk_widget_destroy((GtkWidget *) window->machines);

  list0 = gtk_container_get_children((GtkContainer *) list1->data);
  gtk_widget_destroy((GtkWidget *) list0->data);

  list0 = list1->next;
  gtk_widget_destroy((GtkWidget *) list1->data);
  gtk_widget_destroy((GtkWidget *) list0->data);

  free(window->counter);

  if(gtk_window_destroy)
    gtk_window_destroy(object);
}

void
ags_window_show(GtkWidget *widget)
{
  if(gtk_window_show)
    gtk_window_show(widget);

  gtk_widget_show_all(widget);
}

void
ags_window_connect(AgsWindow *window)
{
  GList *list0, *list1;

  g_signal_connect(G_OBJECT(window), "delete_event\0",
		   G_CALLBACK(ags_window_delete_event_callback), (gpointer) window);

  list0 = gtk_container_get_children((GtkContainer*) window);
  list1 = gtk_container_get_children((GtkContainer*) list0->data);

  ags_menu_bar_connect((AgsMenuBar*) list1->data);
}

AgsWindow*
ags_window_new()
{
  AgsWindow *window;

  window = (AgsWindow *) g_object_new(AGS_TYPE_WINDOW, NULL);

  return(window);
}
