#include "ags_notebook.h"
#include "ags_notebook_callbacks.h"

#include "../../audio/ags_pattern.h"

#include <gdk/gdk.h>
#include <gtk/gtkdrawingarea.h>

GType ags_notebook_get_type(void);
void ags_notebook_class_init(AgsNotebookClass *notebook);
void ags_notebook_init(AgsNotebook *notebook);
void ags_notebook_destroy(GtkObject *object);
void ags_notebook_show(GtkWidget *widget);

void ags_notebook_draw_gutter(GtkDrawingArea *drawing_area, GdkGC *color0, GdkGC *color1, guint width, guint height, guint x0, guint x1, guint y0, guint y1, float size);
void ags_notebook_draw_pattern(GtkDrawingArea *drawing_area, GdkGC *color, AgsPattern *pattern, guint index0, guint index1, guint offset, guint width, guint height, guint x0, guint x1, guint y0, guint y1, float size);

extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget *widget);

GType
ags_notebook_get_type(void)
{
  static GType notebook_type = 0;

  if (!notebook_type){
    static const GtkTypeInfo notebook_info = {
      "AgsNotebook\0",
      sizeof(AgsNotebook), /* base_init */
      sizeof(AgsNotebookClass), /* base_finalize */
      (GtkClassInitFunc) ags_notebook_class_init,
      (GtkObjectInitFunc) ags_notebook_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    notebook_type = gtk_type_unique (GTK_TYPE_VBOX, &notebook_info);
  }

  return (notebook_type);
}

void
ags_notebook_class_init(AgsNotebookClass *notebook)
{
  GtkObjectClass *object = (GtkObjectClass *) notebook;
  GtkWidgetClass *widget = (GtkWidgetClass *) notebook;

  object->destroy = ags_notebook_destroy;

  widget->show = ags_notebook_show;
}

void
ags_notebook_init(AgsNotebook *notebook)
{
}

void
ags_notebook_destroy(GtkObject *object)
{
  if(gtk_container_destroy)
    gtk_container_destroy(object);
}

void
ags_notebook_show(GtkWidget *widget)
{
  if(gtk_widget_real_show)
    gtk_widget_real_show(widget);
}

void
ags_notebook_connect(AgsNotebook *notebook)
{
  g_signal_connect((GObject *) notebook, "switch-page\0",
		   G_CALLBACK(ags_notebook_switch_page_callback), (gpointer) notebook);
}

void
ags_notebook_draw_gutter(GtkDrawingArea *drawing_area, GdkGC *color0, GdkGC *color1, guint width, guint height, guint x0, guint x1, guint y0, guint y1, float size)
{
  GdkGC *color;
  guint width0, height0;
  guint i, j;
  gboolean change0, change1;

  width0 = (guint) width * size;
  height0 = (guint) height * size;

  for(j = 0, change1 = TRUE, i = 0; j < (width - x0 - x1) * size;){
    gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
			color,
			TRUE,
			j, i,
			width0, y0);

    j += width0;
    color = change1 ? color0: color1;
    change1 = ~change1;
  }

  gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
		      color,
		      TRUE,
		      j, i,
		      x1, y0);

  for(; i < (height - y0 - y1) * size;){
    gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
			color,
			TRUE,
			j, i,
			x1, height0);

    i += height0;
    color = change1 ? color0: color1;
    change1 = ~change1;
  }

  gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
		      color,
		      TRUE,
		      j, i,
		      x1, y1);

  for(; j > (width - x0 - x1) * size;){
    gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
			color,
			TRUE,
			j, i,
			width0, y1);

    j -= width0;
    color = change1 ? color0: color1;
    change1 = ~change1;
  }

  gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
		      color,
		      TRUE,
		      j, i,
		      x0, y1);

  for(; i > (height - y0 - y1) * size;){
    gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
			color,
			TRUE,
			j, i,
			x0, height0);

    i -= height0;
    color = change1 ? color0: color1;
    change1 = ~change1;
  }

  gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
		      color,
		      TRUE,
		      j, i,
		      x0, y0);

  for(; i < (height - y0 - y1) * size;){
    for(; j < (width - x0 - x1) * size;){
      gdk_draw_rectangle (GTK_WIDGET (drawing_area)->window,
			  color,
			  TRUE,
			  j * width0, i * height0,
			  width0, height0);

      j += width0;
      color = change1 ? color0: color1;
      change1 = ~change1;
    }

    i += height0;
    color = change0 ? color0: color1;
    change0 = ~change0;
    change1 = change0;
  }
}

void
ags_notebook_draw_pattern(GtkDrawingArea *drawing_area, GdkGC *color, AgsPattern *pattern, guint index0, guint index1, guint offset, guint width, guint height, guint x0, guint x1, guint y0, guint y1, float size)
{
}

AgsNotebook*
ags_notebook_new()
{
  AgsNotebook *notebook;

  notebook = (AgsNotebook *) g_object_new(AGS_TYPE_NOTEBOOK, NULL);

  return(notebook);
}
