#include "ags_toolbar.h"
#include "ags_toolbar_callbacks.h"

#include <gtk/gtkhbox.h>
#include <gtk/gtkbox.h>
#include <gtk/gtkcontainer.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtkcheckbutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkoptionmenu.h>
#include <gtk/gtkimage.h>
#include <gtk/gtkstock.h>

GType ags_toolbar_get_type(void);
void ags_toolbar_class_init(AgsToolbarClass *toolbar);
void ags_toolbar_init(AgsToolbar *toolbar);
void ags_toolbar_destroy(GtkObject *object);
void ags_toolbar_show(GtkWidget *widget);

extern void (*gtk_container_destroy)(GtkObject *object);
extern void (*gtk_widget_real_show)(GtkWidget *widget);

GType
ags_toolbar_get_type(void)
{
  static GType toolbar_type = 0;

  if (!toolbar_type){
    static const GtkTypeInfo toolbar_info = {
      "AgsToolbar\0",
      sizeof(AgsToolbar), /* base_init */
      sizeof(AgsToolbarClass), /* base_finalize */
      (GtkClassInitFunc) ags_toolbar_class_init,
      (GtkObjectInitFunc) ags_toolbar_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    toolbar_type = gtk_type_unique (GTK_TYPE_VBOX, &toolbar_info);
  }

  return (toolbar_type);
}

void
ags_toolbar_class_init(AgsToolbarClass *toolbar)
{
  GtkObjectClass *object = (GtkObjectClass *) toolbar;
  GtkWidgetClass *widget = (GtkWidgetClass *) toolbar;
}

void
ags_toolbar_init(AgsToolbar *toolbar)
{
  GtkToggleButton *button;
  GtkLabel *label;
  GtkOptionMenu *option;

  button = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_button_set_relief((GtkButton *) button, GTK_RELIEF_NONE);
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) button, "edit notes\0", NULL);

  button = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_button_set_relief((GtkButton *) button, GTK_RELIEF_NONE);
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) button, "clear notes\0", NULL);

  button = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_button_set_relief((GtkButton *) button, GTK_RELIEF_NONE);
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_SELECT_ALL, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) button, "select notes\0", NULL);

  button = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_button_set_relief((GtkButton *) button, GTK_RELIEF_NONE);
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) button, "copy notes\0", NULL);

  button = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_button_set_relief((GtkButton *) button, GTK_RELIEF_NONE);
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_CUT, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) button, "cut notes\0", NULL);

  button = (GtkToggleButton *) gtk_toggle_button_new();
  gtk_button_set_relief((GtkButton *) button, GTK_RELIEF_NONE);
  gtk_container_add((GtkContainer *) button, (GtkWidget *) gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_LARGE_TOOLBAR));
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) button, "paste notes\0", NULL);

  label = (GtkLabel *) gtk_label_new("zoom\0");
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) label, NULL, NULL);

  option = (GtkOptionMenu *) gtk_option_menu_new();
  gtk_option_menu_set_menu(option, (GtkWidget *) ags_zoom_menu_new());
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) option, NULL , NULL);

  label = (GtkLabel *) gtk_label_new("tic\0");
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) label, NULL, NULL);

  option = (GtkOptionMenu *) gtk_option_menu_new();
  gtk_option_menu_set_menu(option, (GtkWidget *) ags_tic_menu_new());
  gtk_toolbar_append_widget((GtkToolbar *) toolbar, (GtkWidget *) option, NULL, NULL);
}

void
ags_toolbar_destroy(GtkObject *object)
{
  GtkOptionMenu *option;
  GList *list0, *list1;
  guint i;
  void ags_toolbar_destroy_option_menu(){
    GList *list2, *list3;

    option = (GtkOptionMenu *) list0->data;
    list2 = gtk_container_get_children((GtkContainer *) option->menu);

    while(list2 != NULL){
      list3 = list2->next;

      gtk_widget_destroy((GtkWidget *) list2->data);

      list2 = list3;
    }

    gtk_widget_destroy((GtkWidget *) option->menu);
  }

  list0 = gtk_container_get_children((GtkContainer *) object);

  for(i = 0; i < 7; i++){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  list1 = list0->next;
  ags_toolbar_destroy_option_menu();

  list0 = list1;
  list1 = list0->next;

  gtk_widget_destroy((GtkWidget *) list0->data);

  ags_toolbar_destroy_option_menu();

  if(gtk_container_destroy)
    gtk_container_destroy(object);
}

void
ags_toolbar_show(GtkWidget *widget)
{
  if(gtk_widget_real_show)
    gtk_widget_real_show(widget);

  gtk_widget_show_all((GtkWidget *) widget);
}

void
ags_toolbar_connect(AgsToolbar *toolbar)
{
  GList *list;

  list = gtk_container_get_children((GtkContainer *) toolbar);

  g_signal_connect((GObject *) list->data, "clicked\0",
		   G_CALLBACK(ags_toolbar_edit_callback), (gpointer) toolbar);

  list = list->next;
  g_signal_connect((GObject *) list->data, "clicked\0",
		   G_CALLBACK(ags_toolbar_clear_callback), (gpointer) toolbar);

  list = list->next;
  g_signal_connect((GObject *) list->data, "clicked\0",
		   G_CALLBACK(ags_toolbar_select_callback), (gpointer) toolbar);

  list = list->next;
  g_signal_connect((GObject *) list->data, "clicked\0",
		   G_CALLBACK(ags_toolbar_copy_callback), (gpointer) toolbar);

  list = list->next;
  g_signal_connect((GObject *) list->data, "clicked\0",
		   G_CALLBACK(ags_toolbar_cut_callback), (gpointer) toolbar);

  list = list->next;
  g_signal_connect((GObject *) list->data, "clicked\0",
		   G_CALLBACK(ags_toolbar_paste_callback), (gpointer) toolbar);

  list = list->next->next;
  g_signal_connect((GObject *) list->data, "changed\0",
		   G_CALLBACK(ags_toolbar_zoom_callback), (gpointer) toolbar);

  list = list->next->next;
  g_signal_connect((GObject *) list->data, "changed\0",
		   G_CALLBACK(ags_toolbar_tic_callback), (gpointer) toolbar);
}

AgsToolbar*
ags_toolbar_new()
{
  AgsToolbar *toolbar;

  toolbar = (AgsToolbar *) g_object_new(AGS_TYPE_TOOLBAR, NULL);

  return(toolbar);
}

GtkMenu*
ags_zoom_menu_new()
{
  GtkMenu *menu;
  GtkMenuItem *item;

  menu = (GtkMenu *) gtk_menu_new();

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("16:1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("8:1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("4:1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("2:1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);


  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1:1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);


  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1:2\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1:4\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1:8\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1:16\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  return(menu);
}

GtkMenu*
ags_tic_menu_new()
{
  GtkMenu *menu;
  GtkMenuItem *item;

  menu = (GtkMenu *) gtk_menu_new();

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("16/1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("8/1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("4/1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("2/1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);


  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1:1\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);


  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1/2\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1/4\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1/8\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  item = (GtkMenuItem *) gtk_menu_item_new_with_label("1/16\0");
  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) item);

  return(menu);
}
