#include "ags_drum.h"
#include "ags_drum_callbacks.h"

#include "../ags_window.h"
#include "../ags_pad.h"
#include "../ags_line.h"

#include "../../audio/ags_audio.h"
#include "../../audio/ags_channel.h"
#include "../../audio/ags_input.h"
#include "../../audio/ags_output.h"
#include "../../audio/ags_pattern.h"

GType ags_drum_get_type(void);
void ags_drum_class_init(AgsDrumClass *drum);
void ags_drum_init(AgsDrum *drum);
void ags_drum_destroy(GtkObject *object);
void ags_drum_show(GtkWidget *widget);
void ags_drum_set_audio_channels(AgsAudio *audio, guint audio_channels);
void ags_drum_set_pads(AgsAudio *audio, GType type, guint pads);
void ags_drum_set_lines(AgsAudio *audio, GType type, guint lines);

extern void ags_file_read_drum(AgsFile *file, AgsMachine *machine);
extern void ags_file_write_drum(AgsFile *file, AgsMachine *machine);

GType
ags_drum_get_type(void)
{
  static GType drum_type = 0;

  if (!drum_type){
    static const GtkTypeInfo drum_info = {
      "AgsDrum\0",
      sizeof(AgsDrum), /* base_init */
      sizeof(AgsDrumClass), /* base_finalize */
      (GtkClassInitFunc) ags_drum_class_init,
      (GtkObjectInitFunc) ags_drum_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };
    drum_type = gtk_type_unique (AGS_TYPE_MACHINE, &drum_info);
  }

  return (drum_type);
}

void
ags_drum_class_init(AgsDrumClass *drum)
{
  GtkObjectClass *object = (GtkObjectClass *) drum;
  GtkWidgetClass *widget = (GtkWidgetClass *) drum;
  AgsMachineClass *machine = (AgsMachineClass *) drum;

  object->destroy = ags_drum_destroy;

  widget->show = ags_drum_show;

  machine->read_file = ags_file_read_drum;
  machine->write_file = ags_file_write_drum;
}

void
ags_drum_init(AgsDrum *drum)
{
  GtkVBox *vbox;
  GtkHBox *hbox;
  GtkToggleButton *toggle_button;
  GList *list;
  GtkFrame *frame;
  GtkTable *table0, *table1;
  GtkRadioButton *radio_button;
  int i, j;

  drum->flags = 0;

  drum->vbox = (GtkVBox *) gtk_vbox_new(FALSE, 0);
  gtk_container_add((GtkContainer*) (gtk_container_get_children((GtkContainer *) drum))->data, (GtkWidget *) drum->vbox);

  drum->pad = (GtkHBox *) gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start((GtkBox *) drum->vbox, (GtkWidget *) drum->pad, FALSE, FALSE, 0);

  drum->selected_pad = NULL;
  drum->edit_button = NULL;

  drum->machine.audio->flags |= AGS_AUDIO_OUTPUT_HAS_RECYCLING;

  AGS_AUDIO_GET_CLASS(drum->machine.audio)->set_audio_channels = ags_drum_set_audio_channels;
  AGS_AUDIO_GET_CLASS(drum->machine.audio)->set_pads = ags_drum_set_pads;
  AGS_AUDIO_GET_CLASS(drum->machine.audio)->set_lines = ags_drum_set_lines;

  drum->machine.audio->audio_channels = 2;
  ags_audio_set_pads(drum->machine.audio, AGS_TYPE_OUTPUT, 1);
  ags_audio_set_pads(drum->machine.audio, AGS_TYPE_INPUT, 8);

  hbox = (GtkHBox *) gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start((GtkBox *) drum->vbox, (GtkWidget *) hbox, FALSE, FALSE, 0);

  frame = (GtkFrame *) gtk_frame_new("kit\0");
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) frame, FALSE, FALSE, 0);

  vbox = (GtkVBox *) gtk_vbox_new(FALSE, 0);
  gtk_container_add((GtkContainer*) frame, (GtkWidget *) vbox);

  gtk_box_pack_start((GtkBox *) vbox,
		     (GtkWidget *) gtk_label_new("default\0"), 
		     FALSE, FALSE, 0);
 
  gtk_box_pack_start((GtkBox *) vbox,
		     (GtkWidget *) gtk_button_new_from_stock(GTK_STOCK_OPEN), 
		     FALSE, FALSE, 0);

  frame = (GtkFrame *) gtk_frame_new("Pattern\0");
  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) frame, FALSE, FALSE, 0);

  table0 = (GtkTable *) gtk_table_new(16, 3, FALSE);
  gtk_container_add((GtkContainer*) frame, (GtkWidget *) table0);

  gtk_table_attach_defaults(table0,
			    (GtkWidget *) gtk_toggle_button_new_with_label("run\0"),
			    0, 1, 1, 2);

  table1 = (GtkTable *) gtk_table_new(3, 5, TRUE);
  gtk_table_attach_defaults(table0,
			    (GtkWidget *) table1,
			    1, 2, 0, 2);

  drum->selected1 = NULL;

  for(i = 0; i < 3; i++)
    for(j = 0; j < 4; j++)
      gtk_table_attach_defaults(table1,
				(GtkWidget *) (drum->index1[i * 4 + j] = (GtkToggleButton *) gtk_toggle_button_new()),
				j, j +1, i, i +1);

  drum->selected1 = drum->index1[0];
  gtk_toggle_button_set_active(drum->index1[0], TRUE);

  drum->selected0 = NULL;

  for(j = 0; j < 4; j++)
    gtk_table_attach_defaults(table1,
			      (GtkWidget *) (drum->index0[j] = (GtkToggleButton *) gtk_toggle_button_new()),
			      j, j +1, 4, 5);

  drum->selected0 = drum->index0[0];
  gtk_toggle_button_set_active(drum->index0[0], TRUE);

  gtk_table_attach(table0,
		   (GtkWidget *) gtk_label_new("lenght\0"),
		   4, 5, 0, 1,
		   GTK_EXPAND, GTK_EXPAND,
		   0, 0);
  gtk_table_attach(table0,
		   (GtkWidget *) gtk_spin_button_new_with_range(0.0, 64.0, 1.0),
		   5, 6, 0, 1,
		   GTK_EXPAND, GTK_EXPAND,
		   0, 0);

  drum->block_pattern = FALSE;
  drum->pattern =
    hbox = (GtkHBox *) gtk_hbox_new(FALSE, 0);
  gtk_table_attach_defaults(table0, (GtkWidget *) hbox, 2, 15, 1, 2);

  for(i = 0; i < 16; i++){
    toggle_button = (GtkToggleButton *) gtk_toggle_button_new();
    gtk_widget_set_size_request((GtkWidget *) toggle_button, 24, 24);
    gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) toggle_button, FALSE, FALSE, 0);
  }

  drum->offset = (GtkVBox*) gtk_vbox_new(FALSE, 0);
  gtk_table_attach_defaults(table0, (GtkWidget *) drum->offset, 15, 16, 0, 3);

  radio_button = (GtkRadioButton *) gtk_radio_button_new_with_label(NULL, "1-16\0");
  gtk_box_pack_start((GtkBox*) drum->offset, (GtkWidget *) radio_button, FALSE, FALSE, 0);

  gtk_box_pack_start((GtkBox*) drum->offset,
		     (GtkWidget *) gtk_radio_button_new_with_label(radio_button->group, "17-32\0"),
		     FALSE, FALSE, 0);

  gtk_box_pack_start((GtkBox*) drum->offset,
		     (GtkWidget *) gtk_radio_button_new_with_label(radio_button->group, "33-48\0"),
		     FALSE, FALSE, 0);

  gtk_box_pack_start((GtkBox*) drum->offset,
		     (GtkWidget *) gtk_radio_button_new_with_label(radio_button->group, "49-64\0"),
		     FALSE, FALSE, 0);
}

void
ags_drum_destroy(GtkObject *object)
{
  AgsDrum *drum = (AgsDrum *) object;
  GList *list0, *list1;
  GList *list2, *list3;
  guint i;

  list0 = gtk_container_get_children((GtkContainer *) drum->offset);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  gtk_widget_destroy((GtkWidget *) drum->offset);

  list0 = gtk_container_get_children((GtkContainer *) drum->pattern);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  for(i = 0; i < 4; i++)
    gtk_widget_destroy((GtkWidget *) drum->index0);

  for(i = 0; i < 12; i++)
    gtk_widget_destroy((GtkWidget *) drum->index1);


  list0 = gtk_container_get_children((GtkContainer *) drum->pad);

  while(list0 != NULL){
    list1 = list0->next;

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  gtk_widget_destroy((GtkWidget *) drum->pad);

  list0 = gtk_container_get_children((GtkContainer *) drum->vbox);
  list1 = gtk_container_get_children((GtkContainer *) list0->data);
  list2 = gtk_container_get_children((GtkContainer *) list1->data);
  list3 = list2->next;

  gtk_widget_destroy((GtkWidget *) list3->data);
  gtk_widget_destroy((GtkWidget *) list2->data);
  gtk_widget_destroy((GtkWidget *) list1->data);

  list1 = list0->next;
  gtk_widget_destroy((GtkWidget *) list0->data);

  list2 = gtk_container_get_children((GtkContainer *) list1->data);

  while(list2 != NULL){
    list3 = list2->next;

    gtk_widget_destroy((GtkWidget *) list2->data);

    list2 = list3;
  }

  gtk_widget_destroy((GtkWidget *) list1->data);

  gtk_widget_destroy((GtkWidget *) drum->vbox);

  ags_machine_destroy(object);
}

void
ags_drum_show(GtkWidget *widget)
{
  AgsDrum *drum = (AgsDrum *) widget;
  AgsWindow *window;
  AgsPad *pad;
  GList *list;

  window = (AgsWindow *) gtk_widget_get_toplevel(widget);

  if(drum->machine.name == NULL)
    drum->machine.name = g_strdup_printf("Default %d\0", window->counter->drum);

  window->counter->drum++;

  list = gtk_container_get_children((GtkContainer *) drum->pad);

  while(list != NULL){
    pad = AGS_PAD(list->data);
    gtk_option_menu_set_history(pad->option, 0);

    list = list->next;
  }

  ags_machine_show(widget);

  gtk_widget_show_all((GtkWidget *) drum->vbox);
}

void
ags_drum_connect(AgsDrum *drum)
{
  GList *list;
  int i;

  list = gtk_container_get_children((GtkContainer *) drum->pad);

  while(list != NULL){
    ags_pad_connect((AgsPad*) list->data);

    list = list->next;
  }

  list = drum->edit_button;

  while(list != NULL){
    g_signal_connect(G_OBJECT(list->data), "clicked\0",
		     G_CALLBACK(ags_drum_edit_callback), (gpointer) drum);

    list = list->next;
  }

  for(i  = 0; i < 12; i++){
    g_signal_connect(G_OBJECT(drum->index1[i]), "clicked\0",
		     G_CALLBACK(ags_drum_index1_callback), (gpointer) drum);
  }

  for(i  = 0; i < 4; i++){
    g_signal_connect(G_OBJECT(drum->index0[i]), "clicked\0",
		     G_CALLBACK(ags_drum_index0_callback), (gpointer) drum);
  }

  list = gtk_container_get_children((GtkContainer *) drum->pattern);

  while(list != NULL){
    g_signal_connect(G_OBJECT(list->data), "clicked\0",
		     G_CALLBACK(ags_drum_pad_callback), (gpointer) drum);

    list = list->next;
  }

  list = gtk_container_get_children((GtkContainer *) drum->offset);

  while(list != NULL){
    g_signal_connect(G_OBJECT(list->data), "clicked\0",
		     G_CALLBACK(ags_drum_offset_callback), (gpointer) drum);
		   
    list = list->next;
  }
}

void
ags_drum_set_audio_channels(AgsAudio *audio, guint audio_channels)
{
  AgsDrum *drum;
  guint old_audio_channels;
  GList *list0;
  guint i;

  old_audio_channels = audio_channels;

  if(audio_channels == old_audio_channels)
    return;

  _ags_audio_set_audio_channels(audio, audio_channels);

  drum = (AgsDrum *) audio->machine;
  list0 = gtk_container_get_children((GtkContainer *) drum->pad);

  if(old_audio_channels < audio_channels){
    AgsLine *line;
    AgsChannel *channel;

    channel = audio->input;

    while(list0 != NULL){
      channel = ags_channel_nth(channel, old_audio_channels);

      for(i = old_audio_channels; i < audio_channels; i++){
	line = ags_line_new();
	line->channel = channel;
	gtk_menu_shell_insert((GtkMenuShell *) list0->data, (GtkWidget *) line, i);

	if(GTK_WIDGET_VISIBLE((GtkWidget *) drum)){
	  gtk_widget_show_all((GtkWidget *) line);
	  ags_line_connect(line);
	}

	channel = channel->next;
      }

      list0 = list0->next;
    }
  }else{
    GList *list1;

    while(list0 != NULL){
      list0 = g_list_nth(list0, old_audio_channels -1);

      for(i = audio_channels; i < old_audio_channels; i++){
	list1 = list0->next;

	gtk_widget_destroy((GtkWidget *) list0->data);

	list0 = list1;
      }
    }
  }
}

void
ags_drum_set_pads(AgsAudio *audio, GType type, guint pads)
{
  guint old_pads;

  old_pads = (type == AGS_TYPE_OUTPUT) ? audio->output_pads: audio->input_pads;

  if(pads == old_pads)
    return;

  _ags_audio_set_pads(audio, type, pads);

  if(type == AGS_TYPE_INPUT){
    AgsDrum *drum = (AgsDrum *) audio->machine;
    AgsPad *pad;

    if(old_pads < pads){
      AgsLine *line;
      AgsChannel *channel;
      GtkMenu *menu;
      GList *list;
      guint i, j;

      list = g_list_alloc();

      if(drum->edit_button == NULL)
	drum->edit_button = list;
      else{
	list->prev = g_list_last(drum->edit_button);
	list->prev->next = list;
      }

      channel = ags_channel_nth(drum->machine.audio->input, old_pads);
      i = old_pads;
      goto ags_drum_init0;
  
      for(; i < pads; i++){
	list->next = g_list_alloc();
	list->next->prev = list;
	list = list->next;
      ags_drum_init0:

	pad = ags_pad_new();
	pad->channel = channel;
	menu = (GtkMenu *) pad->option->menu;

	for(j = 0; j < drum->machine.audio->audio_channels; j++){
	  line = ags_line_new();
	  line->channel = channel;
	  gtk_menu_shell_append((GtkMenuShell *) menu, (GtkWidget *) line);

	  channel->pattern = g_list_alloc();
	  channel->pattern->data = (gpointer) ags_pattern_new();
	  ags_pattern_set_dim((AgsPattern *) channel->pattern->data, 4, 12, 64);
	  channel = channel->next;
	}

	gtk_box_pack_start((GtkBox *) drum->pad, (GtkWidget *) pad, FALSE, FALSE, 0);

	list->data = (gpointer) gtk_toggle_button_new_with_label("edit\0");
	gtk_box_pack_start((GtkBox *) pad,
			   (GtkWidget *) list->data,
			   FALSE, FALSE, 0);

	if(GTK_WIDGET_VISIBLE((GtkWidget *) drum)){
	  gtk_widget_show_all((GtkWidget *) pad);
	  gtk_option_menu_set_history(pad->option, 0);
	  ags_pad_connect(pad);
	  g_signal_connect((GObject *) list->data, "clicked\0",
			   G_CALLBACK(ags_drum_edit_callback), (gpointer) drum);
	}
      }

      if(old_pads == 0){
	drum->selected_pad = (AgsPad *) gtk_widget_get_ancestor(GTK_WIDGET(drum->edit_button->data), AGS_TYPE_PAD);
	drum->selected_edit_button = (GtkToggleButton *) drum->edit_button->data;
	gtk_toggle_button_set_active((GtkToggleButton *) drum->edit_button->data, TRUE);
      }
    }else{
      GList *list0, *list1;

      pad = (AgsPad *) gtk_widget_get_ancestor((GtkWidget *) drum->selected_edit_button, AGS_TYPE_PAD);

      if(pads == 0){
	drum->selected_pad = NULL;
	drum->selected_edit_button = NULL;
      }else if(pad->channel->pad > pads){
	drum->selected_pad = (AgsPad *) gtk_widget_get_ancestor(GTK_WIDGET(drum->edit_button->data), AGS_TYPE_PAD);
	drum->selected_edit_button = (GtkToggleButton *) drum->edit_button->data;
	gtk_toggle_button_set_active((GtkToggleButton *) drum->edit_button->data, TRUE);
      }

      list0 = g_list_nth(drum->edit_button, pads);

      while(list0 != NULL){
	list1 = list0->next;
	free(list0);
	list0 = list1;
      }

      list0 = gtk_container_get_children((GtkContainer *) drum->pad);
      list0 = g_list_nth(list0, pads);

      while(list0 != NULL){
	list1 = list0->next;

	gtk_widget_destroy((GtkWidget *) list0->data);

	list0 = list1;
      }
    }
  }
}

void
ags_drum_set_lines(AgsAudio *audio, GType type, guint lines)
{
  _ags_audio_set_lines(audio, type, lines);
}

AgsDrum*
ags_drum_new()
{
  AgsDrum *drum;

  drum = (AgsDrum *) g_object_new(AGS_TYPE_DRUM, NULL);

  return(drum);
}
