#include "ags_ffplayer.h"
#include "ags_ffplayer_callbacks.h"

#include "../ags_window.h"

#include "../../audio/ags_audio.h"
#include "../../audio/ags_input.h"
#include "../../audio/ags_output.h"

GType ags_ffplayer_get_type(void);
void ags_ffplayer_class_init(AgsFFPlayerClass *ffplayer);
void ags_ffplayer_init(AgsFFPlayer *ffplayer);
void ags_ffplayer_destroy(GtkObject *object);
void ags_ffplayer_show(GtkWidget *widget);
void ags_ffplayer_set_audio_channels(AgsAudio *audio, guint audio_channels);
void ags_ffplayer_set_pads(AgsAudio *audio, GType type, guint pads);
void ags_ffplayer_set_lines(AgsAudio *audio, GType type, guint lines);

extern void ags_file_read_ffplayer(AgsFile *file, AgsMachine *machine);
extern void ags_file_write_ffplayer(AgsFile *file, AgsMachine *machine);

GtkStyle *ffplayer_style;

GType
ags_ffplayer_get_type(void)
{
  static GType ffplayer_type = 0;

  if (!ffplayer_type){
    static const GtkTypeInfo ffplayer_info = {
      "AgsFFPlayer\0",
      sizeof(AgsFFPlayer), /* base_init */
      sizeof(AgsFFPlayerClass), /* base_finalize */
      (GtkClassInitFunc) ags_ffplayer_class_init,
      (GtkObjectInitFunc) ags_ffplayer_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    ffplayer_type = gtk_type_unique (AGS_TYPE_MACHINE, &ffplayer_info);
  }

  return (ffplayer_type);
}

void
ags_ffplayer_class_init(AgsFFPlayerClass *ffplayer)
{
  GtkObjectClass *object = (GtkObjectClass *) ffplayer;
  GtkWidgetClass *widget = (GtkWidgetClass *) ffplayer;
  AgsMachineClass *machine = (AgsMachineClass *) ffplayer;

  object->destroy = ags_ffplayer_destroy;

  widget->show = ags_ffplayer_show;

  machine->read_file = ags_file_read_ffplayer;
  machine->write_file = ags_file_write_ffplayer;
}

void
ags_ffplayer_init(AgsFFPlayer *ffplayer)
{
  AGS_AUDIO_GET_CLASS(ffplayer->machine.audio)->set_audio_channels = ags_ffplayer_set_audio_channels;
  AGS_AUDIO_GET_CLASS(ffplayer->machine.audio)->set_pads = ags_ffplayer_set_pads;
  AGS_AUDIO_GET_CLASS(ffplayer->machine.audio)->set_lines = ags_ffplayer_set_lines;

  ffplayer->machine.audio->audio_channels = 2;
  ags_audio_set_pads(ffplayer->machine.audio, AGS_TYPE_OUTPUT, 1);
  ags_audio_set_pads(ffplayer->machine.audio, AGS_TYPE_INPUT, 64);
}

void
ags_ffplayer_destroy(GtkObject *object)
{
  ags_machine_destroy(object);
}

void
ags_ffplayer_show(GtkWidget *widget)
{
  AgsFFPlayer *ffplayer = (AgsFFPlayer *) widget;
  AgsWindow *window;

  window = (AgsWindow *) gtk_widget_get_toplevel(widget);

  if(ffplayer->machine.name == NULL)
    ffplayer->machine.name = g_strdup_printf("Default %d\0", window->counter->ffplayer);

  window->counter->ffplayer++;

  ags_machine_show(widget);
}

void
ags_ffplayer_connect(AgsFFPlayer *ffplayer)
{
  ags_machine_connect(ffplayer->machine);
}

void
ags_ffplayer_set_audio_channels(AgsAudio *audio, guint audio_channels)
{
  _ags_audio_set_audio_channels(audio, audio_channels);
}

void
ags_ffplayer_set_pads(AgsAudio *audio, GType type, guint pads)
{
  _ags_audio_set_pads(audio, type, pads);
}

void
ags_ffplayer_set_lines(AgsAudio *audio, GType type, guint lines)
{
  _ags_audio_set_lines(audio, type, lines);
}

AgsFFPlayer*
ags_ffplayer_new()
{
  AgsFFPlayer *ffplayer;

  ffplayer = (AgsFFPlayer *) g_object_new(AGS_TYPE_FFPLAYER, NULL);

  return(ffplayer);
}
