#include "ags_mixer.h"
#include "ags_mixer_callbacks.h"

#include "../ags_window.h"

#include "../../audio/ags_audio.h"
#include "../../audio/ags_input.h"
#include "../../audio/ags_output.h"

GType ags_mixer_get_type(void);
void ags_mixer_class_init(AgsMixerClass *mixer);
void ags_mixer_init(AgsMixer *mixer);
void ags_mixer_destroy(GtkObject *object);
void ags_mixer_show(GtkWidget *widget);
void ags_mixer_set_audio_channels(AgsAudio *audio, guint audio_channels);
void ags_mixer_set_pads(AgsAudio *audio, GType type, guint pads);
void ags_mixer_set_lines(AgsAudio *audio, GType type, guint lines);

extern void ags_file_read_mixer(AgsFile *file, AgsMachine *machine);
extern void ags_file_write_mixer(AgsFile *file, AgsMachine *machine);

GType
ags_mixer_get_type(void)
{
  static GType mixer_type = 0;

  if (!mixer_type){
    static const GtkTypeInfo mixer_info = {
      "AgsMixer\0",
      sizeof(AgsMixer), /* base_init */
      sizeof(AgsMixerClass), /* base_finalize */
      (GtkClassInitFunc) ags_mixer_class_init,
      (GtkObjectInitFunc) ags_mixer_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    mixer_type = gtk_type_unique (AGS_TYPE_MACHINE, &mixer_info);
  }

  return (mixer_type);
}

void
ags_mixer_class_init(AgsMixerClass *mixer)
{
  GtkObjectClass *object = (GtkObjectClass *) mixer;
  GtkWidgetClass *widget = (GtkWidgetClass *) mixer;
  AgsMachineClass *machine = (AgsMachineClass *) mixer;

  object->destroy = ags_mixer_destroy;

  widget->show = ags_mixer_show;

  machine->read_file = ags_file_read_mixer;
  machine->write_file = ags_file_write_mixer;
}

void
ags_mixer_init(AgsMixer *mixer)
{
  AGS_AUDIO_GET_CLASS(mixer->machine.audio)->set_audio_channels = ags_mixer_set_audio_channels;
  AGS_AUDIO_GET_CLASS(mixer->machine.audio)->set_pads = ags_mixer_set_pads;
  AGS_AUDIO_GET_CLASS(mixer->machine.audio)->set_lines = ags_mixer_set_lines;

  mixer->machine.audio->audio_channels = 2;
  ags_audio_set_pads(mixer->machine.audio, AGS_TYPE_OUTPUT, 1);
  ags_audio_set_pads(mixer->machine.audio, AGS_TYPE_INPUT, 8);
}

void
ags_mixer_destroy(GtkObject *object)
{
  ags_machine_destroy(object);
}

void
ags_mixer_show(GtkWidget *widget)
{
  AgsMixer *mixer = (AgsMixer *) widget;
  AgsWindow *window;

  window = (AgsWindow *) gtk_widget_get_toplevel(widget);

  if(mixer->machine.name == NULL)
    mixer->machine.name = g_strdup_printf("Default %d\0", window->counter->mixer);

  window->counter->mixer++;

  ags_machine_show(widget);
}

void
ags_mixer_set_audio_channels(AgsAudio *audio, guint audio_channels)
{
  _ags_audio_set_audio_channels(audio, audio_channels);
}

void
ags_mixer_set_pads(AgsAudio *audio, GType type, guint pads)
{
  _ags_audio_set_pads(audio, type, pads);
}

void
ags_mixer_set_lines(AgsAudio *audio, GType type, guint lines)
{
  _ags_audio_set_lines(audio, type, lines);
}

AgsMixer*
ags_mixer_new()
{
  AgsMixer *mixer;

  mixer = (AgsMixer *) g_object_new(AGS_TYPE_MIXER, NULL);

  return(mixer);
}
