#include "ags_panel.h"
#include "ags_panel_callbacks.h"

#include "../ags_window.h"

#include "../../audio/ags_audio.h"
#include "../../audio/ags_input.h"
#include "../../audio/ags_output.h"

GType ags_panel_get_type(void);
void ags_panel_class_init(AgsPanelClass *panel);
void ags_panel_init(AgsPanel *panel);
void ags_panel_destroy(GtkObject *object);
void ags_panel_show(GtkWidget *widget);
void ags_panel_set_audio_channels(AgsAudio *audio, guint audio_channels);
void ags_panel_set_pads(AgsAudio *audio, GType type, guint pads);
void ags_panel_set_lines(AgsAudio *audio, GType type, guint lines);

extern void ags_file_read_panel(AgsFile *file, AgsMachine *machine);
extern void ags_file_write_panel(AgsFile *file, AgsMachine *machine);

GType
ags_panel_get_type(void)
{
  static GType panel_type = 0;

  if (!panel_type){
    static const GtkTypeInfo panel_info = {
      "AgsPanel\0",
      sizeof(AgsPanel), /* base_init */
      sizeof(AgsPanelClass), /* base_finalize */
      (GtkClassInitFunc) ags_panel_class_init,
      (GtkObjectInitFunc) ags_panel_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    panel_type = gtk_type_unique (AGS_TYPE_MACHINE, &panel_info);
  }

  return (panel_type);
}

void
ags_panel_class_init(AgsPanelClass *panel)
{
  GtkObjectClass *object = (GtkObjectClass *) panel;
  GtkWidgetClass *widget = (GtkWidgetClass *) panel;
  AgsMachineClass *machine = (AgsMachineClass *) panel;

  object->destroy = ags_panel_destroy;

  widget->show = ags_panel_show;

  machine->read_file = ags_file_read_panel;
  machine->write_file = ags_file_write_panel;
}

void
ags_panel_init(AgsPanel *panel)
{
  panel->vbox = (GtkVBox *) gtk_vbox_new(FALSE, 0);
  gtk_container_add((GtkContainer*) (gtk_container_get_children((GtkContainer *) panel))->data, (GtkWidget *) panel->vbox);

  AGS_AUDIO_GET_CLASS(panel->machine.audio)->set_audio_channels = ags_panel_set_audio_channels;
  AGS_AUDIO_GET_CLASS(panel->machine.audio)->set_pads = ags_panel_set_pads;
  AGS_AUDIO_GET_CLASS(panel->machine.audio)->set_lines = ags_panel_set_lines;

  panel->machine.audio->output_pads = 1;
  panel->machine.audio->input_pads = 1;
  ags_audio_set_audio_channels(panel->machine.audio, 2);
}

void
ags_panel_destroy(GtkObject *object)
{
  AgsPanel *panel = (AgsPanel *) object;
  GList *list0, *list1;
  GList *list2, *list3;

  list0 = gtk_container_get_children((GtkContainer *) panel->vbox);

  while(list0 != NULL){
    list1 = list0->next;

    list2 = gtk_container_get_children((GtkContainer *) list0->data);

    list3 = list2->next;
    gtk_widget_destroy((GtkWidget *) list2->data);

    gtk_widget_destroy((GtkWidget *) list3);

    gtk_widget_destroy((GtkWidget *) list0->data);

    list0 = list1;
  }

  gtk_widget_destroy((GtkWidget *) panel->vbox);

  ags_machine_destroy(object);
}

void
ags_panel_show(GtkWidget *widget)
{
  AgsPanel *panel = (AgsPanel *) widget;
  AgsWindow *window;

  window = (AgsWindow *) gtk_widget_get_toplevel(widget);

  if(panel->machine.name == NULL)
    panel->machine.name = g_strdup_printf("Default %d\0", window->counter->panel);

  window->counter->panel++;

  ags_machine_show(widget);

  gtk_widget_show_all((GtkWidget *) panel->vbox);
}

void
ags_panel_set_audio_channels(AgsAudio *audio, guint audio_channels)
{
  AgsPanel *panel;
  guint old_audio_channels;

  old_audio_channels = audio->audio_channels;

  if(audio_channels == old_audio_channels)
    return;

  _ags_audio_set_audio_channels(audio, audio_channels);

  panel = (AgsPanel *) audio->machine;

  if(old_audio_channels < audio_channels){
    GtkHBox *hbox;
    guint i;

    for(i = old_audio_channels; i < audio_channels; i++){
      hbox = (GtkHBox*) gtk_hbox_new(FALSE, 0);
      gtk_box_pack_start((GtkBox *) panel->vbox, (GtkWidget *) hbox, FALSE, FALSE, 0);

      gtk_box_pack_start((GtkBox *) hbox, 
			 (GtkWidget *) gtk_label_new(g_strdup_printf("channel %d: \0", i)),
			 FALSE, FALSE, 0);

      gtk_box_pack_start((GtkBox *) hbox,
			 (GtkWidget *) gtk_check_button_new_with_label("mute\0"),
			 FALSE, FALSE, 0);

      if(GTK_WIDGET_VISIBLE((GtkWidget *) panel))
	gtk_widget_show_all((GtkWidget *) hbox);
    }
  }else{
    GList *list0, *list1;
    GList *list2, *list3;

    list0 = g_list_nth(gtk_container_get_children((GtkContainer *) panel->vbox),
		       old_audio_channels);

    while(list0 != NULL){
      list1 = list0->next;
      list2 = gtk_container_get_children((GtkContainer *) list0->data);

      list3 = list2->next;
      gtk_widget_destroy((GtkWidget *) list2->data);

      gtk_widget_destroy((GtkWidget *) list3->data);

      gtk_widget_destroy((GtkWidget *) list0->data);

      list0 = list1;
    }
  }
}

void
ags_panel_set_pads(AgsAudio *audio, GType type, guint pads)
{
  printf("AgsPanel only audio channels can be adjusted\n\0");
}

void
ags_panel_set_lines(AgsAudio *audio, GType type, guint lines)
{
  printf("AgsPanel only audio channels can be adjusted\n\0");
}

AgsPanel*
ags_panel_new()
{
  AgsPanel *panel;

  panel = (AgsPanel *) g_object_new(AGS_TYPE_PANEL, NULL);

  return(panel);
}
