#include "ags_synth.h"
#include "ags_synth_callbacks.h"

#include "ags_oscillator.h"

#include "../ags_window.h"

#include "../../audio/ags_audio.h"
#include "../../audio/ags_input.h"
#include "../../audio/ags_output.h"

GType ags_synth_get_type(void);
void ags_synth_class_init(AgsSynthClass *synth);
void ags_synth_init(AgsSynth *synth);
void ags_synth_destroy(GtkObject *object);
void ags_synth_show(GtkWidget *widget);
void ags_synth_set_audio_channels(AgsAudio *audio, guint audio_channels);
void ags_synth_set_pads(AgsAudio *audio, GType type, guint pads);
void ags_synth_set_lines(AgsAudio *audio, GType type, guint lines);

extern void ags_file_read_synth(AgsFile *file, AgsMachine *machine);
extern void ags_file_write_synth(AgsFile *file, AgsMachine *machine);

GType
ags_synth_get_type(void)
{
  static GType synth_type = 0;

  if (!synth_type){
    static const GtkTypeInfo synth_info = {
      "AgsSynth\0",
      sizeof(AgsSynth), /* base_init */
      sizeof(AgsSynthClass), /* base_finalize */
      (GtkClassInitFunc) ags_synth_class_init,
      (GtkObjectInitFunc) ags_synth_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    synth_type = gtk_type_unique (AGS_TYPE_MACHINE, &synth_info);
  }

  return (synth_type);
}

void
ags_synth_class_init(AgsSynthClass *synth)
{
  GtkObjectClass *object = (GtkObjectClass *) synth;
  GtkWidgetClass *widget = (GtkWidgetClass *) synth;
  AgsMachineClass *machine = (AgsMachineClass *) synth;

  object->destroy = ags_synth_destroy;

  widget->show = ags_synth_show;

  machine->read_file = ags_file_read_synth;
  machine->write_file = ags_file_write_synth;
}

void
ags_synth_init(AgsSynth *synth)
{
  GtkMenu *menu;

  synth->hbox = (GtkHBox *) gtk_hbox_new(FALSE, 0);
  gtk_container_add((GtkContainer*) (gtk_container_get_children((GtkContainer *) synth))->data, (GtkWidget *) synth->hbox);

  synth->option = (GtkOptionMenu *) gtk_option_menu_new();
  gtk_box_pack_start((GtkBox*) synth->hbox, (GtkWidget *) synth->option, FALSE, FALSE, 0);

  menu = (GtkMenu *) gtk_menu_new();
  gtk_option_menu_set_menu(synth->option, (GtkWidget *) menu);

  AGS_AUDIO_GET_CLASS(synth->machine.audio)->set_audio_channels = ags_synth_set_audio_channels;
  AGS_AUDIO_GET_CLASS(synth->machine.audio)->set_pads = ags_synth_set_pads;
  AGS_AUDIO_GET_CLASS(synth->machine.audio)->set_lines = ags_synth_set_lines;

  synth->machine.audio->audio_channels = 1;
  ags_audio_set_pads((AgsAudio*) synth->machine.audio, AGS_TYPE_OUTPUT, 1);
  ags_audio_set_pads((AgsAudio*) synth->machine.audio, AGS_TYPE_INPUT, 2);
}

void
ags_synth_destroy(GtkObject *object)
{
  ags_machine_destroy(object);
}

void
ags_synth_show(GtkWidget *widget)
{
  AgsSynth *synth = (AgsSynth *) widget;
  AgsWindow *window;

  window = (AgsWindow *) gtk_widget_get_toplevel(widget);

  if(synth->machine.name == NULL)
    synth->machine.name = g_strdup_printf("Default %d\0", window->counter->synth);

  window->counter->synth++;

  ags_machine_show(widget);

  gtk_widget_show_all((GtkWidget *) synth->hbox);
}

void
ags_synth_connect(AgsSynth *synth)
{
  ags_machine_connect(synth->machine);
}

void
ags_synth_set_audio_channels(AgsAudio *audio, guint audio_channels)
{
  printf("AgsSynth only pads can be adjusted\n\0");
  //  _ags_audio_set_audio_channels(audio, audio_channels);
}

void
ags_synth_set_pads(AgsAudio *audio, GType type, guint pads)
{
  AgsSynth *synth;
  GtkMenuItem *item;
  guint old_pads;

  old_pads = (type == AGS_TYPE_OUTPUT) ? audio->output_pads: audio->input_pads;

  if(pads == old_pads)
    return;

  _ags_audio_set_pads(audio, type, pads);
  synth = (AgsSynth *) audio->machine;

  if(type == AGS_TYPE_INPUT)
    if(pads > old_pads){
      AgsOscillator *oscillator;
      guint i;
      
      for(i = old_pads; i < pads; i++){
	item = (GtkMenuItem *) gtk_menu_item_new();
	oscillator = ags_oscillator_new();
	gtk_container_add((GtkContainer *) item,
			  (GtkWidget *) oscillator);
	gtk_menu_shell_append((GtkMenuShell *) synth->option->menu, (GtkWidget *) item);

	if(GTK_WIDGET_VISIBLE((GtkWidget *) synth)){
	  gtk_widget_show_all((GtkWidget *) item);
	  ags_oscillator_connect(oscillator);
	}
      }

      if(old_pads == 0)
	gtk_option_menu_set_history(synth->option, 0);
    }else{
      GList *list0, *list1;
      
      list0 = g_list_nth(gtk_container_get_children((GtkContainer *) synth->option->menu), pads);

      while(list0 != NULL){
	list1 = list0->next;

	gtk_widget_destroy((GtkWidget *) list0->data);

	list0 = list1;
      }
    }
}

void
ags_synth_set_lines(AgsAudio *audio, GType type, guint lines)
{
  _ags_audio_set_lines(audio, type, lines);
}

AgsSynth*
ags_synth_new()
{
  AgsSynth *synth;

  synth = (AgsSynth *) g_object_new(AGS_TYPE_SYNTH, NULL);

  return(synth);
}
