#include "ags_audio.h"
#include <stdio.h>
#include "ags_output.h"
#include "ags_input.h"

GType ags_audio_get_type();
void ags_audio_class_init (AgsAudioClass *audio_class);
void ags_audio_init (AgsAudio *audio);
void ags_audio_destroy (GObject *gobject);
void _ags_audio_set_audio_channels(AgsAudio *audio, guint audio_channels);
void _ags_audio_set_pads(AgsAudio *audio, GType type, guint channels);
void _ags_audio_set_lines(AgsAudio *audio, GType type, guint lines);

GType
ags_audio_get_type (void)
{
  static GType ags_type_audio = 0;

  if(!ags_type_audio){
    static const GTypeInfo ags_audio_info = {
      sizeof (AgsAudioClass),
      NULL, /* base_init */
      NULL, /* base_finalize */
      (GClassInitFunc) ags_audio_class_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      sizeof (AgsAudio),
      0,    /* n_preallocs */
      (GInstanceInitFunc) ags_audio_init,
    };
    ags_type_audio = g_type_register_static(G_TYPE_OBJECT, "AgsAudio\0", &ags_audio_info, 0);
  }
  return (ags_type_audio);
}

void
ags_audio_class_init(AgsAudioClass *audio_class)
{
  audio_class->set_audio_channels = _ags_audio_set_audio_channels;
  audio_class->set_pads = _ags_audio_set_pads;
  audio_class->set_lines = _ags_audio_set_lines;
}

void
ags_audio_init(AgsAudio *audio)
{
  audio->flags = 0;

  audio->sequence_lenght = 0;
  audio->audio_channels = 0;

  audio->output_pads = 0;
  audio->output_lines = 0;
  audio->input_pads = 0;
  audio->input_lines = 0;

  audio->output = NULL;
  audio->input = NULL;

  audio->machine = NULL;
}

void
ags_audio_destroy(GObject *gobject)
{
  AgsAudio *audio;
  AgsChannel *channel;

  channel = audio->output;

  if(channel != NULL){
    while(channel->next != NULL){
      channel = channel->next;
      ags_channel_destroy(channel->prev);
    }

    ags_channel_destroy((GObject *) channel);
  }

  channel = audio->input;

  if(channel != NULL){
    while(channel->next != NULL){
      channel = channel->next;
      ags_channel_destroy((GObject *) channel->prev);
    }

    ags_channel_destroy(channel);
  }

  g_object_unref (gobject);
}

void
_ags_audio_set_audio_channels(AgsAudio *audio, guint audio_channels)
{
  AgsChannel *channel0, *channel1;
  GType type;
  guint pads;
  guint i, j;

  if(audio_channels == audio->audio_channels)
    return;

  type = AGS_TYPE_OUTPUT;
  pads = audio->output_pads;

  if(audio_channels > audio->audio_channels){
    AgsChannel *channel2, *channel3;
    gboolean alloc_recycling;

    if(AGS_AUDIO_OUTPUT_HAS_RECYCLING & audio->flags)
      alloc_recycling = TRUE;
    else
      alloc_recycling = FALSE;

    if(audio->audio_channels == 0){
    ags_audio_resize_audio_channels0:
      channel0 = g_object_new(type, NULL);

      if(type == AGS_TYPE_OUTPUT)
	audio->output = channel0;
      else
	audio->input = channel0;

      for(i = 1; i < pads; i++){	
	channel0->next =
	  channel0->next_pad = g_object_new(type, NULL);
	channel0->next->prev =
	  channel0->next->prev_pad = channel0;
	channel0 = channel0->next;

	channel0->audio = (GObject *) audio;
	channel0->pad = i;
	channel0->line = i;

	if(alloc_recycling){
	  channel0->first_recycling =
	    channel0->last_recycling = ags_recycling_new();
	}
      }

      if(type == AGS_TYPE_OUTPUT){
	type = AGS_TYPE_INPUT;
	channel0 = audio->input;

	if(AGS_AUDIO_INPUT_HAS_RECYCLING & audio->flags)
	  alloc_recycling = TRUE;
	else
	  alloc_recycling = FALSE;

	goto ags_audio_resize_audio_channels0;
      }

      channel0 = audio->output;
      audio->audio_channels = 1;
    }

    type = AGS_TYPE_OUTPUT;
    channel0 = audio->output;

    if(AGS_AUDIO_OUTPUT_HAS_RECYCLING & audio->flags)
      alloc_recycling = TRUE;
    else
      alloc_recycling = FALSE;

  ags_audio_resize_audio_channels1:

    channel0 = ags_channel_nth(channel0, audio->audio_channels -1);

    for(j = audio->audio_channels; j < audio_channels; j++){
      channel0->next = g_object_new(type, NULL);
      channel0->next->prev = channel0;
      channel0 = channel0->next;

      channel0->audio = (GObject *) audio;
      channel0->audio_channel = j;
      channel0->line = j;
      channel0->prev_pad = NULL;

      if(alloc_recycling)
	channel0->first_recycling =
	  channel0->last_recycling = ags_recycling_new();
    }

    channel2 = channel0;

    for(i = 1; i < pads; i++){
      for(j = 0; j < audio->audio_channels; j++){
	channel0->line = i * audio_channels + j;
	channel0 = channel0->next;
      }

      channel1 = channel0;

      for(; j < audio_channels; j++){
	channel0->next = g_object_new(type, NULL);
	channel0->next->prev = channel0;
	channel0 = channel0->next;

	channel0->audio = (GObject *) audio;
	channel0->pad = i;
	channel0->audio_channel = j;
	channel0->line = i * audio_channels + j;

	if(alloc_recycling){
	  channel0->first_recycling =
	    channel0->last_recycling = ags_recycling_new();
	}
      }

      channel3 = channel1;

      while(channel1 != channel0){
	channel2->next_pad = channel2;
	channel1->prev_pad = channel1;
	channel2 = channel2->next;
	channel1 = channel1->next;
      }

      channel2 = channel3;
    }

    //    channel1 = ags_channel_nth(channel1, audio->audio_channels -1);

    //    while(channel1 != NULL){
    //      channel1->next_pad = NULL;
    //      channel1 = channel1->next;
    //    }

    if(type == AGS_TYPE_OUTPUT){
      type = AGS_TYPE_INPUT;
      channel0 = audio->input;
      pads = audio->input_pads;

      if(AGS_AUDIO_INPUT_HAS_RECYCLING & audio->flags)
	alloc_recycling = TRUE;
      else
	alloc_recycling = FALSE;

      goto ags_audio_resize_audio_channels1;
    }
  }else{
    pads = audio->output_pads;
    channel0 = audio->output;

  ags_audio_resize_audio_channels2:
    i = 0;
    j = audio_channels;
    goto ags_audio_resize_audio_channels3;

    for(; i < pads; i++){
      for(j = 0; j < audio_channels; j++){
	channel0->line = i * audio_channels + j;
	channel0 = channel0->next;
      }

    ags_audio_resize_audio_channels3:
      for(; j < audio->audio_channels; j++){
	channel1 = channel0->next;
	ags_channel_destroy(channel0);
	channel0 = channel1;
      }
    }

    if(type == AGS_TYPE_OUTPUT){
      type = AGS_TYPE_INPUT;
      channel0 = audio->input;
      pads = audio->input_pads;
      goto ags_audio_resize_audio_channels2;
    }

    if(audio_channels == 0){
      audio->output = NULL;
      audio->input = NULL;
    }
  }

  audio->audio_channels = audio_channels;
}

void
_ags_audio_set_pads(AgsAudio *audio, GType type, guint pads)
{
  AgsChannel *start, *nth, *stop;
  guint i, old_pads;

  if(audio->audio_channels == 0){
    fprintf(stderr, "_ags_audio_set_pads: there no audio pads\n\0");
    return;
  }

  old_pads = (type == AGS_TYPE_OUTPUT) ? audio->output_pads: audio->input_pads;

  if(pads < old_pads){
    if(type == AGS_TYPE_OUTPUT){
      nth = ags_channel_nth(audio->output, pads * audio->audio_channels);

      if(pads == 0)
	audio->output = NULL;
    }else{
      nth = ags_channel_nth(audio->input, pads * audio->audio_channels);

      if(pads == 0)
	audio->input = NULL;
    }

    while(nth->next != NULL){
      nth = nth->next;
      ags_channel_destroy(nth->prev); 
    }

    ags_channel_destroy(nth);
  }else if(pads > old_pads){
    gboolean alloc_recycling;
    guint j;

    start =
      nth = (AgsChannel *) g_object_new(type, NULL);

    if(type == AGS_TYPE_OUTPUT){
      if(AGS_AUDIO_OUTPUT_HAS_RECYCLING & audio->flags)
	alloc_recycling = TRUE;
      else
	alloc_recycling = FALSE;
    }else{
      if(AGS_AUDIO_INPUT_HAS_RECYCLING & audio->flags)
	alloc_recycling = TRUE;
      else
	alloc_recycling = FALSE;
    }

    i = old_pads;
    j = 0;
    goto _ags_audio_set_pads0;

    for(; i < pads; i++){
      j = 0;
      for(; j < audio->audio_channels; j++){
	nth->next = (AgsChannel *) g_object_new(type, NULL);
	nth->next->prev = nth;
	nth = nth->next;
      _ags_audio_set_pads0:

	nth->audio = (GObject *) audio;
	nth->pad = i;
	nth->audio_channel = j;
	nth->line = i * audio->audio_channels + j;

	if(alloc_recycling){
	  nth->first_recycling =
	    nth->last_recycling = ags_recycling_new();
	}
      }
    }

    stop = nth;

    for(j = 0; j < audio->audio_channels; j++){
      nth->next_pad = NULL;
      nth = nth->prev;
    }

    for(i = 1; i < pads -1; i++)
      for(j = 0; j < audio->audio_channels; j++){
	nth->next_pad = stop;
	stop->prev_pad = nth;
	nth = nth->prev;
	stop = stop->prev;
      }

    if(old_pads == 0){
      nth = start;

      for(j = 0; j < audio->audio_channels; j++){
	nth->prev_pad = NULL;
	nth = nth->next;
      }

      if(type == AGS_TYPE_OUTPUT)
	audio->output = start;
      else
	audio->input = start;
    }else{
      AgsChannel *channel = (type == AGS_TYPE_OUTPUT) ? audio->output: audio->input;

      nth = ags_channel_last(channel);
      stop = start;

      for(j = 0; j < audio->audio_channels; j++){
	nth->next_pad = stop;
	stop->prev_pad = nth;
	nth = nth->next;
	stop = stop->next;
      }
    }
  }

  if(type == AGS_TYPE_OUTPUT){
    audio->output_pads = pads;
    audio->output_lines = pads * audio->audio_channels;
  }else{
    audio->input_pads = pads;
    audio->input_lines = pads * audio->audio_channels;
  }
}

void
_ags_audio_set_lines(AgsAudio *audio, GType type, guint lines)
{
  if(lines != 0 && lines % audio->audio_channels != 0){
    fprintf(stderr, "_ags_audio_set_lines: invalid count of lines\n\0");
    return;
  }

  ags_audio_set_pads(audio, type, lines / audio->audio_channels);
}

AgsAudio*
ags_audio_new()
{
  AgsAudio *audio;

  audio = (AgsAudio *) g_object_new(AGS_TYPE_AUDIO, NULL);

  return(audio);
}
