#include "ags_audio_signal.h"

GType ags_audio_signal_get_type();
void ags_audio_signal_class_init (AgsAudioSignalClass *audio_signal_class);
void ags_audio_signal_init (AgsAudioSignal *audio_signal);
void ags_audio_signal_destroy (GObject *gobject);

#define AGS_STREAM_LOCK_SIZE sizeof(_AgsStreamLock->lock)

GType
ags_audio_signal_get_type(void)
{
  static GType ags_type_audio_signal = 0;

  if(!ags_type_audio_signal){
    static const GTypeInfo ags_audio_signal_info = {
      sizeof (AgsAudioSignalClass),
      NULL, /* base_init */
      NULL, /* base_finalize */
      (GClassInitFunc) ags_audio_signal_class_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      sizeof (AgsAudioSignal),
      0,    /* n_preallocs */
      (GInstanceInitFunc) ags_audio_signal_init,
    };
    ags_type_audio_signal = g_type_register_static(G_TYPE_OBJECT, "AgsAudioSignal\0", &ags_audio_signal_info, 0);
  }
  return (ags_type_audio_signal);
}

void
ags_audio_signal_class_init(AgsAudioSignalClass *audio_signal)
{
}

void
ags_audio_signal_init(AgsAudioSignal *audio_signal)
{
  audio_signal->devout = NULL;

  audio_signal->start = (AgsOffset *) malloc(sizeof(AgsOffset));
  audio_signal->stop = (AgsOffset *) malloc(sizeof(AgsOffset));
  audio_signal->current = &audio_signal->attack[0];

  audio_signal->attack[0] = 0;
  audio_signal->attack[1] = 0;
  audio_signal->attack[2] = 0;
  audio_signal->stream =
    audio_signal->current_stream = NULL;

  audio_signal->prev_stream = NULL;
  audio_signal->next_stream = NULL;

  audio_signal->lock_attack = 0;
  audio_signal->lock =
    audio_signal->current_lock = NULL;
}

void
ags_audio_signal_destroy(GObject *gobject)
{
  g_object_unref (gobject);
}

GList*
ags_stream_alloc(guint buffer_size)
{
  GList *stream;

  stream = g_list_alloc();
  stream->data = (gpointer) malloc(buffer_size * sizeof(guint));
  memset(stream->data, 0, buffer_size * sizeof(guint));

  return(stream);
}

void
ags_audio_signal_add_stream(AgsAudioSignal *audio_signal)
{
  GList *stream0, *stream1;

  stream1 = ags_audio_signal_alloc_stream(audio_signal->devout->buffer_size);

  if(audio_signal->stream == NULL){
    audio_signal->stream = stream1;
    audio_signal->attack[0] = 0;

    audio_signal->lock = g_list_alloc();
    audio_signal->lock->data = (gpointer) malloc(AGS_STREAM_LOCK_SIZE * sizeof(guint));
    memset(audio_signal->lock, 0, AGS_STREAM_LOCK_SIZE * sizeof(guint));
    audio_signal->lock_attack = 0;

    return;
  }

  stream0 =
    stream1->prev = g_list_last(audio_signal->stream);

  if(audio_signal->lock_attack % AGS_STREAM_LOCK_SIZE == 0){
    gpointer data;

    data = (gpointer) malloc(AGS_STREAM_LOCK_SIZE * sizeof(guint));
    g_list_append(audio_signal->lock, data);

    memset(data, 0, AGS_STREAM_LOCK_SIZE * sizeof(guint));
  }

  stream0->next = stream1;
}

void
ags_audio_signal_resize_stream(AgsAudioSignal *audio_signal, AgsCounter *stop)
{
}

GList*
ags_stream_lock_alloc()
{
  AgsStreamLock *lock;
  GList *stream_lock;

  stream_lock = g_list_alloc();

  lock =
    stream_lock->data = (gpointer) malloc(sizeof(AgsStreamLock));

  memset(&lock->lock, 0, AGS_STREAM_LOCK_SIZE * sizeof(guint));
}

void
ags_audio_signal_add_stream_lock(AgsAudioSignal *audio_signal)
{
}

void
ags_audio_signal_resize_stream_lock(AgsAudioSignal *audio_signal, AgsCounter *stop)
{
}

void
ags_audio_signal_lock(AgsAudioSignal *audio_signal, AgsCounter *start, AgsCounter *stop)
{
  GList *stream, *lock, *list0, *list1;
}

void
ags_audio_signal_remove_unlocked_buffer(AgsAudioSignal *audio_signal)
{
  GList *stream, *stream_prev, *stream_next;
  GList *lock, *lock_prev;
  gboolean check_for_stream;
  guint i, removed_locks;
  void ags_audio_signal_remove_unlocked_buffer_empty(){
    stream = audio_signal->stream;
    audio_signal->stream = NULL;
    audio_signal->current_stream = NULL;

    free(stream->data);
    free(stream);

    lock = audio_signal->lock;
    audio_signal->lock = NULL;
    audio_signal->current_lock = NULL;

    free(lock->data);
    free(lock);
  }

  if(audio_signal->lock == NULL || ((AgsStreamLock *) audio_signal->lock->data)->start >= audio_signal->devout->offset)
    return;

  if(((AgsStreamLock *) audio_signal->lock->data)->stop >= audio_signal->devout->current_lock)
    goto ags_audio_signal_remove_unlocked_buffer0;

  for(i = audio_signal->lock_attack; i < AGS_STREAM_LOCK_SIZE; i++, audio_signal->lock_attack = i){
    if(((AgsStreamLock *) audio_signal->lock->data)->lock[i] == ~0)
      continue;
    else if(((AgsStreamLock *) audio_signal->lock->data)->lock[i] == 0){
      if(audio_signal->stream->next != NULL){
	stream_prev = audio_signal->stream;
	audio_signal->stream = audio_signal->stream->next;
	((AgsStreamLock *) audio_signal->lock->data)->lock[i] = ~0;
	audio_signal->stream->prev = NULL;
	free(stream_prev->data);
	free(stream_prev);
      }else{
	ags_audio_signal_remove_unlocked_buffer_empty();
	return;
      }
    }else{
      i++;
      audio_signal->lock_attack = i;
      stream = audio_signal->stream->next;

      if(stream == NULL)
	return;
      else
	goto ags_audio_signal_remove_unlocked_buffer1;
    }
  }

  lock_prev = audio_signal->lock;
  audio_signal->lock = audio_signal->lock->next;
  free(lock_prev->data);
  free(lock_prev);
  audio_signal->lock->prev = NULL;

  while(!(((AgsStreamLock *) lock->data)->start < audio_signal->devout->offset && ((AgsStreamLock *) lock->data)->stop < audio_signal->devout->lock_offset)){
    for(i = 0; i < AGS_STREAM_LOCK_SIZE && lock != audio_signal->current_lock; i++, audio_signal->lock_attack = i){
      if(((AgsStreamLock *) lock->data)->lock[i] == ~0)
	continue;
      else if(((AgsStreamLock *) lock->data)->lock[i] == 0){
	if(audio_signal->stream->next != NULL){
	  stream_prev = audio_signal->stream;
	  audio_signal->stream = audio_signal->stream->next;
	  ((AgsStreamLock *) audio_signal->lock->data)->lock[i] = ~0;
	  audio_signal->stream->prev = NULL;
	  free(stream_prev->data);
	  free(stream_prev);
	}else{
	  ags_audio_signal_remove_unlocked_buffer_empty();
	  return;
	}
      }else{
	i++;
	audio_signal->lock_attack = i;
	stream = audio_signal->stream->next;

	if(stream == NULL)
	  return;
	else
	  goto ags_audio_signal_remove_unlocked_buffer3;
      }
    }

    lock_prev = lock;
    audio_signal->lock =
      lock = lock->next;
    free(lock_prev->data);
    free(lock_prev);
    lock->prev = NULL;
  }

 ags_audio_signal_remove_unlocked_buffer0:
  for(i = 0; i < audio_signal->devout->current_lock; i++, audio_signal->lock_attack = i){
    if(((AgsStreamLock *) lock->data)->lock[i] == ~0)
      continue;
    else if(((AgsStreamLock *) lock->data)->lock[i] == 0){
      if(audio_signal->stream->next != NULL){
	stream_prev = audio_signal->stream;
	audio_signal->stream = audio_signal->stream->next;
	((AgsStreamLock *) audio_signal->lock->data)->lock[i] = ~0;
	audio_signal->stream->prev = NULL;
	free(stream_prev->data);
	free(stream_prev);
      }else{
	ags_audio_signal_remove_unlocked_buffer_empty();
	return;
      }
    }else{
      i++;
      audio_signal->lock_attack = i;
      stream = audio_signal->stream->next;

      if(stream == NULL)
	return;
      else
	goto ags_audio_signal_remove_unlocked_buffer5;
    }
  }

 ags_audio_signal_remove_unlocked_buffer1:
  for(; i < AGS_STREAM_LOCK_SIZE; i++){
    if(((AgsStreamLock *) lock->data)->lock[i] == ~0)
      continue;
    else if(((AgsStreamLock *) lock->data)->lock[i] == 0){
      stream_prev = stream->prev;
      stream_next =
	stream_prev->next = stream->next;

      ((AgsStreamLock *) lock->data)->lock[i] = ~0;

      free(stream->data);
      free(stream);

      if(stream_next != NULL)
	stream_next->prev = stream_prev;
      else
	return;

      stream = stream_next;
    }else{
      stream = stream->next;

      if(stream == NULL)
	return;
    }
  }

  lock = lock->next;

 ags_audio_signal_remove_unlocked_buffer2:
  while(!(((AgsStreamLock *) lock->data)->start < audio_signal->devout->offset && ((AgsStreamLock *) lock->data)->stop < audio_signal->devout->lock_offset)){
    for(i = 0; i < AGS_STREAM_LOCK_SIZE && lock != audio_signal->current_lock; i++){
      if(((guint *) lock->data)[i] == 0){
	stream_prev = stream->prev;
	stream_next =
	  stream_prev->next = stream->next;

	((AgsStreamLock *) lock->data)->lock[i] = ~0;

	free(stream->data);
	free(stream);

	if(stream_next != NULL)
	  stream_next->prev = stream_prev;
	else
	  return;

	stream = stream_next;
      }else if(((guint *) lock->data)[i] == ~0)
	continue;
      else
	goto ags_audio_signal_remove_unlocked_buffer4;
    }

    lock_prev = lock->prev;
    lock_prev->next = lock->next;
    lock_prev->next->prev = lock_prev;
    free(lock->data);
    free(lock);
    lock = lock_prev->next;
  }

  i = 0;
  goto ags_audio_signal_remove_unlocked_buffer5;

 ags_audio_signal_remove_unlocked_buffer3:
  for(; i < AGS_STREAM_LOCK_SIZE; i++){
    if(((AgsStreamLock *) lock->data)->lock[i] == 0){
      stream_prev = stream->prev;
      stream_prev->next =
	stream_next = stream->next;

      ((AgsStreamLock *) lock->data)->lock[i] = ~0;

      free(stream->data);
      free(stream);

      if(stream_next != NULL)
	stream_next->prev = stream_prev;
      else
	return;

      stream = stream_next;
    }else if(((AgsStreamLock *) lock->data)->lock[i] == ~0)
      continue;
    else{
    ags_audio_signal_remove_unlocked_buffer4:
      stream = stream->next;

      if(stream == NULL)
	return;
    }
  }

  lock = lock->next;
  goto ags_audio_signal_remove_unlocked_buffer2;

 ags_audio_signal_remove_unlocked_buffer5:
  for(; i < audio_signal->devout->current_lock; i++){
    if(((AgsStreamLock *) lock->data)->lock[i] == 0){
      stream_prev = stream->prev;
      stream_prev->next =
	stream_next = stream->next;

      ((AgsStreamLock *) lock->data)->lock[i] = ~0;

      free(stream->data);
      free(stream);

      if(stream_next != NULL)
	stream_next->prev = stream_prev;
      else
	return;

      stream = stream_next;
    }else if(((guint *) lock->data)[i] == ~0)
      continue;
    else{
      stream = stream->next;

      if(stream == NULL)
	return;
    }
  }
}

void
ags_audio_signal_copy_buffer_to_buffer(guint *destination, guint *source, guint size)
{
}

void
ags_audio_signal_copy_stream_to_stream(AgsAudioSignal *destination, AgsCounter *doffset, AgsAudioSignal *source, AgsCounter *soffset)
{
}

void
ags_audio_signal_copy_buffer_to_stream(AgsAudioSignal *destination, AgsCounter *offset, guint *source, guint size)
{
}

AgsAudioSignal*
ags_audio_signal_new()
{
  AgsAudioSignal *audio_signal;

  audio_signal = (AgsAudioSignal *) g_object_new(AGS_TYPE_AUDIO_SIGNAL, NULL);

  return(audio_signal);
}
