#ifndef __AGS_AUDIO_SIGNAL_H__
#define __AGS_AUDIO_SIGNAL_H__

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <glib/glist.h>

#include "ags_devout.h"
#include "ags_counter.h"

#define AGS_TYPE_AUDIO_SIGNAL                (ags_audio_signal_get_type())
#define AGS_AUDIO_SIGNAL(obj)                ((AgsAudioSignal*) G_TYPE_CHECK_INSTANCE_CAST(obj, AGS_TYPE_AUDIO_SIGNAL, AgsAudioSignal))
#define AGS_AUDIO_SIGNAL_CLASS(class)        ((AgsAudioSignalClass*) G_TYPE_CHECK_CLASS_CAST(class, AGS_TYPE_AUDIO_SIGNAL, AgsAudioSignalClass))

typedef struct _AgsAudioSignal AgsAudioSignal;
typedef struct _AgsAudioSignalClass AgsAudioSignalClass;
typedef struct _AgsStreamLock AgsStreamLock;

typedef enum{
  AGS_AUDIO_SIGNAL_USE_COUNTER,
  AGS_AUDIO_SIGNAL_STOPS_FIRST, // if attack[0] is smaller as attack[1], use a *current == &attack[0] to determine if current attack is smaller
  AGS_AUDIO_SIGNAL_SET_LOCK_OFFSET, // if in struct AgsStreamLock for start and stop AgsOffset should be alloced
}AgsAudioSignalFlags;

struct _AgsAudioSignal
{
  GObject object;

  guint flags;

  AgsDevout *devout;

  AgsOffset *start;
  AgsOffset *stop;
  guint *current; // current switches between attack[0] and attack[1]

  guint attack[3]; // attack[2] is for parent AudioSignal
  GList *stream;
  GList *current_stream; // can be NULL

  GList *prev_stream;
  GList *next_stream;

  guint lock_attack; // lock_attack is where ags_audio_signal_remove_unlocked_buffer is currently or has stopped, devout->lock_attack is for initialization of this value, current_lock[devout->current_lock] gives you the current lock
  GList *lock;
  GList *current_lock; // can be NULL
};

struct _AgsAudioSignalClass
{
  GObjectClass object;
};

struct _AgsStreamLock
{
  guint lock[32];

  AgsOffset *start;
  AgsOffset *stop;
};

void ags_audio_signal_add_stream(AgsAudioSignal *audio_signal); // don't use this function else take care of memory leak use rather ags_audio_signal_resize_stream
void ags_audio_signal_resize_stream(AgsAudioSignal *audio_signal, AgsCounter *stop); // don't use this function

void ags_audio_signal_add_stream_lock(AgsAudioSignal *audio_signal);
void ags_audio_signal_resize_stream_lock(AgsAudioSignal *audio_signal, AgsCounter *stop);

void ags_audio_signal_lock(AgsAudioSignal *audio_signal, AgsCounter *start, AgsCounter *stop); // don't use this function, do it rather manually
void ags_audio_signal_remove_unlocked_buffer(AgsAudioSignal *audio_signal);

void ags_audio_signal_copy_buffer_to_buffer(guint *destination, guint *source, guint size);
void ags_audio_signal_copy_stream_to_stream(AgsAudioSignal *destination, AgsCounter *doffset, AgsAudioSignal *source, AgsCounter *soffset); // this function shouldn't be used, use ags_copy_buffer_to_buffer instead
void ags_audio_signal_copy_buffer_to_stream(AgsAudioSignal *destination, AgsCounter *offset, guint *source, guint size);

AgsAudioSignal* ags_audio_signal_new();

#endif /*__AGS_AUDIO_SIGNAL_H__*/
