#include "ags_channel.h"

#include "ags_pattern.h"
#include "ags_recall.h"

#include <stdio.h>

GType ags_channel_get_type();
void ags_channel_class_init (AgsChannelClass *channel_class);
void ags_channel_init (AgsChannel *channel);
void ags_channel_destroy (GObject *gobject);

extern void ags_file_write_channel(AgsFile *file, AgsChannel *channel);

GType
ags_channel_get_type (void)
{
  static GType ags_type_channel = 0;

  if(!ags_type_channel){
    static const GTypeInfo ags_channel_info = {
      sizeof (AgsChannelClass),
      NULL, /* base_init */
      NULL, /* base_finalize */
      (GClassInitFunc) ags_channel_class_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      sizeof (AgsChannel),
      0,    /* n_preallocs */
      (GInstanceInitFunc) ags_channel_init,
    };
    ags_type_channel = g_type_register_static(G_TYPE_OBJECT, "AgsChannel\0", &ags_channel_info, 0);
  }
  return (ags_type_channel);
}

void
ags_channel_class_init(AgsChannelClass *channel)
{
  channel->write_file = ags_file_write_channel;
}

void
ags_channel_init(AgsChannel *channel)
{
  channel->audio = NULL;

  channel->prev = NULL;
  channel->prev_pad = NULL;
  channel->next = NULL;
  channel->next_pad = NULL;

  channel->pad = 0;
  channel->audio_channel = 0;

  channel->recall = NULL;

  channel->link = NULL;
  channel->first_recycling = NULL;
  channel->last_recycling = NULL;

  channel->pattern = NULL;
  channel->notation = NULL;

  channel->line_widget = NULL;
  channel->file_data = NULL;
}

void
ags_channel_destroy(GObject *gobject)
{
  AgsChannel *channel = (AgsChannel *) gobject;
  GList *list0, *list1;

  ags_channel_set_link(channel, NULL);

  if(channel->prev != NULL)
    channel->prev->next = channel->next;

  if(channel->prev_pad != NULL)
    channel->prev_pad->next_pad = channel->next_pad;

  if(channel->next != NULL)
    channel->next->prev = channel->prev;

  if(channel->next_pad != NULL)
    channel->next_pad->prev_pad = channel->prev_pad;

  list0 = channel->recall;

  while(list0 != NULL){
    list1 = list0->next;
    ags_recall_destroy((AgsRecall *) list0->data);
    list0 = list1;
  }

  list0 = channel->pattern;

  while(list0 != NULL){
    list1 = list0->next;
    ags_pattern_destroy((AgsPattern *) list0->data);
    list0 = list1;
  }

  if(channel->notation != NULL)
    ags_notation_destroy(channel->notation);

  g_object_unref (gobject);
}

AgsRecall*
ags_channel_find_recall(AgsChannel *channel, char *effect, char *name)
{
}

AgsChannel*
ags_channel_last(AgsChannel *channel)
{
  while(channel->next_pad != NULL)
    channel = channel->next_pad;

  while(channel->next != NULL)
    channel = channel->next;

  return(channel);
}

AgsChannel*
ags_channel_nth(AgsChannel *channel, guint nth)
{
  int i;

  for(i = 0; i < nth && channel != NULL; i++)
    channel = channel->next;

  if(i != nth)
    fprintf(stderr, "nth channel does not exist\n\0");

  return(channel);
}

void
ags_channel_set_link(AgsChannel *channel, AgsChannel *link)
{
}

AgsChannel*
ags_channel_new()
{
  AgsChannel *channel;

  channel = (AgsChannel *) g_object_new(AGS_TYPE_CHANNEL, NULL);

  return(channel);
}
