#ifndef __AGS_NOTATION_H__
#define __AGS_NOTATION_H__

#include <glib.h>
#include <glib-object.h>

#include "time/ags_offset.h"

#define AGS_TYPE_NOTATION            (ags_notation_get_type())
#define AGS_NOTATION(obj)            (G_TYPE_CHECK_INSTANCE_CAST(obj, AGS_TYPE_NOTATION, AgsNotation))
#define AGS_NOTATION_CLASS(class)    (G_TYPE_CHECK_CLASS_CAST(class, AGS_TYPE_NOTATION, AgsNotationClass))

typedef struct _AgsNotation AgsNotation;
typedef struct _AgsNotationClass AgsNotationClass;
typedef struct _AgsNote AgsNote;

typedef enum{
  AGS_NOTATION_RASTER,
}AgsNotationFlags;

struct _AgsNotation
{
  GObject object;

  guint flags;

  GObject *audio;

  char *key;
  guint base_frequence;

  char *tact;
  guint beat_size; // frame count, beat_size = devout->frequence * 60 / bpm

  GList *note;
};

struct _AgsNotationClass
{
  GObjectClass object;

  void (*add)(AgsNotation *notation, guint frequence, AgsOffset *start, AgsOffset *stop);
  void (*remove)(AgsNotation *notation, guint frequence, AgsOffset *start, AgsOffset *stop);

  void (*add_note_human_readable)(AgsNotation *notation, char *name, AgsOffset *beat_start, AgsOffset *beat_duration);
  void (*remove_note_human_readable)(AgsNotation *notation, char *name, AgsOffset *beat_start, AgsOffset *beat_duration);
};

struct _AgsNote
{
  // human readable
  char *name; // eg. (a' as' b') (-)
  AgsOffset *beat_start;
  AgsOffset *beat_duration;

 // runtime format
  guint frequence; // in Hz 
  AgsOffset *start;
  AgsOffset *stop;
};

void ags_note_convert_to_runtime_offset(AgsNote *note);
void ags_note_convert_to_human_readable(AgsNote *note);

void ags_notation_note_conversion_to_runtime_offset(AgsNotation *notation);
void ags_notation_note_conversion_to_human_readable(AgsNotation *notation);

AgsNotation* ags_notation_new();

#endif /*__AGS_NOTATION_H__*/
