#include "ags_math.h"

typedef enum
{
  AGS_MATH_EQUAL,
  AGS_MATH_LESS,
  AGS_MATH_MORE,
  AGS_MATH_SIGN,
};

#define AGS_MATH_EQUAL_LESS(a, b, value)         ((AGS_MATH_MORE & (value = _ags_math_equal(a, b))) || ((AGS_MATH_LESS | AGS_MATH_SIGN) & VALUE) || (AGS_MATH_EQUAL & value)) ? TRUE: FALSE)
#define AGS_MATH_EQUAL_MORE(a, b, value)         ((AGS_MATH_LESS & (value = _ags_math_equal(a, b))) || ((AGS_MATH_MORE | AGS_MATH_SIGN) & VALUE) || (AGS_MATH_EQUAL & value)) ? TRUE: FALSE)
#define AGS_MATH_EQUAL_RESPECT_SIGN(a, b, value) (((AGS_MATH_EQUAL | AGS_MATH_SIGN) & (value = _ags_math_equal(a, b))) ? FALSE: TRUE)
#define ags_math_equal_less(a, b)                (!ags_math_more(a, b))
#define ags_math_equal_more(a, b)                (!ags_math_less(a, b))
#define ags_math_equal_less_list(a, b)           (!ags_math_more_list(a, b))
#define ags_math_equal_more_list(a, b)           (!ags_math_less_list(a, b))

GList*
ags_math_width_list(GList *stop)
{
  GList *width0, *width1, *word, *list, *one;
  guint value;

  one = g_list_alloc();
  one->data = (gpointer) (1 | (1 << (sizeof(uint) * 8 -1)));
  word = g_list_alloc();
  word->data = (gpointer) (sizeof(uint) * 8);
  list = g_list_alloc();
  list->data = (gpointer) ags_list_lenght_backward(stop);
  width0 = ags_math_multiplication_list(word, list);
  free(word);
  free(list);
  width1 = width0;
  width0 = ags_math_subtraction_list(width0, one);
  g_list_destroy(g_list_first(width1));

#ifdef __AGS_MATH_NO_LEADING_ZEROS__
  list = g_list_first(stop);
#elif
  list = ags_math_cut_leading_zeros(stop);
#endif

  goto ags_math_width_list0;

  while(TRUE){
    for(value = 1 << (sizeof(uint) * 8 -1);;){
      if(value & ((uint) list->data))
	goto ags_math_width_list1;

      width1 = width0;
      width0 = ags_math_subtraction_list(width0, one);
      g_list_destroy(g_list_first(width1));
    ags_math_width_list0:

      if(value == 1)
	break;

      value = value >> 1;
    }

    if(list == stop)
      break;

    list = list->next;
  }

 ags_math_width_list1:

  return(width0);
}

GList*
ags_math_cut_leading_zeros(GList *stop)
{
  GList *start, *list;
  guint value = 1 << (sizeof(guint) * 8 -1);
  gboolean sign;

  if(stop->prev == NULL)
    return(stop);

  list =
    start = g_list_first(stop);

  if(value & ((guint) start->data))
    if(value ^ ((guint) start->data))
      return(stop);
    else{
      sign = TRUE;
      list = list->next;
    }

  while(list != stop && ((guint) list->data) != 0){
    list = list->next;
    free(list->prev);
  }

  if(sign){
    if(list == stop){
      start->next = stop->next;
      free(stop);
      stop = start;

      if(stop->next != NULL)
	stop->next->prev = stop;
    }else if(value & ((guint) list->data)){
      list->prev = start;
      start->next = list;
    }else{
      list->prev = NULL;
      list->data = (gpointer) (value | ((guint ) list->data));
      free(start);
      start = list;
    }
  }else{
    if(value & ((guint) list->data)){
      list->prev = g_list_alloc();
      start = list->prev;
      start->next = list;
      start->data = 0;
    }else
      start = list;
  }

  return(start);
}


guint
_ags_math_equal(int a, int b)
{
  guint value0, value1;
  gboolean sign, sign_equal;
  gboolean jump;
  void __ags_math_equal(){
    if(value0 & a){
      if(!(value0 & b))
	if(sign_equal){
	  value1 |= AGS_MATH_MORE;
	  jump = TRUE;
	}else if(sign){
	  value1 |= AGS_MATH_LESS;
	  jump = TRUE;
	}else{
	  value1 |= AGS_MATH_MORE;
	  jump = TRUE;
	}
    }else if(value0 & b)
      if(sign_equal){
	value1 |= AGS_MATH_LESS;
	jump = TRUE;
      }else if(sign){
	value1 |= AGS_MATH_MORE;
	jump = TRUE;
      }else{
	value1 |= AGS_MATH_LESS;
	jump = TRUE;
      }
  }

  value0 = (1 << (sizeof(uint) * 8 -1));
  value1 = 0;

  if((value0 & a) && !(value0 & b)){
    value1 |= AGS_MATH_SIGN;
    sign = TRUE;
    
  }else if(!(value0 & a) && (value0 & b)){
    value1 |= AGS_MATH_SIGN;
    sign = FALSE;
  }else{
    sign_equal = TRUE;
    goto _ags_math_equal0;
  }
  sign_equal = FALSE;
 _ags_math_equal0:
  value0 >>= 1;

  jump = FALSE;

  for(; value0 != 1;){
    __ags_math_equal();
    if(jump)
      goto _ags_math_equal1;
    value0 >>= 1;
  }
  __ags_math_equal();
    if(jump)
      goto _ags_math_equal1;

  value1 |= AGS_MATH_EQUAL;

 _ags_math_equal1:

  return(value1);
}

gboolean
ags_math_equal(int a, int b)
{
  if(b^a)
    return(FALSE);
  else
    return(TRUE);
}

guint
_ags_math_equal_list(GList *list_a, GList *a_next, GList *list_b, GList *b_next)
{
  GList *list0, *list1;
  guint value, start;

  list0 = list_a;
  list1 = list_b;

  while(list0 != a_next && list1 != b_next){
    list0 = list0->next;
    list1 = list1->next;
  }

  if(list0 != a_next)
    return(2);

  if(list1 != b_next)
    return(0);

  start = 1 << (sizeof(uint) * 8 -1);

  while(list_a != a_next && list_b != b_next){
    for(value = start; value != 1;){
      if(value & (uint) list_a->data){
	if(!(value & (uint) list_b->data))
	  return(0);
      }else if(value & (uint) list_b->data)
	return(2);
    }

    list_a = list_a->next;
    list_b = list_b->next;
  }

  return(1);
}

gboolean
ags_math_equal_list(GList *a, GList *b)
{
#ifdef __AGS_MATH_LIST_NO_LEADING_ZEROS__
  while(a != NULL && b != NULL){
    if(a->data != b->data)
      return(FALSE);

    a = a->prev;
    b = b->prev;
  }

  if(a != NULL || b != NULL)
    return(FALSE);
  else
    return(TRUE);
#else
  guint value0, value1;

  while(a->prev != NULL && b->prev != NULL){
    if(a->data != b->data)
      return(FALSE);

    a = a->prev;
    b = b->prev;
  }

  value0 = 1 << (sizeof(uint) * 8 -1);
  value1 = 1;

  if((value1 & ((uint) a->data)) != (value1 & ((uint) b->data)))
    return(FALSE);

  if(a->prev != NULL){
    if(value0 & ((uint) b->data)){
      if(value0 & ((uint) a->data))
	return(FALSE);

      return(ags_math_leading_zeros(a->prev, TRUE));
    }else
      return(ags_math_leading_zeros(a->prev, FALSE));
  }else if(b->prev != NULL){
    if(value0 & ((uint) a->data)){
      if(value0 & ((uint) b->data))
	return(FALSE);

      return(ags_math_leading_zeros(a->prev, FALSE));
    }else
      return(ags_math_leading_zeros(b->prev, TRUE));
  }else
    return(TRUE);
#endif
}

gboolean
_ags_math_more(uint a, uint b)
{
  guint value;

  value = 1 <<(sizeof(int) * 8 -1);

  for(; value != 1;){
    ags_math_more_bit(a, b, value);
    value = value >> 1;
  }

  if((value & a) && !(value & b))
    return(TRUE);
  else
    return(FALSE);
}

gboolean
ags_math_more(int a, int b)
{
  guint value;

  value = 1 << (sizeof(int) * 8 -1);

  ags_math_less_bit(a, b, value);

  for(; value != 1;){
    value = value >> 1;
    ags_math_more_bit(a, b, value);
  } 

  return(FALSE);
}

gboolean
_ags_math_more_list(GList *list_a, GList *a_next, GList *list_b, GList *b_next)
{
  GList *list0, *list1;
  guint value, start;

  list0 = list_a;
  list1 = list_b;

  while(list0 != a_next && list1 != b_next){
    list0 = list0->next;
    list1 = list1->next;
  }

  if(list0 != a_next)
    return(TRUE);
  else if(list1 != b_next)
    return(FALSE);

  start = 1 << (sizeof(uint) * 8 -1);

  while(list_a != a_next && list_b != b_next){
    for(value = start; value != 1;){
      ags_math_more_bit((uint) list_a->data, (uint) list_b->data, value);

      value = value >> 1;
    }

    ags_math_more_bit((uint) list_a->data, (uint) list_b->data, value);

    list_a = list_a->next;
    list_b = list_b->next;
  }

  if(list_a == NULL)
    return(FALSE);
  else
    return(TRUE);
}

gboolean
ags_math_more_list(GList *a, GList *b)
{
  GList *list_a, *list_b;
  guint value = 1 << (sizeof(uint) * 8 -1);

#ifdef __AGS_MATH_NO_LEADING_ZEROS__
  list_a = g_list_first(a);
  list_b = g_list_first(b);
#elif
  list_a = ags_math_cut_leading_zeros(a, TRUE);
  list_b = ags_math_cut_leading_zeros(b, TRUE);
#endif

  ags_math_less_bit((uint) list_a->data, (uint) list_b->data, value);

  if(list_a->data == 0)
    list_a = list_a->next;

  if(list_b->data == 0)
    list_b = list_b->next;

  return(_ags_math_more_list(list_a, a->next, list_b, b->next));
}

gboolean
ags_math_more_equal_list(GList *a, GList *b)
{
  GList *list_a, *list_b;
  guint value0, value1;

#ifdef __AGS_MATH_NO_LEADING_ZEROS__
  list_a = g_list_first(a);
  list_b = g_list_first(b);
#elif
  list_a = ags_math_cut_leading_zeros(a);
  list_b = ags_math_cut_leading_zeros(b);
#endif

  value0 = 1 << (sizeof(uint) * 8 -1);

  ags_math_less_bit((uint) list_a->data, (uint) list_b->data, value0);

  value1 = _ags_math_equal_list(list_a, a->next, list_b, b->next);

  switch(value1){
  case 2:
  case 1:
    return(TRUE);
  }

  return(FALSE);
}

gboolean
_ags_math_less(uint a, uint b)
{
  guint value;

  value = 1 << (sizeof(int) * 8 -1);

  for(; value != 1;){
    ags_math_less_bit(a, b, value);
    value = value >> 1;
  }

  if(!(value & a) && (value & b))
    return(TRUE);
  else
    return(FALSE);
}

gboolean
ags_math_less(int a, int b)
{
  guint value, sign;

  sign = 1 << (sizeof(int) * 8 -1);

  ags_math_more_bit(a, b, value);

  for(value = sign; value != 1;){
    value = value >> 1;
    ags_math_less_bit(a, b, value);
  } 

  return(FALSE);
}

gboolean
_ags_math_less_list(GList *list_a, GList *a_next, GList *list_b, GList *b_next)
{
  GList *list0,  *list1;
  guint value, start;

  start = 1 << (sizeof(uint) * 8 -1);

  list0 = list_a;
  list1 = list_b;
  
  while(list0 != a_next && list1 != b_next){
    list0 = list0->next;
    list1 = list1->next;
  }

  if(list0 != a_next)
    return(FALSE);
  else if(list1 != b_next)
    return(TRUE);

  if(start == 1)
    goto _ags_math_less_list1;
  else{
    value = start >> 1;
    goto _ags_math_less_list0;
  }

  while(list_a != a_next && list_b != b_next){
    for(value = start; value != 1;){
    _ags_math_less_list0:
      ags_math_less_bit((uint) list_a->data, (uint) list_b->data, value);

      value = value >> 1;
    }

  _ags_math_less_list1:
    ags_math_less_bit((uint) list_a->data, (uint) list_b->data, value);

    list0 = list0->next;
    list1 = list1->next;
  }

  if(list_b == b_next)
    return(FALSE);
  else
    return(TRUE);
}

gboolean
ags_math_less_list(GList *a, GList *b)
{
  GList *list_a, *list_b;
  guint value;

  value = 1 << (sizeof(uint) * 8 -1);

#ifdef __AGS_MATH_NO_LEADING_ZEROS__
  list_a = g_list_first(a);
  list_b = g_list_first(b);
#elif
  list_a = ags_math_cut_leading_zeros(a);
  list_b = ags_math_cut_leading_zeros(b);
#endif

  ags_math_more_bit((uint) list_a->data, (uint) list_b->data, value);

  if(((uint) list_a->data) == 0 && list_a != a)
    list_a = list_a->next;

  if(((uint) list_b->data) == 0 && list_b != b)
    list_b = list_b->next;

  return(_ags_math_less_list(list_a, a->next, list_b, b->next));
}

gboolean
ags_math_less_equal_list(GList *a, GList *b)
{
  GList *list_a, *list_b;
  guint value0, value1;

#ifdef __AGS_MATH_NO_LEADING_ZEROS__
  list_a = g_list_first(a);
  list_b = g_list_first(b);
#elif
  list_a = ags_math_cut_leading_zeros(a);
  list_b = ags_math_cut_leading_zeros(b);
#endif

  value0 = 1 << (sizeof(uint) * 8 -1);

  ags_math_more_bit((uint) list_a->data, (uint) list_b->data, value0);

  value1 = _ags_math_equal_list(list_a, a->next, list_b, b->next);

  switch(value1){
  case 0:
  case 1:
    return(TRUE);
  }

  return(FALSE);
}

void
_ags_math_addition(int a, int b, guint *destination, guint values)
{
  int value0;
  int solution;
  gboolean keep;
  guint value1, i;

  solution = 0;

  if((1 << 2) & values)
    keep = destination[1];
  else
    keep = FALSE;

  value0 = 1;

  for(; value0 != (1 << (sizeof(int) * 8 -1));){
    if(keep){
      if(value0 & a && value0 & b){
	solution |= value0;
	keep = 1;
      }else if(value0 & a || value0 & b)
	keep = 1;
      else{
	solution |= value0;
	keep = 0;
      }
    }else{
      if(value0 & a && value0 & b)
	keep = 1;
      else if(value0 & a || value0 & b){
	solution |= value0;
	keep = 0;
      }
    }

    value0 = value0<<1;
  }

  value1 = 1;
  i = 0;

  if(value1 & values){
    destination[0] = solution;
    i++;
  }

  value1 = 1 << 1;

  if(value1 & values)
    destination[i] = keep;
}

void
ags_math_uaddition(guint a, guint b, guint *destination, guint values)
{
  guint pntr[0];
  guint solution;
  gboolean keep;
  guint value0, value1;
  guint i;

  value0 = 1 << (sizeof(guint) * 8 -1);
  value1 = 1 << 1;

  if(value1 & values)
    keep = destination[1];
  else
    keep = FALSE;

  _ags_math_addition(a, b, pntr, (1 | (1 << 1)));

  if(keep){
    if(value0 & a){
      if(value0 & b)
	solution |= value0;

      keep = TRUE;
    }else
      if(value0 & b)
	keep = TRUE;
      else
	solution |= value0;
  }else{
    if(value0 & a){
      if(value0 & b)
	keep = TRUE;
      else
	solution |= value0;
    }else if(value0 & b)
      solution |= value0;
  }

  i = 0;
  value1 = 1;

  if(value1 & values){
    destination[i] = solution;
    i++;
  }

  value1 = 1 << 1;

  if(value1 & values)
    destination[i] = keep;
}

void
_ags_math_addition_list(GList *a, GList *b, guint *destination, guint values)
{
  GList *list, *tmp, *start, *zero;
  guint pntr[0];
  guint part;
  gboolean keep;
  guint value0, value1;
  guint i;

  void __ags_math_addition_list(){
    ags_math_uaddition((guint) a->data, (guint) b->data, pntr, value1);
    list->data = (gpointer) part;
  }

  void __ags_math_addition_list_zero(){
    ags_math_uaddition((guint) tmp->data, (guint) zero->data, pntr, value1);
    list->data = (gpointer) part;
  }

  keep = FALSE;

  if(a == NULL || b == NULL){
    keep = FALSE;
    start = NULL;
    list = NULL;
    goto _ags_math_addition_listEND;
  }

  start =
    list = g_list_alloc();
  value1 = (1) | (1 << 1);
  __ags_math_addition_list(a, b, pntr, value1, &part, list);

  while(a->prev != NULL && b->prev != NULL){
    a = a->prev;
    b = b->prev;
    list->prev = g_list_alloc();
    list->prev->next = list;
    list = list->prev;
    __ags_math_addition_list(a, b, pntr, value1, &part, list);
  }

  list->prev = g_list_alloc();
  list->prev->next = list;
  list = list->prev;

  _ags_math_addition((guint) a->data, (guint) b->data, pntr, value1);
  list->data = (gpointer) part;

  value0 = 1 << (sizeof(guint) * 8 -1);

  if(b->prev != NULL){
    b = b->prev;
    tmp = a;
    a = zero;
  }else if(a->prev != NULL){
    a = a->prev;
    tmp = b;
    b = zero;
  }else
    goto _ags_math_addition_listEND;

  if(keep){
    if(!(value0 & ((guint) tmp->data))){
      list->data = (gpointer) (value0 | ((guint) list->data));
      keep = FALSE;
    }
  }else if(value0 & ((guint) tmp->data))
    list->data = (gpointer) (value0 | ((guint) list->data));

  zero->prev = g_list_alloc();
  zero->data = (guint) 0;

  while(tmp->prev != NULL){
    list->prev = g_list_alloc();
    list->prev->next = list;
    list = list->prev;

    __ags_math_addition_list_zero(tmp, zero, pntr, value1, &part, list);

    tmp = tmp->prev;
  }

  _ags_math_addition((guint) tmp->data, (guint) zero->data, pntr, value1);
  free(zero);

  if(keep){
    list->data = (gpointer) (value0 | ((guint) list->data));
    list->prev = g_list_alloc();
    list->prev->next = list;
    list = list->prev;
  }

  if(a->prev != NULL)
    a = tmp;
  else
    b = tmp;

 _ags_math_addition_listEND:
  value1 = 1;
  i = 0;

  if(value1 & values){
    destination[i] = (guint) start;
    i++;
  }

  value1 <<= 1;

  if(value1 & values){
    destination[i] = (guint) list;
    i++;
  }

  value1 <<= 1;

  if(value1 & values){
    destination[i] = (guint) a;
    i++;
  }

  value1 <<= 1;

  if(value1 & values){
    destination[i] = (guint) b;
    i++;
  }
}

void
__ags_math_addition_list(GList *a, GList *b, guint *pntr, guint value1, guint *part, GList *list)
{
  ags_math_uaddition((guint) a->data, (guint) b->data, pntr, value1);
  list->data = (gpointer) (*part);
}

void
__ags_math_addition_list_zero(GList *tmp, GList *zero, guint *pntr, guint value1, guint *part, GList *list)
{
  ags_math_uaddition((guint) tmp->data, (guint) zero->data, pntr, value1);
  list->data = (gpointer) (*part);
}

GList*
ags_math_uaddition_list(GList *a, GList *b)
{
  guint pntr[0];
  GList *point;

  _ags_math_addition_list(a, b, pntr, 1);

  return(point);
}


void
_ags_math_subtraction(int a, int b, guint *destination, guint values)
{
  int solution;
  gboolean keep;
  guint value0, stop;
  guint value1, i;

  stop = 1 << (sizeof(guint) * 8 -1);
  if(values & (1 << 2))
    keep = destination[1];
  solution = 0;

  for(value0 = 1; value0 != stop;){
    if(value0 & a){
      if(value0 & b){
	if(keep)
	  solution |= value0;
      }else if(!keep){
	solution |= value0;
	keep = TRUE;
      }
    }else if(value0 & b){
      if(!keep)
	solution |= value0;
    }else if(keep)
      solution |= value0;

    value0 = value0 << 1;
  }

  value1 = 1;
  i = 0;

  if(value1 & values){
    destination[i] = solution;
    i++;
  }

  value1 = 1 << 1;

  if(value1 & values)
    destination[i] = keep;
}

void
ags_math_usubtraction(guint a, guint b, guint *destination, guint values)
{
  guint pntr[0];
  guint solution;
  gboolean keep;
  guint value0;
  guint value1, i;

  if((1 << 2) & values)
    keep = destination[1];

  _ags_math_subtraction(a, b, pntr, (1 | (1 << 1)));
  value0 = 1 << (sizeof(guint) * 8 -1);

  if(keep){
    if(value0 & a){
      if(value0 & b)
	solution |= value0;

      keep = TRUE;
    }else if(value0 & b)
      keep = TRUE;
    else
      solution |= value0;
  }else{
    if(value0 & b){
      if(value0 & a)
	keep = TRUE;
      else
	solution |= value0;
    }else if(value0 & a)
      solution |= value0;
  }

  value1 = 1;
  i = 0;
  if(value1 & values){
    destination[i] = solution;
    i++;
  }

  value1 = 1 << 1;

  if(value1 & values)
    destination[i] = keep;
}

void
_ags_math_subtraction_list(GList *a, GList *b, guint *destination, guint values)
{
  GList *point, *list,  *zero;
  guint pntr[0];
  guint part;
  gboolean keep;
  guint value1, i;

  zero = g_list_alloc();
  zero->data = (guint) 0;

  point =
    list = g_list_alloc();
  list->data = 0;

  if((1 << 2) & values)
    keep = destination[1];
  else
    keep = FALSE;

  value1 = 1 | (1 << 1);

  if(a == NULL && b == NULL){
    if(!keep)
      goto _ags_math_subtraction_listEND;
    else{
      list =
	point = NULL;
      goto _ags_math_subtraction_listEND;
    }
  }else{
    if(a == NULL)
      goto _ags_math_subtraction_list2;
    else if(b == NULL)
      goto _ags_math_subtraction_list3;
    else if(a->prev == NULL || b->prev == NULL)
      goto _ags_math_subtraction_list1;
  }

  while(a->prev != NULL && b->prev != NULL){
    list->prev = g_list_alloc();
    list->prev->next = list;
    list = list->prev;
  _ags_math_subtracion_list0:
    ags_math_usubtraction((guint) a->data, (guint) b->data, pntr, value1);
    list->data = (gpointer) part;

    a = a->prev;
    b = b->prev;
  }

  list->prev = g_list_alloc();
  list->prev->next = list;
  list = list->prev;
 _ags_math_subtraction_list1:
  _ags_math_subtraction((guint) a->data, (guint) b->data, pntr, value1);
  list->data = (gpointer) part;

  a = a->prev;
  b = b->prev;

  while(a != NULL){
    list->prev = g_list_alloc();
    list->prev->next = list;
    list = list->prev;
  _ags_math_subtraction_list2:
    ags_math_usubtraction((guint) a->data, (guint) zero->data, pntr, value1);
    list->data = (gpointer) part;

    a = a->prev;
  }

  while(b != NULL){
    list->prev = g_list_alloc();
    list->prev->next = list;
    list = list->prev;
  _ags_math_subtraction_list3:

    ags_math_usubtraction((guint) zero->data, (guint) b->data, pntr, value1);
    list->data = (gpointer) part;

    b = b->prev;
  }

  ags_math_cut_leading_zeros(point);
 _ags_math_subtraction_listEND:

  free(zero);

  value1 = 1;
  i = 0;

  if(value1 & values){
    destination[i] = (guint) point;
    i++;
  }

  value1 <<= 1;

  if(value1 & values){
    destination[i] = (guint) list;
    i++;
  }

  value1 <<= 1;

  if(value1 & values){
    destination[i] = (guint) a;
    i++;
  }

  value1 <<= 1;

  if(value1 & values){
    destination[i] = (guint) b;
    i++;
  }
}

GList*
ags_math_usubtraction_list(GList *a, GList *b)
{
  guint pntr[0];
  GList *list;

  _ags_math_subtraction_list(a, b, pntr, 1);

  return(list);
}
