#ifndef __AGS_OFFSET_H__
#define __AGS_OFFSET_H__

#include <glib.h>
#include <glib-object.h>

#include "ags_offset_uint.h"
#include "ags_offset_list.h"
#include "ags_offset_uint_frame.h"
#include "ags_offset_list_frame.h"
#include "ags_offset_list_frame_double.h"
#include "ags_offset_list_frame_dimensional.h"

#define AGS_TYPE_OFFSET    (ags_offset_get_type())
#define AGS_OFFSET(obj)    (G_TYPE_CHECK_INSTANCE_CAST(obj, AGS_TYPE_OFFSET, AgsOffset))
#define AGS_IS_OFFSET(obj) (G_TYPE_CHECK_INSTANCE_TYPE(obj, AGS_TYPE_OFFSET))

typedef struct _AgsOffsetInstance AgsOffsetInstance;
typedef struct _AgsOffset AgsOffset; // is a wrapper
typedef struct _AgsOffsetClass AgsOffsetClass;

typedef enum{
  AGS_OFFSET_NAN, // not a number, not implemented in relation functions you have to jump into affecting functions
  AGS_OFFSET_BASE,
  AGS_OFFSET_BASE_ABSOLUTE, // base isn't a frame count multiple of frame size. absolute: base + offset = offset_list.offset; relative: (base * frame_size) + offset = offset_list.offset;
  //  AGS_OFFSET_DONT_FREE_ON_DUPLICATED,
}AgsOffsetFlags;

typedef enum{
  AGS_OFFSET_AFFECT_FRAME, // make frames
  AGS_OFFSET_AFFECT_OFFSET, // make offset
  AGS_OFFSET_AFFECT_BASE, // make base, can be combined with AGS_OFFSET_FRAME
}AgsOffsetAffect;

struct _AgsOffsetInstance{
  GObject object;

  guint flags;
  guint affect;
};

struct _AgsOffset{
  AgsOffsetInstance offset_instance;

  union{
    AgsOffsetUInt offset_uint;
    AgsOffsetList offset_list;
    AgsOffsetUIntFrame offset_uint_frame;
    AgsOffsetListFrame offset_list_frame;
    AgsOffsetListFrameDouble offset_list_frame_double;
    AgsOffsetListFrameDimensional offset_list_frame_dimensional;
  }offset_union;
};

struct _AgsOffsetClass{
  GObjectClass object;
};

typedef enum{
  AGS_OFFSET_DUPLICATE_MAXIMUM_SIZE,
  AGS_OFFSET_DUPLICATE_BASE,
  AGS_OFFSET_DUPLICATE_FRAME_SIZE,
  AGS_OFFSET_DUPLICATE_SIZE,
  AGS_OFFSET_DUPLICATE_ATTACK,
  AGS_OFFSET_DUPLICATE_MATRIX,
  AGS_OFFSET_DUPLICATE_LEVEL,
}AgsOffsetDuplicateFlags; // wheter the corresponding entries should be duplicated, default is to not do so and setting pointer to them

typedef enum{
  AGS_OFFSET_EQUAL_EQUAL,
  AGS_OFFSET_EQUAL_KEEP, // only returned if AGS_EQUAL_EQUAL
  AGS_OFFSET_EQUAL_KEEP0,
  AGS_OFFSET_EQUAL_KEEP1,
}AgsOffsetEqualFlags;

//typedef enum{
//  AGS_OFFSET_RELATION_EQUAL,
//  AGS_OFFSET_RELATION_KEEP,
//  AGS_OFFSET_RELATION_KEEP0,
//  AGS_OFFSET_RELATION_KEEP1,
//}AgsOffsetLessFlags,AgsOffsetGreaterFlags;

/*
 * GList numbers are unsigned 
 * relative offset with a absolute or relative base
 * base: NULL means offset 0, type will probably change to AgsOffset
 * one frame has (frame_size * sizeof(guint) * 8) bits, NULL means 1
 * AgsOffsetListFrameDouble and AgsOffsetListDimensional - not implemented yet (maybe never)
 */

gboolean ags_offset_equal(AgsOffset *offset0, AgsOffset *offset1);

gboolean ags_offset_less(AgsOffset *offset0, AgsOffset *offset1);
gboolean ags_offset_greater(AgsOffset *offset0, AgsOffset *offset1);

AgsOffset* ags_offset_addition(AgsOffset *offset0, AgsOffset *offset1, guint flags);
//AgsOffset* ags_offset_addition_equal(AgsOffset *offset0, AgsOffset *offset1);

AgsOffset* ags_offset_subtraction(AgsOffset *offset0, AgsOffset *offset1, guint flags);
//AgsOffset* ags_offset_subtraction_equal(AgsOffset *offset0, AgsOffset *offset1, gboolean exchange); / / if exchange == TRUE value will be written into offset1

#define ags_offset_not_equal(offset0, offset1) (~ ags_offset_equal(offset0, offset1))

#define ags_offset_less_equal(offset0, offset1) (!ags_offset_greater(offset0, offset1))
#define ags_offset_greater_equal(offset0, offset1) (!ags_offset_less(offset0, offset1))

AgsOffsetList* ags_offset_base_get_absolute(AgsOffset *base, AgsOffset *offset); // simple conversion, probably you may use the function below
AgsOffsetList* ags_offset_get_base_absolute(AgsOffset *offset); // offset keeps base which may have several levels - splitted base to more bases
AgsOffset* ags_offset_to_offset(AgsOffset *offset, GType type, GList *maximum_size, GList *frame_size, GList *base_count, ...); // convert to different types, optionally can recive multiple maximum_size's, this va_arg must be NULL terminated

AgsOffset* ags_offset_new(GType type);
AgsOffset* ags_offset_duplicate(AgsOffset *offset, guint flags);

#endif /*__AGS_OFFSET_H__*/
