#include "ags_file.h"

#include <string.h>
#include <libxml/parser.h>
#include <libxml/xlink.h>
#include <libxml/xpath.h>
#include <libxml/valid.h>
#include <libxml/xmlIO.h>
#include <libxml/xmlmemory.h>
#include <libxml/xmlsave.h>

#include "../main.h"

#include "../audio/ags_audio.h"
#include "../audio/ags_channel.h"
#include "../audio/ags_output.h"
#include "../audio/ags_input.h"
#include "../audio/ags_recycling.h"
#include "../audio/ags_audio_signal.h"
#include "../audio/ags_recall.h"
#include "../audio/ags_effect.h"
#include "../audio/ags_pattern.h"
#include "../audio/ags_notation.h"
#include "../audio/ags_devout.h"

#include "../X/ags_window.h"
#include "../X/ags_menu_bar.h"
#include "../X/ags_machine.h"
#include "../X/ags_pad.h"
#include "../X/ags_line.h"
#include "../X/ags_editor.h"
#include "../X/ags_navigation.h"
#include "../X/ags_machine_editor.h"
#include "../X/ags_pad_editor.h"
#include "../X/ags_line_editor.h"
#include "../X/ags_link_editor.h"
#include "../X/ags_line_member_editor.h"

#include "../X/editor/ags_toolbar.h"
#include "../X/editor/ags_notebook.h"

#include "../X/machine/ags_panel.h"
#include "../X/machine/ags_mixer.h"
#include "../X/machine/ags_drum.h"
#include "../X/machine/ags_matrix.h"
#include "../X/machine/ags_synth.h"
#include "../X/machine/ags_oscillator.h"
#include "../X/machine/ags_ffplayer.h"

GType ags_file_get_type(void);
void ags_file_class_init(AgsFileClass *file);
void ags_file_init (AgsFile *file);
void ags_file_destroy (GObject *gobject);

void ags_file_read_audio(AgsFile *file, AgsAudio *audio);
void ags_file_read_channel(AgsFile *file, AgsChannel *channel, GType type);
void ags_file_read_output(AgsFile *file, AgsOutput *output);
void ags_file_read_input(AgsFile *file, AgsInput *input);
void ags_file_read_recycling(AgsFile *file, AgsRecycling *recycling);
void ags_file_read_audio_signal(AgsFile *file, AgsAudioSignal *audio_signal);
void ags_file_read_stream(AgsFile *file, gpointer stream);
void ags_file_read_stream_lock(AgsFile *file, gpointer stream_lock);
void ags_file_read_recall(AgsFile *file, AgsRecall *recall);
void ags_file_read_pattern(AgsFile *file, AgsPattern *pattern);
void ags_file_read_notation(AgsFile *file, AgsNotation *notation);
void ags_file_read_note(AgsFile *file, AgsNote *note);
void ags_file_read_devout(AgsFile *file, AgsDevout *devout);

void ags_file_read_window(AgsFile *file);
void ags_file_read_menu_bar(AgsFile *file, AgsMenuBar *menu_bar);
void ags_file_read_machine(AgsFile *file);
void ags_file_read_pad(AgsFile *file, AgsPad *pad);
void ags_file_read_line(AgsFile *file, AgsLine *line);
void ags_file_read_editor(AgsFile *file, AgsEditor *editor);
void ags_file_read_navigation(AgsFile *file, AgsNavigation *navigation);
void ags_file_read_machine_editor(AgsFile *file, AgsMachineEditor *machine_editor);
void ags_file_read_pad_editor(AgsFile *file, AgsPadEditor *pad_editor);
void ags_file_read_line_editor(AgsFile *file, AgsLineEditor *line_editor);
void ags_file_read_link_editor(AgsFile *file, AgsLinkEditor *link_editor);
void ags_file_read_line_member_editor(AgsFile *file, AgsLineMemberEditor *line_member_editor);

void ags_file_read_toolbar(AgsFile *file, AgsToolbar *toolbar);
void ags_file_read_notebook(AgsFile *file, AgsNotebook *notebook);

void ags_file_read_panel(AgsFile *file, AgsMachine *machine);
void ags_file_read_mixer(AgsFile *file, AgsMachine *machine);
void ags_file_read_drum(AgsFile *file, AgsMachine *machine);
void ags_file_read_matrix(AgsFile *file, AgsMachine *machine);
void ags_file_read_synth(AgsFile *file, AgsMachine *machine);
void ags_file_read_oscillator(AgsFile *file, AgsOscillator *oscillator);
void ags_file_read_ffplayer(AgsFile *file, AgsMachine *machine);

void ags_file_write_audio(AgsFile *file, AgsAudio *audio);
void ags_file_write_channel(AgsFile *file, AgsChannel *channel);
void ags_file_write_output(AgsFile *file, AgsChannel *channel);
void ags_file_write_input(AgsFile *file, AgsChannel *channel);
void ags_file_write_recycling(AgsFile *file, AgsRecycling *recycling);
void ags_file_write_audio_signal(AgsFile *file, AgsAudioSignal *audio_signal);
void ags_file_write_stream(AgsFile *file, gpointer stream);
void ags_file_write_stream_lock(AgsFile *file, gpointer stream_lock);
void ags_file_write_recall(AgsFile *file, AgsRecall *recall);
void ags_file_write_pattern(AgsFile *file, AgsPattern *pattern);
void ags_file_write_notation(AgsFile *file, AgsNotation *notation);
void ags_file_write_note(AgsFile *file, AgsNote *note);
void ags_file_write_devout(AgsFile *file, AgsDevout *devout);

void ags_file_write_window(AgsFile *file);
void ags_file_write_menu_bar(AgsFile *file, AgsMenuBar *menu_bar);
void ags_file_write_machine(AgsFile *file, AgsMachine *machine);
void ags_file_write_pad(AgsFile *file, AgsPad *pad);
void ags_file_write_line(AgsFile *file, AgsLine *line);
void ags_file_write_editor(AgsFile *file, AgsEditor *editor);
void ags_file_write_navigation(AgsFile *file, AgsNavigation *navigation);
void ags_file_write_machine_editor(AgsFile *file, AgsMachineEditor *machine_editor);
void ags_file_write_pad_editor(AgsFile *file, AgsPadEditor *pad_editor);
void ags_file_write_line_editor(AgsFile *file, AgsLineEditor *line_editor);
void ags_file_write_link_editor(AgsFile *file, AgsLinkEditor *link_editor);
void ags_file_write_line_member_editor(AgsFile *file, AgsLineMemberEditor *line_member_editor);

void ags_file_write_toolbar(AgsFile *file, AgsToolbar *toolbar);
void ags_file_write_notebook(AgsFile *file, AgsNotebook *notebook);

void ags_file_write_panel(AgsFile *file, AgsMachine *machine);
void ags_file_write_mixer(AgsFile *file, AgsMachine *machine);
void ags_file_write_drum(AgsFile *file, AgsMachine *machine);
void ags_file_write_matrix(AgsFile *file, AgsMachine *machine);
void ags_file_write_synth(AgsFile *file, AgsMachine *machine);
void ags_file_write_oscillator(AgsFile *file, AgsOscillator *oscillator);
void ags_file_write_ffplayer(AgsFile *file, AgsMachine *machine);

GType
ags_file_get_type (void)
{
  static GType ags_type_file = 0;
  
  if(!ags_type_file){
    static const GTypeInfo ags_file_info = {
      sizeof (AgsFileClass),
      NULL, /* base_init */
      NULL, /* base_finalize */
      (GClassInitFunc) ags_file_class_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      sizeof (AgsFile),
      0,    /* n_preallocs */
      (GInstanceInitFunc) ags_file_init,
    };
    ags_type_file = g_type_register_static (G_TYPE_OBJECT, "AgsFile\0", &ags_file_info, 0);
  }
  return (ags_type_file);
}

void
ags_file_class_init(AgsFileClass *file)
{
}

void
ags_file_init(AgsFile *file)
{
  file->flags = 0;

  file->name = NULL;
  file->encoding = "UTF-8\0";
  file->dtd = "/usr/share/ags/ags_file.dtd\0";

  file->window = NULL;
}

void
ags_file_destroy(GObject *gobject)
{
  AgsFile *file = (AgsFile *) gobject;
  void ags_file_destroy_node(xmlNode *node){
    xmlNode *child;

    child = node->children;

    while(child != NULL){
      ags_file_destroy_node(child);

      child = child->next;
    }
  }

  if(AGS_FILE_READ & file->flags){
  }else if(AGS_FILE_WRITE & file->flags){
  }else{
    g_object_unref(gobject);
    return;
  }

  xmlFreeDoc(file->doc);
  xmlCleanupParser();
  xmlMemoryDump();
}

void
ags_file_write(AgsFile *file)
{
  xmlDtd *dtd;

  LIBXML_TEST_VERSION;

  file->doc = xmlNewDoc(BAD_CAST "1.0\0");
  file->current = xmlNewNode(NULL, BAD_CAST "ags\0");
  xmlDocSetRootElement(file->doc, file->current);

  dtd = xmlCreateIntSubset(file->doc, BAD_CAST ("ags\0"), NULL, BAD_CAST file->dtd);
  xmlNewProp(file->current, BAD_CAST "version\0", BAD_CAST AGS_VERSION);

  ags_file_write_window(file);

  xmlSaveFormatFileEnc(file->name, file->doc, file->encoding, 1);
}

void
ags_file_read(AgsFile *file)
{
  xmlChar *string;

  LIBXML_TEST_VERSION;

  file->doc = xmlReadFile(file->name, NULL, 0);

  if(file->doc == NULL){
    printf("couldn't parse file: %s\n\0", file->name);
    return;
  }

  file->current = xmlDocGetRootElement(file->doc);

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE && !xmlStrncmp("ags\0", file->current->name, 3))
      break;

    file->current = file->current->next;
  }

  if(file->current != NULL)
    string = xmlGetProp(file->current, "version\0");
  else{
    g_printf("invalid format by opening file: %s\nexiting\n\0", file->name);
    return;
  }

  if(string != NULL){
    if(!xmlStrncmp(string, AGS_VERSION, 8))
      g_printf("version doesn't match\nfile version: %s\nprogram version %s\n", string, AGS_VERSION);
  }else
    g_printf("no version attribute in file: %s\n\0", file->name);

  file->current = file->current->children;
  ags_file_read_window(file);
}

void
ags_file_read_audio(AgsFile *file, AgsAudio *audio)
{
  xmlNode *node, *channel_children;
  xmlChar *prop[2];
  GType type;

  prop[0] = xmlGetProp(file->current, "audio_channels\0");

  if(prop[0] != NULL)
    ags_audio_set_audio_channels(audio, atoi(prop[0]));

  prop[0] = xmlGetProp(file->current, "output_pads\0");

  if(prop[0] != NULL)
    ags_audio_set_pads(audio, AGS_TYPE_OUTPUT, atoi(prop[0]));

  prop[0] = xmlGetProp(file->current, "input_pads\0");

  if(prop[0] != NULL)
    ags_audio_set_pads(audio, AGS_TYPE_INPUT, atoi(prop[0]));

  node = file->current;
  file->current = node->children;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE)
      if(!xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_CHANNEL), 10)){
	channel_children = file->current->children;

	while(channel_children != NULL){
	  if(channel_children->type == XML_ELEMENT_NODE){
	    if(!xmlStrncmp(channel_children->name, "AgsInput\0", 8)){
	      type = AGS_TYPE_INPUT;
	      break;
	    }else if(!xmlStrncmp(channel_children->name, "AgsOutput\0", 9)){
	      type = AGS_TYPE_OUTPUT;
	      break;
	    }
	  }

	  channel_children = channel_children->next;
	}

	if(channel_children == NULL)
	  continue;

	prop[0] = xmlGetProp(channel_children, "pad\0");
	prop[1] = xmlGetProp(channel_children, "audio_channel\0");

	if(prop[0] == NULL || prop[1] == NULL)
	  continue;

	ags_file_read_channel(file, ags_channel_nth(((type == AGS_TYPE_OUTPUT) ? audio->output: audio->input),
						    atoi(prop[0]) * audio->audio_channels + atoi(prop[1])),
			      type);
      }

    file->current = file->current->next;
  }
}

void
ags_file_read_channel(AgsFile *file, AgsChannel *channel, GType type)
{
  xmlNode *node0;
  xmlChar *prop[2];
  GList *recall, *pattern;
  gboolean recycling, first_recall, first_pattern, notation, output, input;

  if(channel == NULL)
    return;

  node0 = file->current;
  first_recall =
    first_pattern = TRUE;
  recycling =
    notation =
    output =
    input = FALSE;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE){
      node0 = file->current;
      file->current = file->current->children;

      if(type == AGS_TYPE_INPUT && !input && !xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_INPUT), 8)){
	input = TRUE;
	ags_file_read_input(file, (AgsInput *) channel);
      }else if(type == AGS_TYPE_OUTPUT && !output && !xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_OUTPUT), 9)){
	output = TRUE;
	ags_file_read_output(file, (AgsOutput *) channel);
      }else if(!xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_RECALL), 9)){
	if(first_recall){
	  recall =
	    channel->recall = g_list_alloc();
	  first_recall = FALSE;
	}else{
	  recall->next = g_list_alloc();
	  recall->next->prev = recall;
	  recall = recall->next;
	}

	recall->data = (gpointer) ags_recall_new();
	ags_file_read_recall(file, (AgsRecall *) recall->data);
      }else if(!xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_PATTERN), 10)){
	if(first_pattern){
	  pattern =
	    channel->pattern = g_list_alloc();
	}else{
	  pattern->next = g_list_alloc();
	  pattern->next->prev = pattern;
	  pattern = pattern->next;
	}
	
	pattern->data = (gpointer) ags_pattern_new();
	ags_file_read_pattern(file, (AgsPattern *) pattern->data);
      }

      file->current = node0;
    }else if(!notation && !xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_NOTATION), 11)){
      notation = TRUE;
    }else if(!recycling && !xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_RECYCLING), 12)){
      recycling = TRUE;
    }

    file->current = file->current->next;
  }
}

void
ags_file_read_output(AgsFile *file, AgsOutput *output)
{
}

void
ags_file_read_input(AgsFile *file, AgsInput *input)
{
  xmlChar *prop;

  prop = xmlGetProp(file->current, "file\0");

  if(prop != NULL)
    ags_input_set_file(input, prop);
}

void
ags_file_read_recycling(AgsFile *file, AgsRecycling *recycling)
{
}

void
ags_file_read_audio_signal(AgsFile *file, AgsAudioSignal *audio_signal)
{
}

void
ags_file_read_stream(AgsFile *file, gpointer stream)
{
}

void
ags_file_read_stream_lock(AgsFile *file, gpointer stream_lock)
{
}

void
ags_file_read_recall(AgsFile *file, AgsRecall *recall)
{
  xmlChar *prop;

  recall->effect = xmlGetProp(file->current, "effect\0");
  recall->name = xmlGetProp(file->current, "name\0");

  prop = xmlGetProp(file->current, "control_count\0");

  if(prop != NULL){
    recall->control_count = atoi(prop);
    recall->control = (gpointer *) malloc(recall->control_count * sizeof(gpointer));
  }
}

void
ags_file_read_pattern(AgsFile *file, AgsPattern *pattern)
{
  xmlNode *node;
  xmlChar *prop[3];
  xmlChar *content;
  gboolean human_readable;
  guint i, j, k;

  prop[0] = xmlGetProp(file->current, "offset_min\0");
  pattern->offset_min = atoi(prop[0]);

  prop[0] = xmlGetProp(file->current, "offset_sec\0");
  pattern->offset_sec = g_strtod(prop[0], NULL);

  prop[0] = xmlGetProp(file->current, "dim0\0");
  prop[1] = xmlGetProp(file->current, "dim1\0");
  prop[2] = xmlGetProp(file->current, "lenght\0");

  ags_pattern_set_dim(pattern, atoi(prop[0]), atoi(prop[1]), atoi(prop[2]));

  node = file->current->children;

  while(node != NULL){
    if(node->type == XML_ELEMENT_NODE && !xmlStrncmp(node->name, "AgsPatternData\0", 14)){
      prop[0] = xmlGetProp(node, "coding\0");

      if(prop != NULL){
	human_readable = (gboolean) !xmlStrncmp(prop[0], "human readable\0", 14);
      }else
	human_readable = TRUE;

      i = atoi(xmlGetProp(node, "index0\0"));
      j = atoi(xmlGetProp(node, "index1\0"));

      content = xmlNodeGetContent(node);

      if(human_readable){
	for(k = 0; k < pattern->dim[2]; k++){
	  if(content[k] == '1')
	    ags_pattern_toggle_bit(pattern, i, j, k);
	}
      }else{
      }
    }

    node = node->next;
  }
}

void
ags_file_read_devout(AgsFile *file, AgsDevout *devout)
{
  xmlChar *prop;

  prop = xmlGetProp(file->current, "audio_channels\0");

  if(prop != NULL)
    devout->audio_channels = atoi(prop);
  else
    devout->audio_channels = 2;

  prop = xmlGetProp(file->current, "buffer_size\0");

  if(prop != NULL)
    devout->buffer_size = atoi(prop);
  else
    devout->buffer_size = 512;

  prop = xmlGetProp(file->current, "method\0");

  if(prop != NULL)
    devout->method = prop;
  else
    devout->method = "oss\0";

  prop = xmlGetProp(file->current, "device\0");

  if(prop != NULL)
    devout->device = prop;
  else
    devout->device = "/dev/dsp\0";
}

void
ags_file_read_window(AgsFile *file)
{
  xmlNode *node0, *node1;
  GList *list;
  gboolean menubar, machines, editor, navigation;

  menubar =
    machines =
    editor =
    navigation = FALSE;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE && !xmlStrncmp(g_type_name(AGS_TYPE_WINDOW), file->current->name, 9))
      break;

    file->current = file->current->next;
  }

  if(file->current == NULL)
    return;

  file->window = (GtkWidget *) ags_window_new();

  list = gtk_container_get_children((GtkContainer *) file->window);
  file->current = file->current->children;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE){
      node0 = file->current;

      if(!menubar && !xmlStrncmp(g_type_name(AGS_TYPE_MENU_BAR), file->current->name, 10)){
	menubar = TRUE;
	ags_file_read_menu_bar(file, (AgsMenuBar *) list->data);
      }else if(!machines && !xmlStrncmp("AgsMachines\0", file->current->name, 11)){
	machines = TRUE;
	file->current = file->current->children;

	while(file->current != NULL){
	  if(file->current->type == XML_ELEMENT_NODE && !xmlStrncmp(g_type_name(AGS_TYPE_MACHINE), file->current->name, 10))
	    ags_file_read_machine(file);

	  file->current = file->current->next;
	}
      }else if(!editor && !xmlStrncmp(g_type_name(AGS_TYPE_EDITOR), file->current->name, 9)){
	editor = TRUE;
	ags_file_read_editor(file, (AgsEditor *) ((GList *) list->next->data)->next->data);
      }else if(!navigation && !xmlStrncmp(g_type_name(AGS_TYPE_NAVIGATION), file->current->name, 13)){
	navigation = TRUE;
	ags_file_read_navigation(file, (AgsNavigation *) g_list_nth(list, 2)->data);
      }else
	g_printf("ignoring unrecognized element: %s\n\0", file->current->name);

      file->current = node0;
    }

    file->current = file->current->next;
  }
}

void
ags_file_read_menu_bar(AgsFile *file, AgsMenuBar *menu_bar)
{
}

void
ags_file_read_machine(AgsFile *file)
{
  AgsMachine *machine;
  xmlNode *node0, *node1, *node2;
  gboolean audio, instance;

  machine = NULL;
  node0 = file->current;
  file->current = file->current->children;
  audio =
    instance = FALSE;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE)
      if(!audio && !xmlStrncmp(g_type_name(AGS_TYPE_AUDIO), file->current->name, 8)){
	audio = TRUE;
	node1 = file->current;
      }else if(!instance){
	if(!xmlStrncmp(g_type_name(AGS_TYPE_PANEL), file->current->name, 8)){
	  machine = (AgsMachine *) g_object_new(AGS_TYPE_PANEL, NULL);
	  AGS_MACHINE_GET_CLASS(machine)->read_file = ags_file_read_panel;
	}else if(!xmlStrncmp(g_type_name(AGS_TYPE_MIXER), file->current->name, 8)){
	  machine = (AgsMachine *) g_object_new(AGS_TYPE_MIXER, NULL);
	  AGS_MACHINE_GET_CLASS(machine)->read_file = ags_file_read_mixer;
	}else if(!xmlStrncmp(g_type_name(AGS_TYPE_DRUM), file->current->name, 7)){
	  machine = (AgsMachine *) g_object_new(AGS_TYPE_DRUM, NULL);
	  AGS_MACHINE_GET_CLASS(machine)->read_file = ags_file_read_drum;
	}else if(!xmlStrncmp(g_type_name(AGS_TYPE_MATRIX), file->current->name, 9)){
	  machine = (AgsMachine *) g_object_new(AGS_TYPE_MATRIX, NULL);
	  AGS_MACHINE_GET_CLASS(machine)->read_file = ags_file_read_matrix;
	}else if(!xmlStrncmp(g_type_name(AGS_TYPE_SYNTH), file->current->name, 8)){
	  machine = (AgsMachine *) g_object_new(AGS_TYPE_SYNTH, NULL);
	  AGS_MACHINE_GET_CLASS(machine)->read_file = ags_file_read_synth;
	}else if(!xmlStrncmp(g_type_name(AGS_TYPE_FFPLAYER), file->current->name, 11)){
	  machine = (AgsMachine *) g_object_new(AGS_TYPE_FFPLAYER, NULL);
	  AGS_MACHINE_GET_CLASS(machine)->read_file = ags_file_read_ffplayer;
	}else{
	  file->current = file->current->next;
	  continue;
	}

	node2 = file->current;
	instance = TRUE;
      }else if(audio)
	break;

    file->current = file->current->next;
  }

  file->current = node2;

  if(!instance)
    machine = (AgsMachine *) g_object_new(AGS_TYPE_MACHINE, NULL);

  machine->name = xmlGetProp(node0, "name\0");

  if(instance)
    AGS_MACHINE_GET_CLASS(machine)->read_file(file, machine);

  file->current = node1;
  ags_file_read_audio(file, machine->audio);
}

void
ags_file_read_pad(AgsFile *file, AgsPad *pad)
{
  xmlNode *node;
  GList *list0;
  xmlChar *prop;

  node = file->current;

  prop = xmlGetProp(node, "show_grouping\0");

  if(prop != NULL)
    if(!xmlStrncmp(prop, "TRUE\0", 4))
      gtk_widget_show((GtkWidget *) pad->hbox);

  prop = xmlGetProp(node, "groupe_pad\0");

  if(prop != NULL){
    if(!xmlStrncmp(prop, "TRUE\0", 4)){
    }
  }

  prop = xmlGetProp(node, "groupe_channel\0");

  if(prop != NULL)
    if(!xmlStrncmp(prop, "TRUE\0", 4)){
    }

  list0 = gtk_container_get_children((GtkContainer *) pad->option->menu);
  file->current = file->current->children;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE && !xmlStrncmp(file->current->name, g_type_name(AGS_TYPE_LINE), 7)){
      ags_file_read_line(file, (AgsLine *) list0->data);
      list0 = list0->next;
    }

    file->current = file->current->next;
  }
}

void
ags_file_read_line(AgsFile *file, AgsLine *line)
{
  xmlChar *prop[2];
  guint rows, cols;
  void ags_file_read_line_member(){
    GtkWidget *widget;
    AgsLineMember *line_member;
    xmlChar *prop[5];
    gboolean all_positions = FALSE;
    gboolean connect = TRUE;

    prop[0] = xmlGetProp(file->current, "visualization\0");

    if(prop[0] == NULL){
      g_printf("AgsLineMember has no visualization\n\0");
      return;
    }

    if(!xmlStrncmp(g_type_name(GTK_TYPE_SCALE), prop[0], 8))
      widget = g_object_new(GTK_TYPE_VSCALE, NULL);
    else if(!xmlStrncmp(g_type_name(GTK_TYPE_SPIN_BUTTON), prop[0], 13))
      widget = g_object_new(GTK_TYPE_SPIN_BUTTON, NULL);
    //    else if(!xmlStrncmp(g_type_name(GTK_TYPE_DIAL), prop))
    //      widget = g_object_new(GTK_TYPE_DIAL);
    else
      return;

    prop[0] = xmlGetProp(file->current, "left_attach\0");
    prop[1] = xmlGetProp(file->current, "right_attach\0");
    prop[2] = xmlGetProp(file->current, "top_attach\0");
    prop[3] = xmlGetProp(file->current, "bottom_attach\0");

    if(prop[0] == NULL || prop[1] == NULL || prop[2] == NULL || prop[3] == NULL)
      gtk_container_add((GtkContainer *) line->table, widget);
    else
      gtk_table_attach(line->table, widget,
		       atoi(prop[0]), atoi(prop[1]),
		       atoi(prop[2]), atoi(prop[3]),
		       GTK_FILL, GTK_FILL,
		       2, 2);

    prop[0] = xmlGetProp(file->current, "effect\0");
    prop[1] = xmlGetProp(file->current, "recall_name\0");
    prop[2] = xmlGetProp(file->current, "control\0");

    if(prop[0] == NULL || prop[1] == NULL || prop[2] == NULL)
      return;

    line_member->recall = ags_channel_find_recall(line->channel, prop[0], prop[1]);
    line_member->recall->control[atoi(prop[2])] = &(GTK_RANGE(widget)->adjustment->value);
  }

  prop[0] = xmlGetProp(file->current, "rows\0");
  prop[1] = xmlGetProp(file->current, "cols\0");

  if(prop[0] != NULL && prop[1]){
    rows = g_strtod(prop[0], NULL);
    cols = g_strtod(prop[1], NULL);
    gtk_table_resize(line->table, rows, cols);
  }

  file->current = file->current->children;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE && !xmlStrncmp(file->current->name, "AgsLineMember\0", 13))
      ags_file_read_line_member();

    file->current = file->current->next;
  }
}

void
ags_file_read_editor(AgsFile *file, AgsEditor *editor)
{
  xmlNode *node;
  GList *list0, *list1;

  node = file->current;
  list0 = gtk_container_get_children((GtkContainer *) editor);

  file->current = file->current->children;

  while(file->current != NULL){
    if(file->current->type == XML_ELEMENT_NODE){
      if(!xmlStrncmp(g_type_name(AGS_TYPE_TOOLBAR), file->current->name, 10)){
	ags_file_read_toolbar(file, (AgsToolbar *) list0->data);
      }//else if(!xmlStrncmp(g_type_name(AGS_TYPE_NOTEBOOK), file->current->name, 11)){
      //      }
    }

    file->current = file->current->next;
  }
}

void
ags_file_read_toolbar(AgsFile *file, AgsToolbar *toolbar)
{
  GList *list0, *list1;
  xmlChar *prop;
  guint i;

  prop = xmlGetProp(file->current, "mode\0");

  list0 = gtk_container_get_children((GtkContainer *) toolbar);

  if(prop != NULL){
    if(!xmlStrncmp(prop, "edit\0", 4)){
      gtk_toggle_button_set_active((GtkToggleButton *) list0->data, TRUE);
      list0 = g_list_nth(list0, 3);
    }else if(!xmlStrncmp(prop, "clear\0", 5)){
      list0 = list0->next;
      gtk_toggle_button_set_active((GtkToggleButton *) list0->data, TRUE);
      list0 = list0->next->next;
    }else if(!xmlStrncmp(prop, "select\0", 6)){
      list0 = list0->next->next;
      gtk_toggle_button_set_active((GtkToggleButton *) list0->data, TRUE);
      list0 = list0->next;
    }
  }

  list0 = g_list_nth(list0, 3);

  prop = xmlGetProp(file->current, "tic\0");
  list0 = list0->next;

  if(prop != NULL){
    list1 = gtk_container_get_children((GtkContainer *) GTK_OPTION_MENU(list0->data)->menu);

    for(i = 0; i < 9; i++){
      if(!xmlStrncmp(prop, gtk_label_get_text((GtkLabel *) GTK_BIN(list1->data)->child), 4))
	break;

      list1 = list1->next;
    }

    gtk_option_menu_set_history((GtkOptionMenu *) list0->data, i);
  }

  prop = xmlGetProp(file->current, "zoom\0");
  list0 = list0->next->next;

  if(prop != NULL){
    list1 = gtk_container_get_children((GtkContainer *) GTK_OPTION_MENU(list0->data)->menu);

    for(i = 0; i < 9; i++){
      if(!xmlStrncmp(prop, gtk_label_get_text((GtkLabel *) GTK_BIN(list1->data)->child), 4))
	break;

      list1 = list1->next;
    }

    gtk_option_menu_set_history((GtkOptionMenu *) list0->data, i);
  }
}

void
ags_file_read_notebook(AgsFile *file, AgsNotebook *notebook)
{
}

void
ags_file_read_navigation(AgsFile *file, AgsNavigation *navigation)
{
  GList *list0, *list1, *list2;
  xmlChar *prop;

  list0 = gtk_container_get_children((GtkContainer *) navigation);
  list1 = gtk_container_get_children((GtkContainer *) list0->next->data);
  list0 = gtk_container_get_children((GtkContainer *) list0->data);

  prop = xmlGetProp(file->current, "expandet\0");

  if(prop != NULL)
    if(!xmlStrncmp(prop, "TRUE\0", 4)){
      list2 = gtk_container_get_children((GtkContainer *) list0->data);
      gtk_arrow_set((GtkArrow *) list2->data, GTK_ARROW_DOWN, GTK_SHADOW_NONE);
    }

  list0 = list0->next->next;
  prop = xmlGetProp(file->current, "bpm\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list0->data)->adjustment, g_strtod(prop, NULL));

  list0 = list0->next;
  prop = xmlGetProp(file->current, "loop\0");

  if(prop != NULL)
    if(!xmlStrncmp(prop, "TRUE\0", 4))
      gtk_toggle_button_set_active((GtkToggleButton *) list0->data, TRUE);

  list0 = list0->next->next;
  prop = xmlGetProp(file->current, "position_min\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list0->data)->adjustment, g_strtod(prop, NULL));

  list0 = list0->next;
  prop = xmlGetProp(file->current, "position_sec\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list0->data)->adjustment, g_strtod(prop, NULL));

  list0 = list0->next->next;
  prop = xmlGetProp(file->current, "duration_min\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list0->data)->adjustment, g_strtod(prop, NULL));

  list0 = list0->next;
  prop = xmlGetProp(file->current, "duration_sec\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list0->data)->adjustment, g_strtod(prop, NULL));


  list1 = list1->next;
  prop = xmlGetProp(file->current, "loop_left_min\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list1->data)->adjustment, g_strtod(prop, NULL));

  list1 = list1->next;
  prop = xmlGetProp(file->current, "loop_left_sec\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list1->data)->adjustment, g_strtod(prop, NULL));

  list1 = list1->next->next;
  prop = xmlGetProp(file->current, "loop_right_min\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list1->data)->adjustment, g_strtod(prop, NULL));

  list1 = list1->next;
  prop = xmlGetProp(file->current, "loop_right_sec\0");

  if(prop != NULL)
    gtk_adjustment_set_value(GTK_RANGE(list1->data)->adjustment, g_strtod(prop, NULL));
}

void
ags_file_read_machine_editor(AgsFile *file, AgsMachineEditor *machine_editor)
{
}

void
ags_file_read_pad_editor(AgsFile *file, AgsPadEditor *pad_editor)
{
}

void
ags_file_read_line_editor(AgsFile *file, AgsLineEditor *line_editor)
{
}

void
ags_file_read_link_editor(AgsFile *file, AgsLinkEditor *link_editor)
{
}

void
ags_file_read_line_member_editor(AgsFile *file, AgsLineMemberEditor *line_member_editor)
{
}

void
ags_file_read_panel(AgsFile *file, AgsMachine *machine)
{
}

void
ags_file_read_mixer(AgsFile *file, AgsMachine *machine)
{
}

void
ags_file_read_drum(AgsFile *file, AgsMachine *machine)
{
}

void
ags_file_read_matrix(AgsFile *file, AgsMachine *machine)
{
}

void
ags_file_read_synth(AgsFile *file, AgsMachine *machine)
{
}

void
ags_file_read_oscillator(AgsFile *file, AgsOscillator *oscillator)
{
}

void
ags_file_read_ffplayer(AgsFile *file, AgsMachine *machine)
{
}

void
ags_file_write_audio(AgsFile *file, AgsAudio *audio)
{
  xmlNode *node;
  AgsChannel *channel;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_AUDIO), NULL);

  xmlNewProp(node, BAD_CAST "audio_channels\0", BAD_CAST g_strdup_printf("%d\0", audio->audio_channels));
  xmlNewProp(node, BAD_CAST "output_pads\0", BAD_CAST g_strdup_printf("%d\0", audio->output_pads));
  xmlNewProp(node, BAD_CAST "input_pads\0", BAD_CAST g_strdup_printf("%d\0", audio->input_pads));

  channel = audio->output;

  while(channel != NULL){
    AGS_CHANNEL_GET_CLASS(channel)->write_file(file, channel);
    file->current = node;

    channel = channel->next;
  }

  channel = audio->input;

  while(channel != NULL){
    AGS_CHANNEL_GET_CLASS(channel)->write_file(file, channel);
    file->current = node;

    channel = channel->next;
  }
}

void
ags_file_write_channel(AgsFile *file, AgsChannel *channel)
{
  xmlNode *node;
  GList *list;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_CHANNEL), NULL);

  xmlNewProp(node, BAD_CAST "pad\0", g_strdup_printf("%d\0", channel->pad));
  xmlNewProp(node, BAD_CAST "audio_channel\0", g_strdup_printf("%d\0", channel->audio_channel));
  xmlNewProp(node, BAD_CAST "link_name\0", ((channel->link != NULL) ? AGS_MACHINE(AGS_AUDIO(channel->link->audio)->machine)->name: "NULL\0"));
  xmlNewProp(node, BAD_CAST "link_line\0", g_strdup_printf("%d\0", ((channel->link != NULL) ? channel->link->line: -1)));

  /*
  if(G_TYPE_CHECK_INSTANCE_TYPE(channel, AGS_TYPE_OUTPUT) && (AGS_AUDIO_OUTPUT_HAS_RECYCLING & channel->audio->flags))
    ags_file_write_recycling(channel->first_recycling);
  else if(G_TYPE_CHECK_INSTANCE_TYPE(channel, AGS_TYPE_INPUT) && (AGS_AUDIO_INPUT_HAS_RECYCLING & channel->audio->flags))
    ags_file_write_recycling(channel->first_recycling);
  */

  list = channel->recall;

  while(list != NULL){
    ags_file_write_recall(file, (AgsRecall *) list->data);
    file->current = node;
  }

  list = channel->pattern;

  if(list != NULL){
    ags_file_write_pattern(file, (AgsPattern *) list->data);
    file->current = node;
  }

  if(channel->notation != NULL)
    ags_file_write_notation(file, channel->notation);
}

void
ags_file_write_output(AgsFile *file, AgsChannel *channel)
{
  ags_file_write_channel(file, channel);

  file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_OUTPUT), NULL);
}

void
ags_file_write_input(AgsFile *file, AgsChannel *channel)
{
  AgsInput *input = (AgsInput *) channel;

  ags_file_write_channel(file, channel);

  file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_INPUT), NULL);

  if(input->file != NULL)
    xmlNewProp(file->current, BAD_CAST "file\0", input->file);
}

void
ags_file_write_recycling(AgsFile *file, AgsRecycling *recycling)
{
  xmlNode *node;
  GList *list;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_RECYCLING), NULL);

  /*
  xmlNewProp(node, NULL, BAD_CAST "toplevel\0", BAD_CAST ((AGS_RECYCLING_TOPLEVEL & recycling->flags) ? "TRUE\0": "FALSE\0"));
  xmlNewProp(node, NULL, BAD_CAST "owns_signal\0", BAD_CAST ((AGS_RECYCLING_OWNS_SIGNAL & recycling->flags) ? "TRUE\0": "FALSE\0"));
  xmlNewProp(node, NULL, BAD_CAST "current_recall\0", BAD_CAST g_strdup_printf("%d\0",
									       g_list_position(recycling->current)));


  list = recycling->audio_signal;

  while(list != NULL){
    ags_file_write_audio_signal(file, (AgsAudioSignal *) list->data);
    file->current = node;

    list = list->next;
  }
  */
}

void
ags_file_write_audio_signal(AgsFile *file, AgsAudioSignal *audio_signal)
{
  xmlNode *node;
  GList *list;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_AUDIO_SIGNAL), NULL);

  xmlNewProp(node, BAD_CAST "offset_min\0", g_strdup_printf("%d\0", audio_signal->offset_min));
  xmlNewProp(node, BAD_CAST "offset_sec\0", g_strdup_printf("%f\0", audio_signal->offset_sec));
  xmlNewProp(node, BAD_CAST "duration_min\0", g_strdup_printf("%d\0", audio_signal->duration_min));
  xmlNewProp(node, BAD_CAST "duration_sec\0", g_strdup_printf("%f\0", audio_signal->duration_sec));
  xmlNewProp(node, BAD_CAST "attack\0", g_strdup_printf("%d\0", audio_signal->attack));
  xmlNewProp(node, BAD_CAST "lock_attack\0", g_strdup_printf("%d\0", audio_signal->lock_attack));


  list = audio_signal->stream;

  while(list != NULL){
    ags_file_write_stream(file, list->data);
    file->current = node;

    list = list->next;
  }

  list = audio_signal->lock;

  while(list != NULL){
    ags_file_write_stream_lock(file, list->data);
    file->current = node;

    list = list->next;
  }
}

void
ags_file_write_stream(AgsFile *file, gpointer stream)
{
  file->current = xmlNewChild(file->current, NULL, BAD_CAST "AgsStream\0", NULL);
}

void
ags_file_write_stream_lock(AgsFile *file, gpointer lock)
{
  file->current = xmlNewChild(file->current, NULL, BAD_CAST "AgsStreamLock\0", NULL);
}

void
ags_file_write_recall(AgsFile *file, AgsRecall *recall)
{
  xmlChar *content, *old;
  guint i;

  if(!xmlStrncmp(recall->effect, AGS_EFFECT_COPY_STREAM, 11) || !xmlStrncmp(recall->effect, AGS_EFFECT_COPY_BUFFER, 11) || !xmlStrncmp(recall->effect, AGS_EFFECT_DOWN_MIX, 8))
    return;

  file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_RECALL), NULL);

  xmlNewProp(file->current, BAD_CAST "effect\0", BAD_CAST recall->effect);
  xmlNewProp(file->current, BAD_CAST "name\0", BAD_CAST recall->name);
  xmlNewProp(file->current, BAD_CAST "control_count\0", BAD_CAST g_strdup_printf("%d\0", recall->control_count));

  if(recall->control_count > 0){
    content = g_strdup_printf("%f\0", recall->control[0]);
  }else
    return;

  for(i = 1; i < recall->control_count; i++){
    old = content;
    content = g_strdup_printf("%s %f\0", content, recall->control[i]);
    free(old);
  }

  xmlNodeAddContent(file->current, content);
}

void
ags_file_write_pattern(AgsFile *file, AgsPattern *pattern)
{
  xmlNode *node;
  xmlChar *content;
  uint i, j, k;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_PATTERN), NULL);

  xmlNewProp(node, BAD_CAST "offset_min\0", BAD_CAST g_strdup_printf("%d\0", pattern->offset_min));
  xmlNewProp(node, BAD_CAST "offset_sec\0", BAD_CAST g_strdup_printf("%f\0", pattern->offset_sec));
  xmlNewProp(node, BAD_CAST "dim0\0", BAD_CAST g_strdup_printf("%d\0", pattern->dim[0]));
  xmlNewProp(node, BAD_CAST "dim1\0", BAD_CAST g_strdup_printf("%d\0", pattern->dim[1]));
  xmlNewProp(node, BAD_CAST "lenght\0", BAD_CAST g_strdup_printf("%d\0", pattern->dim[2]));

  for(i = 0; i < pattern->dim[0]; i++)
    for(j = 0; j < pattern->dim[1]; j++){
      file->current = xmlNewChild(node, NULL, BAD_CAST "AgsPatternData\0", NULL);
	
      xmlNewProp(file->current, BAD_CAST "coding\0", BAD_CAST "human_readable\0");
      xmlNewProp(file->current, BAD_CAST "index0\0", BAD_CAST g_strdup_printf("%d\0", i));
      xmlNewProp(file->current, BAD_CAST "index1\0", BAD_CAST g_strdup_printf("%d\0", j));

      content = (char *) malloc((pattern->dim[2] + 1) * sizeof(xmlChar));
      content[pattern->dim[2]] = '\0';

      for(k = 0; k < pattern->dim[2]; k++)
	content[i] = (ags_pattern_get_bit(pattern, i, j, k)) ? '1': '0';

      xmlNodeAddContent(file->current, content);
    }
}

void
ags_file_write_notation(AgsFile *file, AgsNotation *notation)
{
  xmlNode *node;
  GList *list0, *list1;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_NOTATION), NULL);

  xmlNewProp(node, BAD_CAST "raster\0", BAD_CAST ((AGS_NOTATION_RASTER & notation->flags) ? "TRUE\0": "FALSE\0"));

  list0 = notation->note;

  while(list0 != NULL){
    list1 = (GList *) list0->data;

    while(list1 != NULL){
      ags_file_write_note(file, (AgsNote *) list1->data);
      file->current = node;

      list1 = list1->next;
    }

    list0 = list0->next;
  }
}

void
ags_file_write_note(AgsFile *file, AgsNote *note)
{
  file->current = xmlNewChild(file->current, NULL, BAD_CAST "AgsNote\0", NULL);

  xmlNewProp(file->current, BAD_CAST "frequence\0", BAD_CAST g_strdup_printf("%d\0", note->frequence));
  xmlNewProp(file->current, BAD_CAST "offset_min\0", BAD_CAST g_strdup_printf("%d\0", note->offset_min));
  xmlNewProp(file->current, BAD_CAST "offset_sec\0", BAD_CAST g_strdup_printf("%f\0", note->offset_sec));
  xmlNewProp(file->current, BAD_CAST "duration_min\0", BAD_CAST g_strdup_printf("%d\0", note->duration_min));
  xmlNewProp(file->current, BAD_CAST "duration_sec\0", BAD_CAST g_strdup_printf("%f\0", note->duration_sec));
}

void
ags_file_write_devout(AgsFile *file, AgsDevout *devout)
{
  file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_DEVOUT), NULL);

  xmlNewProp(file->current, BAD_CAST "audio_channels\0", BAD_CAST g_strdup_printf("%d\0", devout->audio_channels));
  xmlNewProp(file->current, BAD_CAST "buffer_size\0", BAD_CAST g_strdup_printf("%d\0", devout->buffer_size));
  xmlNewProp(file->current, BAD_CAST "method\0", BAD_CAST devout->method);
  xmlNewProp(file->current, BAD_CAST "device\0", BAD_CAST devout->device);
}

void
ags_file_write_window(AgsFile *file)
{
  xmlNode *node0, *node1;
  GList *list0, *list1, *list2;

  node0 = 
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_WINDOW), NULL);

  list0 = gtk_container_get_children((GtkContainer *) file->window);
  ags_file_write_menu_bar(file, list0->data);
  list0 = list0->next;

  node1 =
    file->current = xmlNewChild(node0, NULL, BAD_CAST "AgsMachines\0", NULL);
  list2 = gtk_container_get_children((GtkContainer *) AGS_WINDOW(file->window)->machines);

  while(list2 != NULL){
    AGS_MACHINE_GET_CLASS((AgsMachine *) list2->data)->write_file(file, (AgsMachine *) list2->data);
    file->current = node1;

    list2 = list2->next;
  }

  file->current = node0;

  list1 = gtk_container_get_children((GtkContainer *) list0->data);
  list1 = list1->next;
  ags_file_write_editor(file, (AgsEditor *) list1->data);

  list0 = list0->next;

  ags_file_write_navigation(file, (AgsNavigation *) list0->data);
}

void
ags_file_write_menu_bar(AgsFile *file, AgsMenuBar *menu_bar)
{
  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_MENU_BAR), NULL);
}

void
ags_file_write_machine(AgsFile *file, AgsMachine *machine)
{
  xmlNode *node;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_MACHINE), NULL);
  xmlNewProp(node, BAD_CAST "name\0", machine->name);

  ags_file_write_audio(file, (AgsAudio *) machine->audio);
  file->current = node;
}

void
ags_file_write_pad(AgsFile *file, AgsPad *pad)
{
  xmlNode *node;
  GList *list;

  node =
    file->current = xmlNewChild(file->current, NULL, g_type_name(AGS_TYPE_PAD), NULL);

  xmlNewProp(file->current, BAD_CAST "show_grouping\0", BAD_CAST (GTK_WIDGET_VISIBLE(pad->hbox) ? "TRUE\0": "FALSE\0"));

  list = gtk_container_get_children((GtkContainer *) pad->hbox);
  xmlNewProp(file->current, BAD_CAST "groupe_pad\0", BAD_CAST (GTK_TOGGLE_BUTTON(list->data)->active ? "TRUE\0": "FALSE\0"));

  list = list->next;
  xmlNewProp(file->current, BAD_CAST "groupe_pad\0", BAD_CAST (GTK_TOGGLE_BUTTON(list->data)->active ? "TRUE\0": "FALSE\0"));

  list = gtk_container_get_children((GtkContainer *) pad->option->menu);

  while(list != NULL){
    ags_file_write_line(file, (AgsLine *) list->data);
    file->current = node;

    list = list->next;
  }
}

void
ags_file_write_line(AgsFile *file, AgsLine *line)
{
  xmlNode *node;
  GList *list;
  AgsLineMember *line_member;
  char *visualization;

  file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_LINE), NULL);

  xmlNewProp(file->current, BAD_CAST "rows\0", g_strdup_printf("%d\0", line->table->nrows));
  xmlNewProp(file->current, BAD_CAST "cols\0", g_strdup_printf("%d\0", line->table->ncols));

  list = line->table->children;

  while(list != NULL){
    if(GTK_IS_SCALE(list->data))
      visualization = "GtkScale\0";
    else if(GTK_IS_SPIN_BUTTON(list->data))
      visualization = "GtkSpinButton\0";
    //    else if(GTK_IS_DIAL(list->data))
    //      visualization = "GtkDial\0";
    else{
      list = list->next;
      continue;
    }

    node = xmlNewChild(file->current, NULL, BAD_CAST "AgsLineMember\0", NULL);

    line_member = (AgsLineMember *) g_object_get_data((GObject *) list->data, "AgsLineMember\0");

    xmlNewProp(node, BAD_CAST "effect\0", BAD_CAST line_member->recall->effect);
    xmlNewProp(node, BAD_CAST "recall_name\0", BAD_CAST line_member->recall->name);
    xmlNewProp(node, BAD_CAST "control\0", BAD_CAST g_strdup_printf("%d\0", line_member->control));

    xmlNewProp(node, BAD_CAST "visualization\0", BAD_CAST visualization);

    xmlNewProp(node, BAD_CAST "left_attach\0", BAD_CAST g_strdup_printf("%d\0", ((GtkTableChild*) list->data)->left_attach));
    xmlNewProp(node, BAD_CAST "right_attach\0", BAD_CAST g_strdup_printf("%d\0", ((GtkTableChild*) list->data)->right_attach));
    xmlNewProp(node, BAD_CAST "top_attach\0", BAD_CAST g_strdup_printf("%d\0", ((GtkTableChild*) list->data)->top_attach));
    xmlNewProp(node, BAD_CAST "bottom_attach\0", BAD_CAST g_strdup_printf("%d\0", ((GtkTableChild*) list->data)->bottom_attach));

    list = list->next;
  }
}

void
ags_file_write_editor(AgsFile *file, AgsEditor *editor)
{
  xmlNode *node;
  GList *list;

  node =
    file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_EDITOR), NULL);

  list = gtk_container_get_children((GtkContainer *) editor);
  ags_file_write_toolbar(file, (AgsToolbar *) list->data);

  file->current = node;
  list = gtk_container_get_children((GtkContainer *)
				    gtk_container_get_children((GtkContainer *) list->next->data
							       )->data);
  ags_file_write_notebook(file, (AgsNotebook *) list->data);
}

void
ags_file_write_navigation(AgsFile *file, AgsNavigation *navigation)
{
  GList *list0, *list1, *list2;

  list0 = gtk_container_get_children((GtkContainer *) navigation);
  list1 = gtk_container_get_children((GtkContainer *) list0->data);
  list2 = gtk_container_get_children((GtkContainer *) list1->data);

  file->current = xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_NAVIGATION), NULL);

  xmlNewProp(file->current, BAD_CAST "expandet\0", ((GTK_ARROW(list2->data)->arrow_type == GTK_ARROW_DOWN) ? "TRUE\0": "FALSE\0"));
  list1 = list1->next->next;

  xmlNewProp(file->current, BAD_CAST "bpm\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next;

  xmlNewProp(file->current, BAD_CAST "loop\0",  (GTK_TOGGLE_BUTTON(list1->data)->active ? "TRUE\0": "FALSE\0"));
  list1 = list1->next->next;

  xmlNewProp(file->current, BAD_CAST "position_min\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next->next;

  xmlNewProp(file->current, BAD_CAST "position_sec\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next;

  xmlNewProp(file->current, BAD_CAST "duration_min\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next;

  xmlNewProp(file->current, BAD_CAST "duration_sec\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next;


  list1 = gtk_container_get_children((GtkContainer *) list0->next->data);

  xmlNewProp(file->current, BAD_CAST "loop_left_min\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next->next;

  xmlNewProp(file->current, BAD_CAST "loop_left_sec\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next;

  xmlNewProp(file->current, BAD_CAST "loop_right_min\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next;

  xmlNewProp(file->current, BAD_CAST "loop_right_sec\0", BAD_CAST g_strdup_printf("%f\0", (double) GTK_RANGE(list1->data)->adjustment->value));
  list1 = list1->next;

  xmlNewProp(file->current, BAD_CAST "raster\0", (GTK_TOGGLE_BUTTON(list1->data)->active ? "TRUE\0": "FALSE\0"));
}

void
ags_file_write_machine_editor(AgsFile *file, AgsMachineEditor *machine_editor)
{
}

void
ags_file_write_pad_editor(AgsFile *file, AgsPadEditor *pad_editor)
{
}

void
ags_file_write_line_editor(AgsFile *file, AgsLineEditor *line_editor)
{
}

void
ags_file_write_link_editor(AgsFile *file, AgsLinkEditor *link_editor)
{
}

void
ags_file_write_line_member_editor(AgsFile *file, AgsLineMemberEditor *line_member_editor)
{
}

void
ags_file_write_toolbar(AgsFile *file, AgsToolbar *toolbar)
{
  GList *list0, *list1;

  list0 = gtk_container_get_children((GtkContainer *) toolbar);

  if(GTK_TOGGLE_BUTTON(list0->data)->active)
    xmlNewProp(file->current, BAD_CAST "mode\0", BAD_CAST "edit\0");
  else if(GTK_TOGGLE_BUTTON(list0->next->data)->active)
    xmlNewProp(file->current, BAD_CAST "mode\0", BAD_CAST "clear\0");
  else if(GTK_TOGGLE_BUTTON(list0->next->next->data)->active)
    xmlNewProp(file->current, BAD_CAST "mode\0", BAD_CAST "select\0");

  list0 = g_list_nth(list0, 7);
  list1 = gtk_container_get_children((GtkContainer *) GTK_OPTION_MENU(list0->data)->menu);
  list1 = g_list_nth(list1, gtk_option_menu_get_history((GtkOptionMenu *) list0->data));
  xmlNewProp(file->current, BAD_CAST "tic\0", BAD_CAST gtk_label_get_text((GtkLabel *) GTK_BIN(list1->data)->child));

  list0 = list0->next->next;
  list1 = gtk_container_get_children((GtkContainer *) GTK_OPTION_MENU(list0->data)->menu);
  list1 = g_list_nth(list1, gtk_option_menu_get_history((GtkOptionMenu *) list0->data));
  xmlNewProp(file->current, BAD_CAST "zoom\0", BAD_CAST gtk_label_get_text((GtkLabel *) GTK_BIN(list1->data)->child));
}

void
ags_file_write_notebook(AgsFile *file, AgsNotebook *notebook)
{
}

void
ags_file_write_panel(AgsFile *file, AgsMachine *machine)
{
  AgsPanel *panel;

  ags_file_write_machine(file, machine);

  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_PANEL), NULL);
}

void
ags_file_write_mixer(AgsFile *file, AgsMachine *machine)
{
  AgsMixer *mixer;

  ags_file_write_machine(file, machine);

  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_MIXER), NULL);
}

void
ags_file_write_drum(AgsFile *file, AgsMachine *machine)
{
  AgsDrum *drum;

  ags_file_write_machine(file, machine);

  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_DRUM), NULL);
}

void
ags_file_write_matrix(AgsFile *file, AgsMachine *machine)
{
  AgsMatrix *matrix;

  ags_file_write_machine(file, machine);

  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_MATRIX), NULL);
}

void
ags_file_write_synth(AgsFile *file, AgsMachine *machine)
{
  AgsSynth *synth;

  ags_file_write_machine(file, machine);

  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_SYNTH), NULL);
}

void
ags_file_write_oscillator(AgsFile *file, AgsOscillator *oscillator)
{
  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_OSCILLATOR), NULL);
}

void
ags_file_write_ffplayer(AgsFile *file, AgsMachine *machine)
{
  AgsFFPlayer *ffplayer;

  ags_file_write_machine(file, machine);

  xmlNewChild(file->current, NULL, BAD_CAST g_type_name(AGS_TYPE_FFPLAYER), NULL);
}

AgsFile*
ags_file_new()
{
  AgsFile *file;

  file = (AgsFile *) g_object_new(AGS_TYPE_FILE, NULL);

  return(file);
}
